-- RBT-TEST - ДПРД для столбцов
-- Проверка дискреционного разграничения доступа для столбцов
-- Права доступа для столбцов SELECT, INSERT, UPDATE, REFERENCES
--
\i 'support/init.sql'
\i 'support/create-test-table.sql'
--
-- Общая подготовка
\connect - postgres
--
-- Отбираем права у PUBLIC на тестовую таблицу
REVOKE ALL ON "Проверка" FROM PUBLIC;
--
-- Добавим несколько записей от лица владельца таблицы "Проверка"
INSERT INTO "Проверка" VALUES (1, 'Первая запись ' || USER);
INSERT INTO "Проверка" VALUES (2, 'Вторая запись ' || USER);
--
-- Просматриваем созданные записи
SELECT * FROM "Проверка" ORDER BY "идентификатор";
--
-- Подключение пользователя u_0_00
\connect - u_0_00
--
-- Создаем проверочную таблицу для проверки создания ограничений
CREATE TABLE "Проверка2" ("идентификатор2" INTEGER PRIMARY KEY, "ключ" INTEGER);
--
-- Изначально все действия пользователю над таблицей "Проверка" запрещены
--
-- Попытка выполнить SELECT
-- Ожидается ошибка доступа
SELECT * FROM "Проверка";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 2;
--
-- Попытка выполнить INSERT
-- Ожидается ошибка доступа
INSERT INTO "Проверка" ("идентификатор") VALUES (3);
--
-- Попытка создать ограничение ссылающееся на таблицу
-- Ожидается ошибка доступа
ALTER TABLE "Проверка2" ADD CONSTRAINT "Ссылка" FOREIGN KEY ("ключ") REFERENCES "Проверка" ("идентификатор")
   ON UPDATE NO ACTION ON DELETE NO ACTION;
--
-- Администратор postgres дает право SELECT для пользователя u_0_00 на столбец "идентификатор"
\connect - postgres
GRANT SELECT ("идентификатор") ON "Проверка" TO u_0_00;
--
-- Подключение пользователя u_0_00
\connect - u_0_00
--
-- Попытка выполнить SELECT из всей таблицы
-- Ожидается ошибка доступа
SELECT * FROM "Проверка";
--
-- Попытка выполнить SELECT из столбца "идентификатор"
-- Ожидается успех
SELECT "идентификатор" FROM "Проверка";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 2;
--
-- Попытка выполнить INSERT
-- Ожидается ошибка доступа
INSERT INTO "Проверка" ("идентификатор") VALUES (3);
--
-- Попытка создать ограничение ссылающееся на таблицу
-- Ожидается ошибка доступа
ALTER TABLE "Проверка2" ADD CONSTRAINT "Ссылка" FOREIGN KEY ("ключ") REFERENCES "Проверка" ("идентификатор")
   ON UPDATE NO ACTION ON DELETE NO ACTION;
--
-- Администратор postgres предоставляет пользователю u_0_00 право на выполнение операции INSERT
-- и отбирает право на операцию SELECT
\connect - postgres
GRANT INSERT ("идентификатор") ON "Проверка" TO u_0_00;
REVOKE SELECT ("идентификатор") ON "Проверка" FROM u_0_00;
--
-- Подключение пользователя u_0_00
\connect - u_0_00
--
-- Попытка выполнить SELECT
-- Ожидается ошибка доступа
SELECT "идентификатор" FROM "Проверка";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 2;
--
-- Попытка выполнить INSERT всей строки
-- Ожидается ошибка доступа
INSERT INTO "Проверка" VALUES (3, 'Запись пользователя u_0_00');
--
-- Попытка выполнить INSERT только в разрешенный столбец
-- Ожидается успех
INSERT INTO "Проверка" ("идентификатор") VALUES (3);
--
-- Попытка создать ограничение ссылающееся на таблицу
-- Ожидается ошибка доступа
ALTER TABLE "Проверка2" ADD CONSTRAINT "Ссылка" FOREIGN KEY ("ключ") REFERENCES "Проверка" ("идентификатор")
   ON UPDATE NO ACTION ON DELETE NO ACTION;
--
-- Переподключиться от имени администратора postgres
\connect - postgres
--
-- Убедиться, что в таблице "Проверка" появилась запись, занесенная пользователем u_0_00
SELECT * FROM "Проверка";
--
-- Администратор postgres предоставляет пользователю u_0_00 право на выполнение операции UPDATE
-- и отбирает право на операцию INSERT
GRANT UPDATE ("идентификатор") ON "Проверка" TO u_0_00;
REVOKE INSERT ("идентификатор") ON "Проверка" FROM u_0_00;
--
-- Подключение пользователя u_0_00
\connect - u_0_00
--
-- Попытка выполнить SELECT
-- Ожидается ошибка доступа
SELECT "идентификатор" FROM "Проверка";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа из-за отсутствия права SELECT
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 3;
--
-- Попытка выполнить INSERT
-- Ожидается ошибка доступа
INSERT INTO "Проверка" ("идентификатор") VALUES (4);
--
-- Попытка создать ограничение ссылающееся на таблицу
-- Ожидается ошибка доступа
ALTER TABLE "Проверка2" ADD CONSTRAINT "Ссылка" FOREIGN KEY ("ключ") REFERENCES "Проверка" ("идентификатор")
   ON UPDATE NO ACTION ON DELETE NO ACTION;
--
-- Администратор postgres предоставляет пользователю u_0_00 право на выполнение операции SELECT
\connect - postgres
GRANT SELECT ("идентификатор") ON "Проверка" TO u_0_00;
--
-- Подключение пользователя u_0_00
\connect - u_0_00
--
-- Попытка выполнить UPDATE к неразрешенному столбцу
-- Ожидается ошибка доступа
UPDATE "Проверка" SET "данные" = 'данные пользователя u_0_00' WHERE "идентификатор" = 3;
--
-- Попытка выполнить UPDATE
-- Ожидается успех, так как разрешены и SELECT и UPDATE
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 3;
--
-- Проверяем, что идентификатор у записи пользователя увеличился на единицу
\connect - postgres
SELECT * FROM "Проверка";
--
-- Администратор postgres предоставляет пользователю u_0_00 право на создание ссылки на поле "идентификатор"
-- и отбирает право на операцию UPDATE и SELECT
GRANT REFERENCES ("идентификатор") ON "Проверка" TO u_0_00;
REVOKE UPDATE ("идентификатор") ON "Проверка" FROM u_0_00;
REVOKE SELECT ("идентификатор") ON "Проверка" FROM u_0_00;
--
-- Подключение пользователя u_0_00
\connect - u_0_00
--
-- Попытка выполнить SELECT
-- Ожидается ошибка доступа
SELECT "идентификатор" FROM "Проверка";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 4;
--
-- Попытка выполнить INSERT
-- Ожидается ошибка доступа
INSERT INTO "Проверка" VALUES (5, 'Запись пользователя u_0_00');
--
-- Попытка создать ограничение ссылающееся на таблицу
-- Ожидается успех
ALTER TABLE "Проверка2" ADD CONSTRAINT "Ссылка" FOREIGN KEY ("ключ") REFERENCES "Проверка" ("идентификатор")
   ON UPDATE NO ACTION ON DELETE NO ACTION;
--
\i 'support/done.sql'
