-- RBT-TEST - ДПРД для внешних баз данных
-- Права работы с объектами FOREIGN DATA WRAPPER, FOREIGN SERVER - USAGE
-- Тест написан для основного сервера, работающем на порту 6000 и для удаленного, работающем на 6001
--
\i 'support/init.sql'
--
-- Подключаемся на удаленный кластер
\connect postgres postgres - 6001
\i 'support/init.sql'
-- Назначаем пароль на роль 'u2_0_00'
ALTER ROLE u2_0_00 PASSWORD '123456';
-- Создаём тестувую таблицу
CREATE TABLE "Таблица" ("Идентификатор" INTEGER, "Значение" TEXT);
-- Наполняем записями таблицу 'Таблица'
INSERT INTO "Таблица" VALUES (1, 'Первая запись postgres');
INSERT INTO "Таблица" VALUES (2, 'Вторая запись postgres');
INSERT INTO "Таблица" VALUES (3, 'Третья запись postgres');
-- Изымаем все права у пользователя 'u2_0_00'
REVOKE ALL ON TABLE "Таблица" FROM u2_0_00;
--
-- Подключаемся на основной кластер
\connect mtest u_0_00 - 6000
-- Выполняем попытку зарегистрировать wrapper postgresql_fdw
-- Ожидается ошибка, т.к. для выполнения команды необходимы права суперпользователя
CREATE EXTENSION postgres_fdw;
-- Выполняем попытку подключиться к внешнему серверу
-- Ожидается ошибка, т.к. wrapper postgres_fdw не был зарегистрирован и
-- для выполнения команды необходимы права суперпользователя
CREATE SERVER foreign_server FOREIGN DATA WRAPPER postgres_fdw OPTIONS(host '127.0.0.1',
                                                                      port '6001',
                                                                      dbname 'mtest');
-- Подключаемся как пользователь postgres
\connect - postgres
-- Регистрируем расширение postgres_fdw - FOREIGN DATA WRAPPER для СУБД PostgreSQL
CREATE EXTENSION postgres_fdw;
-- Создаём внешний сервер (подключение к базе данных 'mtest')
-- Ожидается успех
CREATE SERVER foreign_server FOREIGN DATA WRAPPER postgres_fdw OPTIONS(host '127.0.0.1',
                                                                      port '6001',
                                                                      dbname 'mtest');
-- Подключаемся к базе данных 'mtest' как пользователь 'u_0_00'
\connect - u_0_00
-- Убеждаемся, что внешний сервер был создан
\des
-- Создаём внешнюю таблицу, являющуюся проекцией таблицы 'Таблица'
-- Ожидается ошибка - у пользователя 'u_0_00' отсутствует USAGE
CREATE FOREIGN TABLE "Внешняя_таблица"("Идентификатор" INTEGER, "Значение" TEXT) SERVER foreign_server OPTIONS (table_name 'Таблица');
-- Подключаемся как postgres и выдаём USAGE пользователю 'u_0_00'
\connect - postgres
GRANT USAGE ON FOREIGN SERVER foreign_server TO u_0_00;
\connect - u_0_00
-- Повторяем попытку создать внешнюю таблицу
-- Ожидается успех
CREATE FOREIGN TABLE "Внешняя_таблица"("Идентификатор" INTEGER, "Значение" TEXT) SERVER foreign_server OPTIONS (table_name 'Таблица');
-- Выполняем попытку посмотреть содержимое таблицы
-- Ожидается ошибка, т.к. не задано сопоставление пользователей
SELECT * FROM "Внешняя_таблица";
-- Сопоставляем пользователя 'u_0_00' пользователю 'u_0_00'
CREATE USER MAPPING FOR u_0_00 SERVER foreign_server OPTIONS (user 'u2_0_00', password '123456');
-- Выполняем попытку просмотреть содержимое таблицы 'Внешняя_таблица'
-- Ожидается ошибка, т.к. у пользователя 'u2_0_00' на удаленном сервере отсутствует привилегия SELECT на таблицу
SELECT * FROM "Внешняя_таблица";
-- Выдаём права SELECT пользователю u2_0_00
\connect - postgres - 6001
GRANT SELECT ON TABLE "Таблица" TO u2_0_00;
--Подключаемся к mtest и повторяем попытку
\connect - u_0_00 - 6000
-- Ожидается 3 записи
SELECT * FROM "Внешняя_таблица";
-- Пытаемся сделать INSERT в таблицу
-- Ожидается ошибка, т.к. у пользователя 'u2_0_00' на удаленном сервере отсутствует привилегия INSERT на таблицу
INSERT INTO "Внешняя_таблица" VALUES (4, 'Запись пользователя u_0_00');
-- Выдаём право INSERT пользователю 'u2_0_00'
\connect - postgres - 6001
GRANT INSERT ON TABLE "Таблица" TO u2_0_00;
-- Подключаемся как пользователь 'u_0_0' и повторяем
\connect mtest u_0_00 - 6000
INSERT INTO "Внешняя_таблица" VALUES(4, 'Запись пользователя u_0_00');
-- Удостоверимся, что запись действительно добавлена в таблицу
-- Ожидается 4 записи
SELECT * FROM "Внешняя_таблица";
-- Убедимся, что запись была добавлена в таблицу 'Таблица' в базе данных 'mtest' на удаленном сервере
\connect - u2_0_00 - 6001
-- Ожидается 4 записи
SELECT * FROM "Таблица";
-- Пробуем обновить данные в таблице
\connect - u_0_00 - 6000
-- Ожидается ошибка т.к. отсутствует UPDATE у пользователя 'u2_0_00'
UPDATE "Внешняя_таблица" SET "Значение" = 'Обновленная запись пользователем u_0_00' WHERE "Идентификатор" <= 3;
-- Выдаем право UPDATE пользователю 'u2_0_00'
\connect - postgres - 6001
GRANT UPDATE ON TABLE "Таблица" TO u2_0_00;
-- Повторяем попытку обновления записей
\connect - u_0_00 - 6000
-- Ожидается обновление трёх записей
UPDATE "Внешняя_таблица" SET "Значение" = 'Обновленная запись пользователем u_0_00' WHERE "Идентификатор" <= 3;
-- Убедимся в том, что записи были обновлены
-- Ожидается 4 записи, 3 из которых обновлены
SELECT * FROM "Внешняя_таблица";
--
--
-- Проверка дискреционных прав доступа на объекты FOREIGN TABLE
--
--
-- Отбираем все права на FOREIGN TABLE 'Внешняя_таблица'
\connect - postgres
REVOKE ALL ON TABLE "Внешняя_таблица" FROM u_0_00;
-- Подключаемся как u_0_00 и убедимся в отсутствии доступа к таблице 'Внешняя_таблица'
-- Ожидается ошибка доступа
\connect - u_0_00
SELECT * FROM "Внешняя_таблица";
INSERT INTO "Внешняя_таблица" VALUES(5, 'Новая запись');
UPDATE "Внешняя_таблица" SET "Значение" = 'Запись пользователя u2_0_00' WHERE "Идентификатор" = 1;
--
-- Даём право SELECT на таблицу 'Внешняя_таблица' пользователю 'u_0_00'
\connect - postgres
GRANT SELECT ON TABLE "Внешняя_таблица" TO u_0_00;
-- Проверяем, что пользователь 'u_0_00' обладает правом SELECT
-- Ожидается 4 записи
\connect - u_0_00
SELECT * FROM "Внешняя_таблица";
-- Даём право INSERT на таблицу 'Внешняя_таблица' пользователю 'u_0_00'
\connect - postgres
GRANT INSERT ON TABLE "Внешняя_таблица" TO u_0_00;
-- Проверяем, что пользователь 'u_0_00' обладает правом INSERT
-- Ожидается успех
\connect - u_0_00
INSERT INTO "Внешняя_таблица" VALUES(5, 'Новая запись');
-- Даём право UPDATE на таблицу 'Внешняя_таблица' пользоваетелю 'u_0_00'
\connect - postgres
GRANT UPDATE ON TABLE "Внешняя_таблица" TO u_0_00;
-- Проверяем, что пользователь 'u_0_00' обладает правом UPDATE
-- Ожидается успех
\connect - u_0_00
UPDATE "Внешняя_таблица" SET "Значение" = 'Запись пользователя u_0_00' WHERE "Идентификатор" = 5;
--
--
-- Проводим очистку на удаленном сервере
\connect - postgres - 6001
\i 'support/done.sql'
-- Проводим очистку на основном сервере
\connect - postgres - 6000
\i 'support/done.sql'