-- RBT-TEST - ДПРД для последовательностей
-- Проверка дискреционного разграничения доступа для последовательностей
-- Права доступа для последовательностей: USAGE, SELECT, UPDATE
--
\i 'support/init.sql'
--
-- Общая подготовка
\connect - postgres
--
-- Создаем последовательность
CREATE SEQUENCE "Последовательность"
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
--
-- Владельцу все действия разрешены
--
-- Проверка выполнения SELECT
-- ожидается успех
SELECT * FROM "Последовательность";
--
-- Проверка вызова функции nextval
-- ожидается успех
SELECT nextval('"Последовательность"');
--
-- Проверка вызова функции setval
-- ожидается успех
SELECT setval('"Последовательность"', 100);
--
-- Проверка вызова функции currval
-- ожидается успех
SELECT currval('"Последовательность"');
--
-- Подключение пользователя u_0_00
\connect - u_0_00
--
-- Изначально все действия пользователю над вновь созданной последовательностью запрещены
--
-- Проверка выполнения SELECT
-- ожидается ошибка доступа
SELECT * FROM "Последовательность";
--
-- Проверка вызова функции nextval
-- ожидается ошибка доступа
SELECT nextval('"Последовательность"');
--
-- Проверка вызова функции setval
-- ожидается ошибка доступа
SELECT setval('"Последовательность"', 100);
--
-- Проверка вызова функции currval
-- ожидается ошибка доступа
SELECT currval('"Последовательность"');
--
-- Администратор postgres дает право SELECT для пользователя u_0_00
-- все действия по получению данных разрешены
\connect - postgres
GRANT SELECT ON SEQUENCE "Последовательность" TO u_0_00;
--
-- Подключение пользователя u_0_00
\connect - u_0_00
--
-- Проверка выполнения SELECT
-- ожидается успех
SELECT * FROM "Последовательность";
--
-- Проверка вызова функции nextval
-- ожидается ошибка доступа
SELECT nextval('"Последовательность"');
--
-- Проверка вызова функции setval
-- ожидается ошибка доступа
SELECT setval('"Последовательность"', 100);
--
-- Проверка вызова функции currval
-- ожидается ошибка (не имеет смысла без возможности вызова nextval)
SELECT currval('"Последовательность"');
--
-- Администратор postgres отбирает право SELECT и дает право USAGE для пользователя u_0_00
\connect - postgres
REVOKE SELECT ON SEQUENCE "Последовательность" FROM u_0_00;
GRANT USAGE ON SEQUENCE "Последовательность" TO u_0_00;
--
-- Подключение пользователя u_0_00
\connect - u_0_00
--
-- Проверка выполнения SELECT
-- ожидается ошибка доступа
SELECT * FROM "Последовательность";
--
-- Проверка вызова функции nextval
-- ожидается успех
SELECT nextval('"Последовательность"');
--
-- Проверка вызова функции setval
-- ожидается ошибка доступа
SELECT setval('"Последовательность"', 100);
--
-- Проверка вызова функции currval
-- ожидается успех
SELECT currval('"Последовательность"');
--
-- Администратор postgres отбирает право USAGE и дает право UPDATE для пользователя u_0_00
\connect - postgres
REVOKE USAGE ON SEQUENCE "Последовательность" FROM u_0_00;
GRANT UPDATE ON SEQUENCE "Последовательность" TO u_0_00;
--
-- Подключение пользователя u_0_00
\connect - u_0_00
--
-- Проверка выполнения SELECT
-- ожидается ошибка доступа
SELECT * FROM "Последовательность";
--
-- Проверка вызова функции nextval
-- ожидается успех
SELECT nextval('"Последовательность"');
--
-- Проверка вызова функции setval
-- ожидается успех
SELECT setval('"Последовательность"', 100);
--
-- Проверка вызова функции currval
-- ожидается ошибка доступа
SELECT currval('"Последовательность"');
--
\i 'support/done.sql'
