-- RBT-TEST - МПРД при модификации метаданных
-- Проверка работы МПРД при выполнении команд изменяющих схему
-- ALTER, DROP, COMMENT, TRUNCATE
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
--
-- Администратор postgres дает право CREATE на базу данных для пользователя u_1_11
\connect - postgres
GRANT CREATE ON DATABASE mtest TO u_1_11;
--
-- Создает роль и назначает ее пользователю u_1_11
CREATE ROLE "роль";
GRANT "роль" TO u_1_11;
--
-- Создает табличное пространство и дает права на его использование пользователю u_1_11
CREATE TABLESPACE dbspace LOCATION '/var/lib/postgresql/9.6/testspace';
--
-- Устанавливаем метку табличного пространства dbspace
MAC LABEL ON TABLESPACE dbspace IS '{2,3}';
--
-- Сбрасываем признак MAC CСR у табличного пространства dbspace
MAC CCR ON TABLESPACE dbspace IS OFF;
--
GRANT ALL ON TABLESPACE dbspace TO u_1_11;
GRANT ALL ON TABLESPACE pg_default TO u_1_11;
--
-- Проверим выполнение команд для таблиц
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Создаем от его имени таблицу
CREATE TABLE test_u_1_11 ("идентификатор" integer) WITH (MACS=TRUE);
--
-- Создаем от его имени схему
CREATE SCHEMA "схема";
--
-- Проверка метки таблицы test_u_1_11
-- Ожидается, что метка таблицы соответствует метке пользователя {1,3}
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname = 'test_u_1_11';
--
-- Проверяем, что пользователь имеет возможность модифицировать свою таблицу со своей меткой
-- Ожидается успех
COMMENT ON TABLE test_u_1_11 IS 'проверочная таблица';
ALTER TABLE test_u_1_11 SET WITH OIDS;
ALTER TABLE test_u_1_11 ADD CONSTRAINT idnpk PRIMARY KEY ("идентификатор");
ALTER TABLE test_u_1_11 DROP CONSTRAINT idnpk;
ALTER TABLE test_u_1_11 ENABLE TRIGGER ALL;
ALTER TABLE test_u_1_11 SET WITHOUT CLUSTER;
ALTER TABLE test_u_1_11 OWNER TO "роль";
ALTER TABLE test_u_1_11 OWNER TO u_1_11;
ALTER TABLE test_u_1_11 RENAME TO "таблица";
ALTER TABLE "таблица" RENAME TO test_u_1_11;
ALTER TABLE test_u_1_11 SET SCHEMA "схема";
ALTER TABLE "схема".test_u_1_11 SET SCHEMA public;
ALTER TABLE test_u_1_11 SET TABLESPACE dbspace;
ALTER TABLE test_u_1_11 SET TABLESPACE pg_default;
TRUNCATE TABLE test_u_1_11;
ALTER TABLE test_u_1_11 ADD COLUMN "данные" TEXT;
COMMENT ON COLUMN test_u_1_11."данные" IS 'проверочный столбец';
ALTER TABLE test_u_1_11 ALTER COLUMN "данные" SET DEFAULT 'пусто';
ALTER TABLE test_u_1_11 ALTER COLUMN "данные" DROP DEFAULT;
ALTER TABLE test_u_1_11 ALTER COLUMN "данные" SET NOT NULL;
ALTER TABLE test_u_1_11 RENAME COLUMN "данные" TO "текст";
ALTER TABLE test_u_1_11 DROP COLUMN "текст";
--
-- Администратор кластера postgres устанавливает метку таблицы в {0,0}
\connect - postgres
MAC LABEL ON TABLE test_u_1_11 IS '{0,0}';
--
-- Проверка метки таблицы test_u_1_11
-- Ожидается, что метка таблицы установлена в {0,0}
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname = 'test_u_1_11';
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь не имеет возможность модифицировать свою таблицу
-- Ожидается ошибка доступа
COMMENT ON TABLE test_u_1_11 IS 'проверочная таблица';
ALTER TABLE test_u_1_11 SET WITH OIDS;
ALTER TABLE test_u_1_11 ADD CONSTRAINT idnpk PRIMARY KEY ("идентификатор");
ALTER TABLE test_u_1_11 ENABLE TRIGGER ALL;
ALTER TABLE test_u_1_11 SET WITHOUT CLUSTER;
ALTER TABLE test_u_1_11 OWNER TO "роль";
ALTER TABLE test_u_1_11 RENAME TO "таблица";
ALTER TABLE test_u_1_11 SET SCHEMA "схема";
ALTER TABLE test_u_1_11 SET TABLESPACE dbspace;
TRUNCATE TABLE test_u_1_11;
ALTER TABLE test_u_1_11 ADD COLUMN "данные" TEXT;
COMMENT ON COLUMN test_u_1_11."идентификатор" IS 'проверочный столбец';
ALTER TABLE test_u_1_11 ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER TABLE test_u_1_11 ALTER COLUMN "идентификатор" DROP DEFAULT;
ALTER TABLE test_u_1_11 ALTER COLUMN "идентификатор" SET NOT NULL;
ALTER TABLE test_u_1_11 RENAME COLUMN "идентификатор" TO "идн";
ALTER TABLE test_u_1_11 DROP COLUMN "идентификатор";
--
-- Администратор кластера postgres устанавливает метку таблицы в {2,0}
\connect - postgres
MAC LABEL ON TABLE test_u_1_11 IS '{2,0}';
--
-- Проверка метки таблицы test_u_1_11
-- Ожидается, что метка таблицы установлена в {2,0}
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname = 'test_u_1_11';
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь не имеет возможность модифицировать свою таблицу
-- Ожидается, что объект недоступен для пользователя
COMMENT ON TABLE test_u_1_11 IS 'проверочная таблица';
ALTER TABLE test_u_1_11 SET WITH OIDS;
ALTER TABLE test_u_1_11 ADD CONSTRAINT idnpk PRIMARY KEY ("идентификатор");
ALTER TABLE test_u_1_11 ENABLE TRIGGER ALL;
ALTER TABLE test_u_1_11 SET WITHOUT CLUSTER;
ALTER TABLE test_u_1_11 OWNER TO "роль";
ALTER TABLE test_u_1_11 RENAME TO "таблица";
ALTER TABLE test_u_1_11 SET SCHEMA "схема";
ALTER TABLE test_u_1_11 SET TABLESPACE dbspace;
TRUNCATE TABLE test_u_1_11;
ALTER TABLE test_u_1_11 ADD COLUMN "данные" TEXT;
COMMENT ON COLUMN test_u_1_11."идентификатор" IS 'проверочный столбец';
ALTER TABLE test_u_1_11 ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER TABLE test_u_1_11 ALTER COLUMN "идентификатор" DROP DEFAULT;
ALTER TABLE test_u_1_11 ALTER COLUMN "идентификатор" SET NOT NULL;
ALTER TABLE test_u_1_11 RENAME COLUMN "идентификатор" TO "идн";
ALTER TABLE test_u_1_11 DROP COLUMN "идентификатор";
--
-- Администратор кластера postgres сбрасывает признак MAC CCR на таблицу
\connect - postgres
MAC CCR ON TABLE test_u_1_11 IS OFF;
--
-- Проверка метки таблицы test_u_1_11
-- Ожидается, что признак MAC CCR таблицы сброшен
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname = 'test_u_1_11';
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь не имеет возможность модифицировать свою таблицу
-- Ожидается ошибка доступа
COMMENT ON TABLE test_u_1_11 IS 'проверочная таблица';
ALTER TABLE test_u_1_11 SET WITH OIDS;
ALTER TABLE test_u_1_11 ADD CONSTRAINT idnpk PRIMARY KEY ("идентификатор");
ALTER TABLE test_u_1_11 DROP CONSTRAINT idnpk;
ALTER TABLE test_u_1_11 ENABLE TRIGGER ALL;
ALTER TABLE test_u_1_11 SET WITHOUT CLUSTER;
ALTER TABLE test_u_1_11 OWNER TO "роль";
ALTER TABLE test_u_1_11 OWNER TO u_1_11;
ALTER TABLE test_u_1_11 RENAME TO "таблица";
ALTER TABLE test_u_1_11 SET SCHEMA "схема";
ALTER TABLE test_u_1_11 SET TABLESPACE dbspace;
TRUNCATE TABLE test_u_1_11;
ALTER TABLE test_u_1_11 ADD COLUMN "данные" TEXT;
ALTER TABLE test_u_1_11 ALTER COLUMN "данные" SET DEFAULT 'пусто';
ALTER TABLE test_u_1_11 ALTER COLUMN "данные" DROP DEFAULT;
ALTER TABLE test_u_1_11 ALTER COLUMN "данные" SET NOT NULL;
ALTER TABLE test_u_1_11 RENAME COLUMN "данные" TO "текст";
ALTER TABLE test_u_1_11 DROP COLUMN "текст";
--
-- Проверим выполнение команд для видов
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Создание проверочного вида
CREATE VIEW test_view_u_1_11 AS SELECT * FROM test_u_1_11;
--
-- Проверка метки вида test_view_u_1_11
-- Ожидается, что метка вида соответствует метке пользователя {1,3}
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname = 'test_view_u_1_11';
--
-- Проверяем, что пользователь имеет возможность модифицировать свой вид со своей меткой
-- Ожидается успех
COMMENT ON VIEW test_view_u_1_11 IS 'проверочный вид';
ALTER VIEW test_view_u_1_11 OWNER TO "роль";
ALTER VIEW test_view_u_1_11 OWNER TO u_1_11;
ALTER VIEW test_view_u_1_11 RENAME TO "вид";
ALTER VIEW "вид" RENAME TO test_view_u_1_11;
ALTER VIEW test_view_u_1_11 SET SCHEMA "схема";
ALTER VIEW "схема".test_view_u_1_11 SET SCHEMA public;
ALTER TABLE test_view_u_1_11 OWNER TO "роль";
ALTER TABLE test_view_u_1_11 OWNER TO u_1_11;
ALTER TABLE test_view_u_1_11 RENAME TO "вид";
ALTER TABLE "вид" RENAME TO test_view_u_1_11;
ALTER TABLE test_view_u_1_11 SET SCHEMA "схема";
ALTER TABLE "схема".test_view_u_1_11 SET SCHEMA public;
COMMENT ON COLUMN test_view_u_1_11."идентификатор" IS 'проверочный столбец';
ALTER VIEW test_view_u_1_11 ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER VIEW test_view_u_1_11 ALTER COLUMN "идентификатор" DROP DEFAULT;
--
-- Переименование столбцов командой ALTER TABLE
-- Ожидается успех
ALTER TABLE test_view_u_1_11 RENAME COLUMN "идентификатор" TO "идн";
ALTER TABLE test_view_u_1_11 RENAME COLUMN "идн" TO "идентификатор";
--
-- Администратор кластера postgres устанавливает метку вида в {0,0}
\connect - postgres
MAC LABEL ON VIEW test_view_u_1_11 IS '{0,0}';
--
-- Проверка метки вида test_view_u_1_11
-- Ожидается, что метка вида установлена в {0,0}
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname = 'test_view_u_1_11';
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь не имеет возможность модифицировать свой вид
-- Ожидается ошибка доступа
COMMENT ON VIEW test_view_u_1_11 IS 'проверочный вид';
ALTER VIEW test_view_u_1_11 OWNER TO "роль";
ALTER VIEW test_view_u_1_11 RENAME TO "вид";
ALTER VIEW test_view_u_1_11 SET SCHEMA "схема";
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается ошибка доступа
ALTER TABLE test_view_u_1_11 OWNER TO "роль";
ALTER TABLE test_view_u_1_11 RENAME TO "вид";
ALTER TABLE test_view_u_1_11 SET SCHEMA "схема";
--
-- Ожидается ошибка доступа
COMMENT ON COLUMN test_view_u_1_11."идентификатор" IS 'проверочный столбец';
ALTER VIEW test_view_u_1_11 ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER VIEW test_view_u_1_11 ALTER COLUMN "идентификатор" DROP DEFAULT;
--
-- Переименование столбцов командой ALTER TABLE
-- Ожидается ошибка доступа
ALTER TABLE test_view_u_1_11 RENAME COLUMN "идентификатор" TO "идн";
--
-- Администратор кластера postgres устанавливает метку вида в {2,0}
\connect - postgres
MAC LABEL ON VIEW test_view_u_1_11 IS '{2,0}';
--
-- Проверка метки вида test_view_u_1_11
-- Ожидается, что метка вида установлена в {2,0}
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname = 'test_view_u_1_11';
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь не имеет возможность модифицировать свой вид
-- Ожидается, что объект недоступен для пользователя
COMMENT ON VIEW test_view_u_1_11 IS 'проверочный вид';
ALTER VIEW test_view_u_1_11 OWNER TO "роль";
ALTER VIEW test_view_u_1_11 RENAME TO "вид";
ALTER VIEW test_view_u_1_11 SET SCHEMA "схема";
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается ошибка доступа
ALTER TABLE test_view_u_1_11 OWNER TO "роль";
ALTER TABLE test_view_u_1_11 RENAME TO "вид";
ALTER TABLE test_view_u_1_11 SET SCHEMA "схема";
--
-- Ожидается ошибка доступа
COMMENT ON COLUMN test_view_u_1_11."идентификатор" IS 'проверочный столбец';
ALTER VIEW test_view_u_1_11 ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER VIEW test_view_u_1_11 ALTER COLUMN "идентификатор" DROP DEFAULT;
--
-- Переименование столбцов командой ALTER TABLE
-- Ожидается ошибка доступа
ALTER TABLE test_view_u_1_11 RENAME COLUMN "идентификатор" TO "идн";
--
-- Проверим выполнение команд ALTER SEQUENCE и ALTER TABLE для последовательностей
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Создаем проверочную последовательность
CREATE SEQUENCE test_seq_u_1_11
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
--
-- Проверка метки последовательности test_seq_u_1_11
-- Ожидается, что метка последовательности соответствует метке пользователя {1,3}
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname = 'test_seq_u_1_11';
--
-- Проверяем, что пользователь имеет возможность модифицировать свою последовательность со своей меткой
-- Ожидается успех
COMMENT ON SEQUENCE test_seq_u_1_11 IS 'проверочная последовательность';
ALTER SEQUENCE test_seq_u_1_11 INCREMENT BY 2;
ALTER SEQUENCE test_seq_u_1_11 RESTART;
ALTER SEQUENCE test_seq_u_1_11 RENAME TO "последовательность";
ALTER SEQUENCE "последовательность" RENAME TO test_seq_u_1_11;
ALTER SEQUENCE test_seq_u_1_11 SET SCHEMA "схема";
ALTER SEQUENCE "схема".test_seq_u_1_11 SET SCHEMA public;
ALTER SEQUENCE test_seq_u_1_11 OWNER TO "роль";
ALTER SEQUENCE test_seq_u_1_11 OWNER TO u_1_11;
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается успех
ALTER TABLE test_seq_u_1_11 RENAME TO "последовательность";
ALTER TABLE "последовательность" RENAME TO test_seq_u_1_11;
ALTER TABLE test_seq_u_1_11 SET SCHEMA "схема";
ALTER TABLE "схема".test_seq_u_1_11 SET SCHEMA public;
ALTER TABLE test_seq_u_1_11 OWNER TO "роль";
ALTER TABLE test_seq_u_1_11 OWNER TO u_1_11;
--
-- Администратор кластера postgres устанавливает метку последовательности в {0,0}
\connect - postgres
MAC LABEL ON SEQUENCE test_seq_u_1_11 IS '{0,0}';
--
-- Проверка метки последовательности test_seq_u_1_11
-- Ожидается, что метка последовательности установлена в {0,0}
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname = 'test_seq_u_1_11';
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь не имеет возможность модифицировать свою последовательность
-- Ожидается ошибка доступа
COMMENT ON SEQUENCE test_seq_u_1_11 IS 'проверочная последовательность';
ALTER SEQUENCE test_seq_u_1_11 INCREMENT BY 2;
ALTER SEQUENCE test_seq_u_1_11 RESTART;
ALTER SEQUENCE test_seq_u_1_11 RENAME TO "последовательность";
ALTER SEQUENCE test_seq_u_1_11 SET SCHEMA "схема";
ALTER SEQUENCE test_seq_u_1_11 OWNER TO "роль";
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается ошибка доступа
ALTER TABLE test_seq_u_1_11 RENAME TO "последовательность";
ALTER TABLE test_seq_u_1_11 SET SCHEMA "схема";
ALTER TABLE test_seq_u_1_11 OWNER TO "роль";
--
-- Администратор кластера postgres устанавливает метку последовательности в {2,0}
\connect - postgres
MAC LABEL ON SEQUENCE test_seq_u_1_11 IS '{2,0}';
--
-- Проверка метки последовательности test_seq_u_1_11
-- Ожидается, что метка последовательности установлена в {2,0}
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname = 'test_seq_u_1_11';
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь не имеет возможность модифицировать свою последовательность
-- Ожидается, что объект недоступен для пользователя
COMMENT ON SEQUENCE test_seq_u_1_11 IS 'проверочная последовательность';
ALTER SEQUENCE test_seq_u_1_11 INCREMENT BY 2;
ALTER SEQUENCE test_seq_u_1_11 RESTART;
ALTER SEQUENCE test_seq_u_1_11 RENAME TO "последовательность";
ALTER SEQUENCE test_seq_u_1_11 SET SCHEMA "схема";
ALTER SEQUENCE test_seq_u_1_11 OWNER TO "роль";
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается ошибка доступа
ALTER TABLE test_seq_u_1_11 RENAME TO "последовательность";
ALTER TABLE test_seq_u_1_11 SET SCHEMA "схема";
ALTER TABLE test_seq_u_1_11 OWNER TO "роль";
--
-- Проверим выполнение команды DROP для объектов
-- DROP для столбцов уже был рассмотрен выше
--
-- Проверяем, что пользователь не имеет возможность удалять свои объекты с отличной от своей меткой
-- Ожидается ошибка доступа
DROP SEQUENCE test_seq_u_1_11;
DROP VIEW test_view_u_1_11;
DROP TABLE test_u_1_11;
--
-- Администратор кластера postgres устанавливает метки объектов в {1, 3}
-- а метки столбцов в {2,0}
-- и включает режим использования меток столбцов
\connect - postgres
MAC LABEL ON SEQUENCE test_seq_u_1_11 IS '{1,3}';
MAC LABEL ON VIEW test_view_u_1_11 IS '{1,3}';
MAC LABEL ON TABLE test_u_1_11 IS '{1,3}';
--
-- Проверка метки объектов
-- Ожидается, что метки установлены в {1,3}
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname LIKE 'test_%';
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь может удалять свои объекты со своей меткой
-- Ожидается успех
DROP SEQUENCE test_seq_u_1_11;
DROP VIEW test_view_u_1_11;
DROP TABLE test_u_1_11;
--
--
-- Проверка доступности всех команд для пользователя с атрибутами игнорирования мандатного доступа dbsa
--
-- Администратор postgres дает право CREATE на базу данных для пользователя dbsa
\connect - postgres
GRANT CREATE ON DATABASE mtest TO dbsa;
--
-- Назначает роль пользователю dbsa
GRANT "роль" TO dbsa;
--
-- Дает права на использование области хранения данных пользователю dbsa
GRANT ALL ON TABLESPACE dbspace TO dbsa;
GRANT ALL ON TABLESPACE pg_default TO dbsa;
--
-- Подключаемся как пользователь dbsa с меткой {7,1fffffffffffffff}
\connect - dbsa
--
-- Создаем от его имени таблицу
CREATE TABLE test_u_dbsa ("идентификатор" integer);
--
-- Создаем от его имени схему
CREATE SCHEMA "схема dbsa";
--
-- Проверка метки таблицы test_u_dbsa
-- Ожидается, что метка таблицы соответствует метке пользователя {7,1fffffffffffffff}
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname = 'test_u_dbsa';
--
-- Проверяем, что пользователь имеет возможность модифицировать свою таблицу со своей меткой
-- Ожидается успех
COMMENT ON TABLE test_u_dbsa IS 'проверочная таблица';
ALTER TABLE test_u_dbsa SET WITH OIDS;
ALTER TABLE test_u_dbsa ADD CONSTRAINT idnpk PRIMARY KEY ("идентификатор");
ALTER TABLE test_u_dbsa DROP CONSTRAINT idnpk;
ALTER TABLE test_u_dbsa ENABLE TRIGGER ALL;
ALTER TABLE test_u_dbsa SET WITHOUT CLUSTER;
ALTER TABLE test_u_dbsa OWNER TO "роль";
ALTER TABLE test_u_dbsa OWNER TO dbsa;
ALTER TABLE test_u_dbsa RENAME TO "таблица";
ALTER TABLE "таблица" RENAME TO test_u_dbsa;
ALTER TABLE test_u_dbsa SET SCHEMA "схема dbsa";
ALTER TABLE "схема dbsa".test_u_dbsa SET SCHEMA public;
ALTER TABLE test_u_dbsa SET TABLESPACE dbspace;
ALTER TABLE test_u_dbsa SET TABLESPACE pg_default;
TRUNCATE TABLE test_u_dbsa;
--
-- Ожидается успех
ALTER TABLE test_u_dbsa ADD COLUMN "данные" TEXT;
COMMENT ON COLUMN test_u_dbsa."данные" IS 'проверочный столбец';
ALTER TABLE test_u_dbsa ALTER COLUMN "данные" SET DEFAULT 'пусто';
ALTER TABLE test_u_dbsa ALTER COLUMN "данные" DROP DEFAULT;
ALTER TABLE test_u_dbsa ALTER COLUMN "данные" SET NOT NULL;
ALTER TABLE test_u_dbsa RENAME COLUMN "данные" TO "текст";
ALTER TABLE test_u_dbsa DROP COLUMN "текст";
--
-- Пользователь dbsa устанавливает метку таблицы в {0,0}
MAC LABEL ON TABLE test_u_dbsa IS '{0,0}';
--
-- Проверка метки таблицы test_u_dbsa
-- Ожидается, что метка таблицы установлена в {0,0}
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname = 'test_u_dbsa';
--
-- Проверяем, что пользователь имеет возможность модифицировать свою таблицу с любой меткой
-- Ожидается успех
COMMENT ON TABLE test_u_dbsa IS 'проверочная таблица';
ALTER TABLE test_u_dbsa SET WITH OIDS;
ALTER TABLE test_u_dbsa ADD CONSTRAINT idnpk PRIMARY KEY ("идентификатор");
ALTER TABLE test_u_dbsa DROP CONSTRAINT idnpk;
ALTER TABLE test_u_dbsa ENABLE TRIGGER ALL;
ALTER TABLE test_u_dbsa SET WITHOUT CLUSTER;
ALTER TABLE test_u_dbsa OWNER TO "роль";
ALTER TABLE test_u_dbsa OWNER TO dbsa;
ALTER TABLE test_u_dbsa RENAME TO "таблица";
ALTER TABLE "таблица" RENAME TO test_u_dbsa;
ALTER TABLE test_u_dbsa SET SCHEMA "схема dbsa";
ALTER TABLE "схема dbsa".test_u_dbsa SET SCHEMA public;
ALTER TABLE test_u_dbsa SET TABLESPACE dbspace;
ALTER TABLE test_u_dbsa SET TABLESPACE pg_default;
TRUNCATE TABLE test_u_dbsa;
--
-- Ожидается успех
ALTER TABLE test_u_dbsa ADD COLUMN "данные" TEXT;
COMMENT ON COLUMN test_u_dbsa."данные" IS 'проверочный столбец';
ALTER TABLE test_u_dbsa ALTER COLUMN "данные" SET DEFAULT 'пусто';
ALTER TABLE test_u_dbsa ALTER COLUMN "данные" DROP DEFAULT;
ALTER TABLE test_u_dbsa ALTER COLUMN "данные" SET NOT NULL;
ALTER TABLE test_u_dbsa RENAME COLUMN "данные" TO "текст";
ALTER TABLE test_u_dbsa DROP COLUMN "текст";
--
--
-- Проверим выполнение команд для видов
--
-- Создание проверочного вида
CREATE VIEW test_view_u_dbsa AS SELECT * FROM test_u_dbsa;
--
-- Проверка метки вида test_view_u_dbsa
-- Ожидается, что метка вида соответствует метке пользователя {7,1fffffffffffffff}
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname = 'test_view_u_dbsa';
--
-- Проверяем, что пользователь имеет возможность модифицировать свой вид со своей меткой
-- Ожидается успех
COMMENT ON VIEW test_view_u_dbsa IS 'проверочный вид';
ALTER VIEW test_view_u_dbsa OWNER TO "роль";
ALTER VIEW test_view_u_dbsa OWNER TO dbsa;
ALTER VIEW test_view_u_dbsa RENAME TO "вид";
ALTER VIEW "вид" RENAME TO test_view_u_dbsa;
ALTER VIEW test_view_u_dbsa SET SCHEMA "схема dbsa";
ALTER VIEW "схема dbsa".test_view_u_dbsa SET SCHEMA public;
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается успех
ALTER TABLE test_view_u_dbsa OWNER TO "роль";
ALTER TABLE test_view_u_dbsa OWNER TO dbsa;
ALTER TABLE test_view_u_dbsa RENAME TO "вид";
ALTER TABLE "вид" RENAME TO test_view_u_dbsa;
ALTER TABLE test_view_u_dbsa SET SCHEMA "схема dbsa";
ALTER TABLE "схема dbsa".test_view_u_dbsa SET SCHEMA public;
--
-- Ожидается успех
COMMENT ON COLUMN test_view_u_dbsa."идентификатор" IS 'проверочный столбец';
ALTER VIEW test_view_u_dbsa ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER VIEW test_view_u_dbsa ALTER COLUMN "идентификатор" DROP DEFAULT;
--
-- Переименование столбцов командой ALTER TABLE
-- Ожидается успех
ALTER TABLE test_view_u_dbsa RENAME COLUMN "идентификатор" TO "идн";
ALTER TABLE test_view_u_dbsa RENAME COLUMN "идн" TO "идентификатор";
--
-- Пользователь dbsa устанавливает метку вида в {0,0}
MAC LABEL ON VIEW test_view_u_dbsa IS '{0,0}';
--
-- Проверка метки вида test_view_u_dbsa
-- Ожидается, что метка вида установлена в {0,0}
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname = 'test_view_u_dbsa';
--
-- Проверяем, что пользователь имеет возможность модифицировать свой вид
-- Ожидается успех
COMMENT ON VIEW test_view_u_dbsa IS 'проверочный вид';
ALTER VIEW test_view_u_dbsa OWNER TO "роль";
ALTER VIEW test_view_u_dbsa OWNER TO dbsa;
ALTER VIEW test_view_u_dbsa RENAME TO "вид";
ALTER VIEW "вид" RENAME TO test_view_u_dbsa;
ALTER VIEW test_view_u_dbsa SET SCHEMA "схема dbsa";
ALTER VIEW "схема dbsa".test_view_u_dbsa SET SCHEMA public;
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается успех
ALTER TABLE test_view_u_dbsa OWNER TO "роль";
ALTER TABLE test_view_u_dbsa OWNER TO dbsa;
ALTER TABLE test_view_u_dbsa RENAME TO "вид";
ALTER TABLE "вид" RENAME TO test_view_u_dbsa;
ALTER TABLE test_view_u_dbsa SET SCHEMA "схема dbsa";
ALTER TABLE "схема dbsa".test_view_u_dbsa SET SCHEMA public;
--
-- Ожидается успех
COMMENT ON COLUMN test_view_u_dbsa."идентификатор" IS 'проверочный столбец';
ALTER VIEW test_view_u_dbsa ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER VIEW test_view_u_dbsa ALTER COLUMN "идентификатор" DROP DEFAULT;
--
-- Переименование столбцов командой ALTER TABLE
-- Ожидается успех
ALTER TABLE test_view_u_dbsa RENAME COLUMN "идентификатор" TO "идн";
ALTER TABLE test_view_u_dbsa RENAME COLUMN "идн" TO "идентификатор";
--
--
-- Проверим выполнение команд ALTER SEQUENCE и ALTER TABLE для последовательностей
--
-- Создаем проверочную последовательность
CREATE SEQUENCE test_seq_u_dbsa
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
--
-- Проверка метки последовательности test_seq_u_dbsa
-- Ожидается, что метка последовательности соответствует метке пользователя {7,1fffffffffffffff}
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname = 'test_seq_u_dbsa';
--
-- Проверяем, что пользователь имеет возможность модифицировать свою последовательность со своей меткой
-- Ожидается успех
COMMENT ON SEQUENCE test_seq_u_dbsa IS 'проверочная последовательность';
ALTER SEQUENCE test_seq_u_dbsa INCREMENT BY 2;
ALTER SEQUENCE test_seq_u_dbsa RESTART;
ALTER SEQUENCE test_seq_u_dbsa RENAME TO "последовательность";
ALTER SEQUENCE "последовательность" RENAME TO test_seq_u_dbsa;
ALTER SEQUENCE test_seq_u_dbsa SET SCHEMA "схема dbsa";
ALTER SEQUENCE "схема dbsa".test_seq_u_dbsa SET SCHEMA public;
ALTER SEQUENCE test_seq_u_dbsa OWNER TO "роль";
ALTER SEQUENCE test_seq_u_dbsa OWNER TO dbsa;
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается успех
ALTER TABLE test_seq_u_dbsa RENAME TO "последовательность";
ALTER TABLE "последовательность" RENAME TO test_seq_u_dbsa;
ALTER TABLE test_seq_u_dbsa SET SCHEMA "схема dbsa";
ALTER TABLE "схема dbsa".test_seq_u_dbsa SET SCHEMA public;
ALTER TABLE test_seq_u_dbsa OWNER TO "роль";
ALTER TABLE test_seq_u_dbsa OWNER TO dbsa;
--
-- Пользователь dbsa устанавливает метку последовательности в {0,0}
MAC LABEL ON SEQUENCE test_seq_u_dbsa IS '{0,0}';
--
-- Проверка метки последовательности test_seq_u_dbsa
-- Ожидается, что метка последовательности установлена в {0,0}
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname = 'test_seq_u_dbsa';
--
-- Проверяем, что пользователь имеет возможность модифицировать свою последовательность с любой меткой
-- Ожидается успех
COMMENT ON SEQUENCE test_seq_u_dbsa IS 'проверочная последовательность';
ALTER SEQUENCE test_seq_u_dbsa INCREMENT BY 2;
ALTER SEQUENCE test_seq_u_dbsa RESTART;
ALTER SEQUENCE test_seq_u_dbsa RENAME TO "последовательность";
ALTER SEQUENCE "последовательность" RENAME TO test_seq_u_dbsa;
ALTER SEQUENCE test_seq_u_dbsa SET SCHEMA "схема dbsa";
ALTER SEQUENCE "схема dbsa".test_seq_u_dbsa SET SCHEMA public;
ALTER SEQUENCE test_seq_u_dbsa OWNER TO "роль";
ALTER SEQUENCE test_seq_u_dbsa OWNER TO dbsa;
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается успех
ALTER TABLE test_seq_u_dbsa RENAME TO "последовательность";
ALTER TABLE "последовательность" RENAME TO test_seq_u_dbsa;
ALTER TABLE test_seq_u_dbsa SET SCHEMA "схема dbsa";
ALTER TABLE "схема dbsa".test_seq_u_dbsa SET SCHEMA public;
ALTER TABLE test_seq_u_dbsa OWNER TO "роль";
ALTER TABLE test_seq_u_dbsa OWNER TO dbsa;
--
-- Проверяем, что пользователь имеет возможность удалять свои объекты с любой меткой
-- Ожидается успех
DROP SEQUENCE test_seq_u_dbsa;
DROP VIEW test_view_u_dbsa;
DROP TABLE test_u_dbsa;
--
-- Администратор кластера postgres удаляет созданные объекты
\connect - postgres
REVOKE "роль" FROM u_1_11;
REVOKE "роль" FROM dbsa;
DROP ROLE "роль";
REVOKE ALL ON TABLESPACE pg_default FROM u_1_11;
REVOKE ALL ON TABLESPACE pg_default FROM dbsa;
DROP TABLESPACE dbspace;
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'