-- RBT-TEST - ограничения CHMAC для внешних ключей
-- Проверка работы ограничений CHMAC для внешних ключей
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
--
-- Создаем таблицу с составным ключом и защищенными строками
CREATE TABLE macpkr (
    id   INTEGER,
    num  NUMERIC,
    data TEXT,

    PRIMARY KEY (id, num)
) WITH (MACS=TRUE);
GRANT ALL ON macpkr TO PUBLIC;
MAC LABEL ON TABLE macpkr IS '{2,3}';
MAC CCR ON TABLE macpkr IS OFF;
--
-- Создаем таблицу с составным ключом и незащищенными строками
CREATE TABLE macpk (
    id   INTEGER,
    num  NUMERIC,
    data TEXT,

    PRIMARY KEY (id, num)
);
GRANT ALL ON macpk TO PUBLIC;
--
-- Создаем таблицу с составным ключом и защищенными строками
CREATE TABLE macfk (
    id   INTEGER,
    num  NUMERIC,
    data TEXT,

    PRIMARY KEY (id, num)
) WITH (MACS=TRUE);
GRANT ALL ON macfk TO PUBLIC;
MAC LABEL ON TABLE macfk IS '{2,3}';
MAC CCR ON TABLE macfk IS OFF;
--
-- Создаем внешний ключ с действием по умолчанию для CHMAC
-- Ожидается ошибка
ALTER TABLE macfk ADD CONSTRAINT chmacfk FOREIGN KEY (id, num) REFERENCES macpk(id, num) ON CHMAC NO ACTION;
--
-- Создаем внешний ключ с действием по умолчанию для CHMAC
-- Ожидается успех
ALTER TABLE macfk ADD CONSTRAINT chmacfk FOREIGN KEY (id, num) REFERENCES macpkr(id, num) ON CHMAC NO ACTION;
--
-- Заполняем данными первичную таблицу macpkr
INSERT INTO macpkr(id, num, data) VALUES(1, 2, 'macpkr');
INSERT INTO macpkr(id, num, data) VALUES(2, 2, 'macpkr');
INSERT INTO macpkr(id, num, data) VALUES(3, 2, 'macpkr');
--
-- Проверяем таблицу macpkr
SELECT maclabel, * FROM macpkr ORDER BY id, num;
--
-- Заполняем данными вторичную таблицу macfk
-- Ожидается ошибка
INSERT INTO macfk(id, num, data) VALUES(0, 1, 'macfk');
--
-- Ожидается успех
INSERT INTO macfk(id, num, data) VALUES(1, 2, 'macfk');
INSERT INTO macfk(id, num, data) VALUES(2, 2, 'macfk');
--
-- Проверяем таблицу macfk
SELECT maclabel, * FROM macfk ORDER BY id, num;
--
-- Проверяем модификацию метки пользователем с атрибутами игнорирования МПРД
\connect - dbsa
--
-- Ожидается успех
CHMAC macpkr SET maclabel = '{1,0}' WHERE id = 3;
--
-- Ожидается успех
CHMAC macpkr SET maclabel = '{0,0}' WHERE id = 1;
--
-- Ожидается ошибка
CHMAC macpkr SET maclabel = '{1,0}' WHERE id = 1;
--
-- Проверяем таблицу macpkr
SELECT maclabel, * FROM macpkr ORDER BY id, num;
--
-- Проверяем таблицу macfk
SELECT maclabel, * FROM macfk ORDER BY id, num;
--
-- Удаляем внешний ключ с действием по умолчанию для CHMAC
-- Ожидается успех
\connect - postgres
ALTER TABLE macfk DROP CONSTRAINT chmacfk;
--
-- Создаем внешний ключ с действием RESTRICT для CHMAC
-- Ожидается успех
ALTER TABLE macfk ADD CONSTRAINT chmacfk FOREIGN KEY (id, num) REFERENCES macpkr(id, num) ON CHMAC RESTRICT;
--
--
-- Ожидается успех
\connect - dbsa
CHMAC macpkr SET maclabel = '{0,0}' WHERE id = 3;
--
-- Ожидается успех
CHMAC macpkr SET maclabel = '{0,0}' WHERE id = 1;
--
-- Ожидается ошибка
CHMAC macpkr SET maclabel = '{1,2}' WHERE id = 1;
--
-- Проверяем таблицу macpkr
SELECT maclabel, * FROM macpkr ORDER BY id, num;
--
-- Проверяем таблицу macfk
SELECT maclabel, * FROM macfk ORDER BY id, num;
--
-- Удаляем внешний ключ с действием RESTRICT для CHMAC
-- Ожидается успех
\connect - postgres
ALTER TABLE macfk DROP CONSTRAINT chmacfk;
--
-- Создаем внешний ключ с действием CASCADE для CHMAC
-- Ожидается успех
ALTER TABLE macfk ADD CONSTRAINT chmacfk FOREIGN KEY (id, num) REFERENCES macpkr(id, num) ON CHMAC CASCADE;
--
-- Ожидается успех
\connect - dbsa
CHMAC macpkr SET maclabel = '{1,2}' WHERE id = 3;
--
-- Ожидается успех
CHMAC macpkr SET maclabel = '{0,0}' WHERE id = 1;
--
-- Ожидается успех
CHMAC macpkr SET maclabel = '{1,2}' WHERE id = 1;
--
-- Проверяем таблицу macpkr
SELECT maclabel, * FROM macpkr ORDER BY id, num;
--
-- Проверяем таблицу macfk
SELECT maclabel, * FROM macfk ORDER BY id, num;
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'