-- RBT-TEST - CHMAC, работа триггеров и правил ON CHMAC
-- Проверка работы команды CHMAC, правил и триггеров
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Создаем вспомогательную таблицу
CREATE TABLE "Проверка_CHMAC" (info TEXT) WITH (MACS=TRUE);
GRANT ALL ON "Проверка_CHMAC" TO PUBLIC;
MAC LABEL ON TABLE "Проверка_CHMAC" IS '{2,3}';
MAC CCR ON TABLE "Проверка_CHMAC" IS OFF;
--
-- Подключается администратор postgres
\connect - postgres
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами вызывающего пользователя
CREATE FUNCTION "ТрФунк1"() RETURNS trigger LANGUAGE plpgsql AS $$
BEGIN
  NEW."данные" = NEW."данные" || '-ТрФунк1';
  RETURN NEW;
END;$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФунк1" () TO PUBLIC;
--
-- Создаем триггер
CREATE TRIGGER "ТриггерBU"
  BEFORE UPDATE
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФунк1"();
--
-- Подключается администратор postgres
\connect - postgres
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами вызывающего пользователя
CREATE FUNCTION "ТрФункCHMAC"() RETURNS trigger LANGUAGE plpgsql AS $$
BEGIN
  NEW."данные" = NEW."данные" || '-ТрФункCHMAC';
  INSERT INTO "Проверка_CHMAC"(info) VALUES(NEW."идентификатор" || '-BRCHMAC-' || USER);
  RETURN NEW;
END;$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФункCHMAC" () TO PUBLIC;
--
-- Создаем триггер
CREATE TRIGGER "ТриггерBCHMAC"
  BEFORE CHMAC
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункCHMAC"();
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами вызывающего пользователя
CREATE FUNCTION "ТрФункCHMAC2"() RETURNS trigger LANGUAGE plpgsql AS $$
BEGIN
  INSERT INTO "Проверка_CHMAC"(info) VALUES(NEW."идентификатор" || '-ARCHMAC-' || USER);
  RETURN NEW;
END;$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФункCHMAC2" () TO PUBLIC;
--
-- Создаем триггер
CREATE TRIGGER "ТриггерACHMAC"
  AFTER CHMAC
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункCHMAC2"();
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами вызывающего пользователя
CREATE FUNCTION "ТрФункCHMAC3"() RETURNS trigger LANGUAGE plpgsql AS $$
BEGIN
  INSERT INTO "Проверка_CHMAC"(info) VALUES('-BSCHMAC-' || USER);
  RETURN NEW;
END;$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФункCHMAC3" () TO PUBLIC;
--
-- Создаем триггер
CREATE TRIGGER "ТриггерBSCHMAC"
  BEFORE CHMAC
  ON "Проверка"
  FOR EACH STATEMENT
  EXECUTE PROCEDURE "ТрФункCHMAC3"();
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами вызывающего пользователя
CREATE FUNCTION "ТрФункCHMAC4"() RETURNS trigger LANGUAGE plpgsql AS $$
BEGIN
  INSERT INTO "Проверка_CHMAC"(info) VALUES('-ASCHMAC-' || USER);
  RETURN NEW;
END;$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФункCHMAC4" () TO PUBLIC;
--
-- Создаем триггер
CREATE TRIGGER "ТриггерASCHMAC"
  AFTER CHMAC
  ON "Проверка"
  FOR EACH STATEMENT
  EXECUTE PROCEDURE "ТрФункCHMAC4"();
--
-- Проверяем модификацию данных в таблице без признака MAC CCR
-- Пользователи меняют данные
-- Ожидается, что метки записей не изменятся
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_2_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем модификацию данных администратором
-- и пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
\connect - postgres
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что обновление данных никак не повлияло на их зашиту
-- Ожидается, что суперюзер postgres и пользователь dbsa с флагами игнорирования МПРД
-- CAP_IGNMACLVL и CAP_IGNMACCAT смогли обновить все записи,
-- остальные пользователи смогли обновить только записи своего уровня,
-- т.е. не смогли осуществить запись выше своего уровня или ниже
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем модификацию метки пользователем без CHMAC
-- ожидается ошибка доступа
\connect - u_1_11
CHMAC "Проверка" SET maclabel = '{1,2}' WHERE maclabel > '{0,0}';
--
-- Проверяем модификацию метки пользователем с флагом CHMAC
-- ожидается успех
\connect - dbsa
EXPLAIN VERBOSE
CHMAC "Проверка" SET maclabel = '{1,2}' WHERE maclabel > '{0,0}';
--
CHMAC "Проверка" SET maclabel = '{1,2}' WHERE maclabel > '{0,0}';
--
-- Проверяем
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем
SELECT maclabel,* FROM "Проверка_CHMAC" ORDER BY info;
--
-- Создаем представление
CREATE VIEW "ПроверкаVIEW" AS SELECT maclabel, "идентификатор", "данные" FROM "Проверка" ORDER BY "идентификатор";
--
SELECT * FROM "ПроверкаVIEW" ORDER BY "идентификатор";
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами вызывающего пользователя
CREATE FUNCTION "ТрФункCHMAC5"() RETURNS trigger LANGUAGE plpgsql AS $$
BEGIN
  INSERT INTO "Проверка_CHMAC"(info) VALUES(NEW."идентификатор" || '-IRCHMAC-' || USER);
  RETURN NEW;
END;$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФункCHMAC5" () TO PUBLIC;
--
-- Создаем триггер
CREATE TRIGGER "ТриггерIRCHMAC"
  INSTEAD OF CHMAC
  ON "ПроверкаVIEW"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункCHMAC5"();
--
-- Проверяем работу триггера instead
CHMAC "ПроверкаVIEW" SET maclabel = '{1,0}' WHERE maclabel > '{0,0}';
--
-- Проверяем
SELECT * FROM "ПроверкаVIEW" ORDER BY "идентификатор";
--
-- Проверяем
SELECT maclabel,* FROM "Проверка_CHMAC" ORDER BY info;
--
-- Удаляем триггер
DROP TRIGGER "ТриггерIRCHMAC" ON "ПроверкаVIEW";
--
-- Создаем правило CHMAC ALSO
\connect - postgres
CREATE RULE CHMAC_ALSO AS ON CHMAC TO "Проверка"
WHERE OLD."идентификатор" > 50 DO ALSO
INSERT INTO "Проверка_CHMAC"(info) VALUES(NEW."идентификатор" || '-ALSO RULE-' || USER);
--
-- Проверяем модификацию метки
\connect - dbsa
CHMAC "Проверка" SET maclabel = '{1,1}' WHERE maclabel > '{0,0}';
--
-- Проверяем
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем
SELECT maclabel,* FROM "Проверка_CHMAC" ORDER BY info;
--
-- Создаем правило CHMAC INSTEAD
CREATE RULE CHMAC_INSTEAD AS ON CHMAC TO "ПроверкаVIEW"
DO INSTEAD
INSERT INTO "Проверка_CHMAC"(info) VALUES(NEW."идентификатор" || '-INSTEAD RULE-' || USER);
--
-- Проверяем модификацию метки
CHMAC "ПроверкаVIEW" SET maclabel = '{1,0}' WHERE maclabel > '{0,0}';
--
-- Проверяем
SELECT * FROM "ПроверкаVIEW" ORDER BY "идентификатор";
--
-- Проверяем
SELECT maclabel,* FROM "Проверка_CHMAC" ORDER BY info;
--
-- Проверяем функцию \d+
\d+ "Проверка"
--
\d+ "ПроверкаVIEW"
--
-- Проверяем корректность dump
\! /usr/lib/postgresql/9.6/bin/pg_dump -U postgres mtest > ~/mac.out
--
-- Просматриваем результат dump
\! cat ~/mac.out
--
-- Удаляем
\! rm -f ~/mac.out
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'