-- RBT-TEST - COPY TO <файл> WITH MACS
-- Проверка работы МПРД при использовании команды COPY TO <файл> WITH MACS
-- при установленном параметре разрешения работы команды COPY с файлами ac_enable_copy_to_file
-- Перед выполнением теста папка /tmp_test/ должна быть очищена от файлов test*.data, создаваемых тестом
-- Для возможности выполнения теста, с помощью visudo для тестирующего пользователя должна быть добавлена строчка "{testuser}   ALL=NOPASSWD: /bin/rm
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
\i 'support/init-config-tools.sql'
\i 'support/enable-copy-to-file.sql'
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
\i 'support/select-test-table.sql'
--
-- Удаляем временные файлы
\! sudo rm -f /tmp_test/test*.data
--
-- Подключение пользователя u_0_00 c меткой {0,0}
\connect - u_0_00
--
-- Попытка выполнить команду COPY в файл не администратором
-- Ожидается ошибка доступа
COPY "Проверка" TO '/tmp_test/test.data';
--
-- Попытка выполнить команду COPY в файл администратором
-- Ожидается успех
\connect - postgres
COPY "Проверка" TO '/tmp_test/test.data';
--
-- Просматриваем результирующий файл
-- Ожидаются все 10 строк, при этом метки не сохраняются
\! cat "/tmp_test/test.data"
--
-- Попытка выполнить команду COPY с выводом меток в файл
-- Ожидается успех
-- При отсутвии привилегии установки метки у postgres ожидаются предупреждения
COPY "Проверка" TO '/tmp_test/test_wm.data' WITH MACS;
--
-- Просматриваем результирующий файл
-- Ожидаются все 10 строк, с соответствующими метками
\! cat "/tmp_test/test_wm.data"
--
-- Так же делаем попытку вывода в бинарном формате
-- Попытка выполнить команду COPY в бинарном формате
-- Ожидается успех
COPY "Проверка" TO '/tmp_test/test_b.data' BINARY;
--
-- Просматриваем результирующий файл
-- Ожидаются все строки в бинарном формате
-- Поскольку бинарный вывод команды COPY платформозависим, 
-- содержимое файла не проверяем командой cat, 
-- а делаем это позже с помощью заполнения таблицы
-- \! echo -e | cat "/tmp_test/test_b.data" -
--
-- Попытка выполнить команду COPY с выводом меток в файл в бинарном формате
-- Ожидается успех
-- При отсутвии привилегии установки метки у postgres ожидаются предупреждения
COPY "Проверка" TO '/tmp_test/test_b_wm.data' WITH MACS BINARY;
--
-- Просматриваем результирующий файл
-- Ожидаются все строки в бинарном формате с метками
-- Поскольку бинарный вывод команды COPY платформозависим, 
-- содержимое файла не проверяем командой cat, 
-- а делаем это позже с помощью заполнения таблицы
-- \! echo -e | cat "/tmp_test/test_b_wm.data" -
--
-- Проверяем права доступа и метки получившихся файлов
-- В случае присутствия привилегии установки метки у администратора ожидается, что метка будет максимальной
\! pdp-ls -Mn /tmp_test/test*.data | awk '{ print $1, $5, $6}'
--
-- Очищаем таблицу
DELETE FROM "Проверка";
--
-- Подключение пользователя u_2_11 c меткой {2,3}
\connect - u_2_11
--
-- Пытаемся заполнить таблицу из файла
-- Ожидается ошибка доступа
COPY "Проверка" FROM '/tmp_test/test.data';
--
-- Подключается суперюзер postgres
\connect - postgres
--
-- Пытаемся заполнить таблицу из файла без меток
-- Ожидается успех
COPY "Проверка" FROM '/tmp_test/test.data';
--
-- Проверяем содержимое таблицы
-- Ожидаются 10 строк, метки соответствуют метке пользователя, т.е. {0,0}
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Очищаем таблицу
DELETE FROM "Проверка";
--
-- Пытаемся заполнить таблицу из файла с метками без меток
-- Ожидается ошибка формата файла
COPY "Проверка" FROM '/tmp_test/test_wm.data';
--
-- Проверяем содержимое таблицы
-- Ожидается, что загрузка не была осуществлена
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Пытаемся заполнить таблицу из файла с метками
COPY "Проверка" FROM '/tmp_test/test_wm.data' WITH MACS;
--
-- Проверяем содержимое таблицы
-- Ожидаются 10 строк, метки должны восстановится корректно
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем, что метки загрузились корректно
\i 'support/select-test-table.sql'
--
-- Подключается суперюзер postgres
\connect - postgres
--
-- Очищаем таблицу
DELETE FROM "Проверка";
--
-- Пытаемся заполнить таблицу из бинарного файла без меток
-- Ожидается успех
COPY "Проверка" FROM '/tmp_test/test_b.data' BINARY;
--
-- Проверяем содержимое таблицы
-- Ожидаются 10 строк, метки соответствуют метке пользователя, т.е. {0,0}
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Очищаем таблицу
DELETE FROM "Проверка";
--
-- Пытаемся заполнить таблицу из бинарного файла с метками без меток
COPY "Проверка" FROM '/tmp_test/test_b_wm.data' BINARY;
--
-- Проверяем содержимое таблицы
-- Ожидаются 10 строк, метки соответствуют метке пользователя, т.е. {0,0}
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Очищаем таблицу
DELETE FROM "Проверка";
--
-- Пытаемся заполнить таблицу из бинарного файла с метками
COPY "Проверка" FROM '/tmp_test/test_b_wm.data' WITH MACS BINARY;
--
-- Проверяем содержимое таблицы
-- Ожидаются 10 строк, метки должны восстановится корректно
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем, что метки загрузились корректно
\i 'support/select-test-table.sql'
--
-- Удаляем временные файлы
\! sudo rm -f /tmp_test/test*.data
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'