-- RBT-TEST - CREATE TABLE AS/LIKE, SELECT INTO
-- Проверка сохранения атрибутов безопасности при выполнении команд CREATE TABLE ... AS ...
-- SELECT ... INTO ... и CREATE TABLE ... (LIKE ...)
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
\i 'support/select-test-table.sql'
--
-- Подключаемся пользователем u_1_10 с меткой {1,2}
\connect - u_1_10
--
-- Создаем таблицу на основе существующей с помощью CREATE TABLE ... AS ...
CREATE TABLE "Проверка2" AS SELECT * FROM "Проверка";
--
-- Проверяем метку созданной таблицы
-- Ожидается, что метка созданной таблицы соответствует метке пользователя {1,2}
-- и записи таблицы незащищены (меток не имеют)
-- признак MAC CCR установлен по умолчанию
SELECT relname AS "Наименование таблицы", maclabel AS "Метка",
       relhasmacs AS "Записи имеют метки", relmacccr AS "MAC CCR"
FROM pg_class WHERE oid = 'Проверка2'::regclass::oid;
--
-- Проверяем содержимое таблицы
-- Ожидается 5 записей, в таблицу попали только доступные по МПРД
SELECT * FROM "Проверка2" ORDER BY "идентификатор";
--
-- Создаем таблицу с защищенными строками на основе существующей
CREATE TABLE "Проверка3" WITH (MACS=TRUE) AS SELECT * FROM "Проверка";
--
-- Проверяем метку созданной таблицы
-- Ожидается, что метка созданной таблицы соответствует метке пользователя {1,2}
-- и записи таблицы защищены (имеют метки)
-- признак MAC CCR установлен по умолчанию
SELECT relname AS "Наименование таблицы", maclabel AS "Метка",
       relhasmacs AS "Записи имеют метки", relmacccr AS "MAC CCR"
FROM pg_class WHERE oid = 'Проверка3'::regclass::oid;
--
-- Проверяем содержимое таблицы
-- Ожидается, что в таблицу попали только доступные по МПРД записи, с метками пользователя {1,2}
SELECT maclabel, * FROM "Проверка3" ORDER BY "идентификатор";
--
-- Создаем таблицу с защищенными строками на основе существующей
CREATE TABLE "Проверка4" WITH (MACS=TRUE) AS SELECT * FROM "Проверка2";
--
-- Проверяем метку созданной таблицы
-- Ожидается, что метка созданной таблицы соответствует метке пользователя {1,2}
-- и записи таблицы защищены (имеют метки)
-- признак MAC CCR установлен по умолчанию
SELECT relname AS "Наименование таблицы", maclabel AS "Метка",
       relhasmacs AS "Записи имеют метки", relmacccr AS "MAC CCR"
FROM pg_class WHERE oid = 'Проверка4'::regclass::oid;
--
-- Проверяем содержимое таблицы
-- Ожидается, что в таблицу попали только доступные по МПРД записи, с метками пользователя {1,2}
SELECT maclabel, * FROM "Проверка4" ORDER BY "идентификатор";
--
-- Подключаемся пользователем dbsa с привилегией CHMAC
\connect - dbsa
--
-- Создаем таблицу с защищенными строками на основе существующей
CREATE TABLE "Проверка5" WITH (MACS=TRUE) AS SELECT * FROM "Проверка";
--
-- Проверяем метку созданной таблицы
-- Ожидается, что метка созданной таблицы соответствует метке пользователя dbsa
-- и записи таблицы защищены (имеют метки)
-- признак MAC CCR установлен по умолчанию
SELECT relname AS "Наименование таблицы", maclabel AS "Метка",
       relhasmacs AS "Записи имеют метки", relmacccr AS "MAC CCR"
FROM pg_class WHERE oid = 'Проверка5'::regclass::oid;
--
-- Проверяем содержимое таблицы
-- Ожидается, что в таблицу попали записи c исходными метками
SELECT maclabel, * FROM "Проверка5" ORDER BY "идентификатор";
--
-- Меняем уровень сессии
SET ac_session_maclabel = '{1, 3}';
--
-- Создаем таблицу с защищенными строками на основе существующей
-- Ожидается ошибка, так как запрос содержит данные с меткой выше чем
-- у создаваемой таблицы
CREATE TABLE "Проверка6" WITH (MACS=TRUE) AS SELECT * FROM "Проверка";
--
-- Подключаемся пользователем u_1_10 с меткой {1,2}
\connect - u_1_10
--
-- Создаем таблицу на основе существующей с помощью SELECT ... INTO ...
SELECT *  INTO "Проверка7" FROM "Проверка";
--
-- Проверяем метку созданной таблицы
-- Ожидается, что метка созданной таблицы соответствует метке пользователя {1,2}
-- и записи таблицы незащищены (меток не имеют)
-- признак MAC CCR установлен по умолчанию
SELECT relname AS "Наименование таблицы", maclabel AS "Метка",
       relhasmacs AS "Записи имеют метки", relmacccr AS "MAC CCR"
FROM pg_class WHERE oid = 'Проверка7'::regclass::oid;
--
-- Проверяем содержимое таблицы
-- Ожидается 5 записей, в таблицу попали только доступные по МПРД
SELECT * FROM "Проверка7" ORDER BY "идентификатор";
--
-- Создаем таблицу на основе существующей с помощью CREATE ... (LIKE ...)
CREATE TABLE "Проверка8" (LIKE "Проверка");
--
-- Проверяем метку созданной таблицы
-- Ожидается, что метка созданной таблицы соответствует метке пользователя {1,2}
-- и записи таблицы незащищены (меток не имеют)
-- признак MAC CCR установлен по умолчанию
SELECT relname AS "Наименование таблицы", maclabel AS "Метка",
       relhasmacs AS "Записи имеют метки", relmacccr AS "MAC CCR"
FROM pg_class WHERE oid = 'Проверка8'::regclass::oid;
--
--
-- Создаем таблицу с защищенными строками на основе существующей с помощью CREATE ... (LIKE ...)
CREATE TABLE "Проверка9" (LIKE "Проверка") WITH(MACS=TRUE);
--
-- Проверяем метку созданной таблицы
-- Ожидается, что метка созданной таблицы соответствует метке пользователя {1,2}
-- и записи таблицы защищены (имеют метки)
-- признак MAC CCR установлен по умолчанию
SELECT relname AS "Наименование таблицы", maclabel AS "Метка",
       relhasmacs AS "Записи имеют метки", relmacccr AS "MAC CCR"
FROM pg_class WHERE oid = 'Проверка9'::regclass::oid;
--
-- Подключаемся пользователем dbsa с привилегией CHMAC
\connect - dbsa
--
-- Создаем таблицу с защищенными строками на основе существующей с помощью CREATE ... (LIKE ...)
CREATE TABLE "Проверка10" (LIKE "Проверка") WITH(MACS=TRUE);
--
-- Проверяем метку созданной таблицы
-- Ожидается, что метка созданной таблицы соответствует метке пользователя dbsa
-- и записи таблицы защищены (имеют метки)
-- признак MAC CCR установлен по умолчанию
SELECT relname AS "Наименование таблицы", maclabel AS "Метка",
       relhasmacs AS "Записи имеют метки", relmacccr AS "MAC CCR"
FROM pg_class WHERE oid = 'Проверка10'::regclass::oid;
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'