-- RBT-TEST - МПРД для модуля dblink
-- Тест написан для основного сервера, работающем на порту 6000 и для удаленного, работающем на 6001
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
\i 'support/init-config-tools.sql'
-- Подключаемся к удаленному кластеру
\connect postgres postgres - 6001
-- Выполняем настройку кластера
\i 'support/init.sql'
\i 'support/create-test-table.sql'
\i 'support/init-mac.sql'
--
-- Устанавливаем мандатные атрибуты созданной таблицы
MAC LABEL ON TABLE "Проверка" IS '{2,3}';
MAC CCR ON TABLE "Проверка" IS OFF;
--
-- Заполняем таблицу данными
INSERT INTO "Проверка" VALUES (1, 'Запись с меткой {0,0}');
INSERT INTO "Проверка" VALUES (2, 'Запись с меткой {0,1}');
INSERT INTO "Проверка" VALUES (3, 'Запись с меткой {1,1}');
INSERT INTO "Проверка" VALUES (4, 'Запись с меткой {0,2}');
INSERT INTO "Проверка" VALUES (5, 'Запись с меткой {1,2}');
INSERT INTO "Проверка" VALUES (6, 'Запись с меткой {1,3}');
INSERT INTO "Проверка" VALUES (7, 'Запись с меткой {2,3}');
-- Изменяем мандатные метки записей таблицы
CHMAC "Проверка" SET maclabel = '{0,0}' WHERE "идентификатор" = 1;
CHMAC "Проверка" SET maclabel = '{0,1}' WHERE "идентификатор" = 2;
CHMAC "Проверка" SET maclabel = '{1,1}' WHERE "идентификатор" = 3;
CHMAC "Проверка" SET maclabel = '{0,2}' WHERE "идентификатор" = 4;
CHMAC "Проверка" SET maclabel = '{1,2}' WHERE "идентификатор" = 5;
CHMAC "Проверка" SET maclabel = '{1,3}' WHERE "идентификатор" = 6;
CHMAC "Проверка" SET maclabel = '{2,3}' WHERE "идентификатор" = 7;
--
-- Выдаём все дискреционные права на таблицу 'Проверка' пользователю 'PUBLIC'
GRANT ALL ON "Проверка" TO PUBLIC;
-- Устанавливаем пароль пользователям
ALTER USER u_1_11 PASSWORD '123456';
ALTER USER u_2_11 PASSWORD '123456';
ALTER USER dbsa PASSWORD '123456';
--
-- Удостоверимся, что в таблице содержится 7 записей с различными метками
-- Ожидается 7 записей
SELECT maclabel, * FROM "Проверка";
--
--
-- На основном сервере создаем расширение dblink
\connect mtest postgres - 6000
-- Регистрируем расширение dblink
-- Ожидается успех
CREATE EXTENSION dblink;
-- Выдаём права на исполнение пользователю 'PUBLIC'
GRANT EXECUTE ON FUNCTION dblink_connect_u(TEXT) TO PUBLIC;
GRANT EXECUTE ON FUNCTION dblink_connect_u(TEXT, TEXT) TO PUBLIC;
GRANT EXECUTE ON FUNCTION dblink(TEXT) TO PUBLIC;
GRANT EXECUTE ON FUNCTION dblink_exec(TEXT) TO PUBLIC;
GRANT EXECUTE ON FUNCTION dblink_open(TEXT, TEXT) TO PUBLIC;
GRANT EXECUTE ON FUNCTION dblink_fetch(TEXT, INTEGER) TO PUBLIC;
GRANT EXECUTE ON FUNCTION dblink_close(TEXT) TO PUBLIC;
GRANT EXECUTE ON FUNCTION dblink_send_query(TEXT, TEXT) TO PUBLIC;
GRANT EXECUTE ON FUNCTION dblink_disconnect() TO PUBLIC;
GRANT EXECUTE ON FUNCTION dblink_disconnect(TEXT) TO PUBLIC;
--
--
-- Подключаемся как пользователь u_1_11
\connect - u_1_11
-- Просматриваем метку пользователя
-- Ожидается {1,3}
SELECT session_maclabel;
-- Пробуем подключиться к удаленной базе данных по dblink как пользователь u_2_11 (с уровнем выше)
-- Ожидается ошибка доступа
SELECT dblink_connect_u('dbname = mtest user = u_2_11 host = 127.0.0.1 port = 6001 password = 123456');
-- Пробуем подключиться к удаленной базе данных по dblink как пользователь dbsa (с игнорированием МРД)
-- Ожидается ошибка доступа,
-- т.к. текущие привилегии пользователя (u_1_11) не совпадают
-- с привилегиями пользователя (dbsa) на удаленном сервере
SELECT dblink_connect_u('dbname = mtest user = dbsa host = 127.0.0.1 port = 6001 password = 123456');
--
-- Подключаемся как пользователь u_2_11
\connect - u_2_11
-- Просматриваем метку пользователя
-- Ожидается {2,3}
SELECT session_maclabel;
-- Пробуем подключиться к удаленной базе данных по dblink как пользователь u_1_11 (с уровнем ниже)
-- Ожидается ошибка доступа
SELECT dblink_connect_u('dbname = mtest user = u_1_11 host = 127.0.0.1 port = 6001 password = 123456');
-- Подключаемся к БД по dblink как пользователь u_2_11 (с совпадающей меткой)
-- Ожидается успех
SELECT dblink_connect_u('dbname = mtest user = u_2_11 host = 127.0.0.1 port = 6001 password = 123456');
-- Убеждаемся в том, что метка сессии dblink совпадает с меткой пользователя - {2,3}
SELECT * FROM dblink('SELECT session_maclabel;')  AS "Мандатная метка сессии" ("Мандатная метка" TEXT);
-- Проверка работу функции dblink_exec() -- исполнение запроса в удалённой базе данных
-- Ожидается ошибка доступа,
-- т.к. для использования этой функции необходимо включить ac_enable_dblink_mac
SELECT dblink_exec('INSERT INTO "Проверка" VALUES(8, ''Новая запись пользователя u_2_11'');');
-- Закрывем подключение
-- Ожидается успех
SELECT dblink_disconnect();
--
-- Включаем возможность использовать non-SELECT запросы удаленно
\i 'support/enable-dblink-mac.sql'
--
-- Подключаемся как пользователь u_2_11
\connect - u_2_11
-- Пробуем подключиться к удаленной базе данных по dblink
-- Ожидается успех
SELECT dblink_connect_u('dbname = mtest user = u_2_11 host = 127.0.0.1 port = 6001 password = 123456');
-- Пробуем удаленно выполнить вставку данных в таблицу
-- Ожидается успех
SELECT dblink_exec('INSERT INTO "Проверка" VALUES(8, ''Новая запись пользователя u_2_11'');');
-- Закрывем подключение
-- Ожидается успех
SELECT dblink_disconnect();
-- Выключаем возможность использовать non-SELECT запросы удаленно
\i 'support/disable-dblink-mac.sql'
--
\connect - u_2_11
-- Пробуем подключиться к удаленной базе данных по dblink
-- Ожидается успех
SELECT dblink_connect_u('dbname = mtest user = u_2_11 host = 127.0.0.1 port = 6001 password = 123456');
-- Проверяем работу функции dblink_open() -- открытие курсора
SELECT dblink_open('cursor', 'SELECT maclabel, * FROM "Проверка";');
-- Получаем данные из таблицы 'Проверка'
-- Ожидается 8 записей, 8-я имеет метку {2,3} -- добавленную с помощью dblink_exec()
SELECT * FROM dblink_fetch('cursor', 8) AS ("Метка записи" maclabel, "Идентификатор" INTEGER, "Данные" TEXT);
-- Закрываем курсор
-- Ожидается успех
SELECT dblink_close('cursor');
-- Закрываем анонимное соединение
-- Ожидается успех
SELECT dblink_disconnect();
-- Открываем именованное соединение
-- Ожидается успех
SELECT dblink_connect_u('connection', 'dbname = mtest user = u_2_11 host = 127.0.0.1 port = 6001 password = 123456');
-- Выполняем асинхронную команду
-- Ожидается 1 -- код успешной операции
SELECT dblink_send_query('connection', 'SELECT pg_sleep(1) || ''успех'';');
-- Закрываем именованное подключение
SELECT dblink_disconnect('connection');
--
-- Подключаемся к основному кластеру и проводим очистку
\connect postgres postgres - 6000
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'
-- Подключаемся к удаленному кластеру и проводим очистку
\connect postgres postgres - 6001
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'