-- RBT-TEST - МПРД при удалении (DELETE)
-- Проверка работы МПРД при удалении данных командой DELETE
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
\i 'support/select-test-table.sql'
--
--
-- Проверяем удаление данных в таблице без признака MAC CCR
-- ожидается удаление только строк с меткой, равной метке пользователя
--
-- После удаления пользователем с меткой {0,0} ожидается 8 записей
\connect - u_0_00
DELETE FROM "Проверка";
\connect - postgres
SELECT * FROM "Проверка" ORDER BY "идентификатор";
--
-- После удаления пользователем с меткой {2,3} ожидается 7 записей
\connect - u_2_11
DELETE FROM "Проверка";
\connect - postgres
SELECT * FROM "Проверка" ORDER BY "идентификатор";
--
-- После удаления пользователем с dbsa с установленными флагами игнорирования МПРД
-- ожидается 0 записей
\connect - dbsa
DELETE FROM "Проверка";
\connect - postgres
SELECT * FROM "Проверка" ORDER BY "идентификатор";
--
--
-- Проверяем удаление данных в таблице с признаком MAC CCR
-- Создаем от имени пользователя u_1_01 таблицу с меткой {1, 1}
\connect - u_1_01
CREATE TABLE "Схема_1_01"."Проверка_1_01" ("идентификатор" INTEGER PRIMARY KEY, "данные" TEXT) WITH (MACS=TRUE);
GRANT ALL ON "Схема_1_01"."Проверка_1_01" TO PUBLIC;
INSERT INTO "Схема_1_01"."Проверка_1_01" VALUES (1, USER || '-init');
INSERT INTO "Схема_1_01"."Проверка_1_01" VALUES (2, USER || '-init');
INSERT INTO "Схема_1_01"."Проверка_1_01" VALUES (3, USER || '-init');
SELECT * FROM "Схема_1_01"."Проверка_1_01" ORDER BY "идентификатор";
--
-- Проверка возможности удаления пользователями с разным уровнем
-- Попытка удаления пользователем u_0_00 с меткой {0,0}
-- Ожидается ошибка доступа
\connect - u_0_00
DELETE FROM "Схема_1_01"."Проверка_1_01";
--
-- Попытка удаления пользователем u_0_01 с меткой {0,1}
-- Ожидается ошибка доступа
\connect - u_0_01
DELETE FROM "Схема_1_01"."Проверка_1_01";
--
-- Попытка удаления пользователем u_1_01 с меткой {1,1}
-- Ожидается успех
\connect - u_1_01
DELETE FROM "Схема_1_01"."Проверка_1_01" WHERE "идентификатор" = 1;
--
-- Попытка удаления пользователем u_0_10 с меткой {0,2}
-- Ожидается ошибка доступа
\connect - u_0_10
DELETE FROM "Схема_1_01"."Проверка_1_01";
--
-- Попытка удаления пользователем u_1_10 с меткой {1,2}
-- Ожидается ошибка доступа
\connect - u_1_10
DELETE FROM "Схема_1_01"."Проверка_1_01";
--
-- Попытка удаления пользователем u_1_11 с меткой {1,3}
-- Ожидается ошибка доступа
\connect - u_1_11
DELETE FROM "Схема_1_01"."Проверка_1_01";
--
-- Попытка удаления пользователем u_2_11 с меткой {2,3}
-- Ожидается ошибка доступа
\connect - u_2_11
DELETE FROM "Схема_1_01"."Проверка_1_01";
--
-- Проверяем что были удалена одна запись
-- Ожидается 2 записи
\connect - postgres
SELECT * FROM "Схема_1_01"."Проверка_1_01" ORDER BY "идентификатор";
--
-- Попытка удаления пользователем dbsa с флагами игнорирования МПРД
-- Ожидается успех
\connect - dbsa
DELETE FROM "Схема_1_01"."Проверка_1_01" WHERE "идентификатор" = 2;
--
-- Ожидается 1 запись
\connect - postgres
SELECT * FROM "Схема_1_01"."Проверка_1_01" ORDER BY "идентификатор";

--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'