-- RBT-TEST - МПРД для внешних баз данных
-- Тест написан для основного сервера, работающем на порту 6000 и для удаленного, работающем на 6001
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
\i 'support/init-config-tools.sql'
-- Подключаемся к удаленному кластеру
\connect postgres postgres - 6001
-- Выполняем настройку кластера
\i 'support/init.sql'
\i 'support/create-test-table.sql'
\i 'support/init-mac.sql'
--
-- Устанавливаем мандатные атрибуты созданной таблицы
MAC LABEL ON TABLE "Проверка" IS '{2,3}';
MAC CCR ON TABLE "Проверка" IS OFF;
--
-- Заполняем таблицу данными
INSERT INTO "Проверка" VALUES (1, 'Запись с меткой {0,0}');
INSERT INTO "Проверка" VALUES (2, 'Запись с меткой {0,1}');
INSERT INTO "Проверка" VALUES (3, 'Запись с меткой {1,1}');
INSERT INTO "Проверка" VALUES (4, 'Запись с меткой {0,2}');
INSERT INTO "Проверка" VALUES (5, 'Запись с меткой {1,2}');
INSERT INTO "Проверка" VALUES (6, 'Запись с меткой {1,3}');
INSERT INTO "Проверка" VALUES (7, 'Запись с меткой {2,3}');
-- Изменяем мандатные метки записей таблицы
CHMAC "Проверка" SET maclabel = '{0,0}' WHERE "идентификатор" = 1;
CHMAC "Проверка" SET maclabel = '{0,1}' WHERE "идентификатор" = 2;
CHMAC "Проверка" SET maclabel = '{1,1}' WHERE "идентификатор" = 3;
CHMAC "Проверка" SET maclabel = '{0,2}' WHERE "идентификатор" = 4;
CHMAC "Проверка" SET maclabel = '{1,2}' WHERE "идентификатор" = 5;
CHMAC "Проверка" SET maclabel = '{1,3}' WHERE "идентификатор" = 6;
CHMAC "Проверка" SET maclabel = '{2,3}' WHERE "идентификатор" = 7;
--
-- Выдаём все дискреционные права на таблицу 'Проверка' пользователю 'PUBLIC'
GRANT ALL ON "Проверка" TO PUBLIC;
-- Устанавливаем пароль пользователям
ALTER USER u_1_11 PASSWORD '123456';
ALTER USER u_2_11 PASSWORD '123456';
ALTER USER dbsa PASSWORD '123456';
--
-- Удостоверимся, что в таблице содержится 7 записей с различными метками
-- Ожидается 7 записей
SELECT maclabel, * FROM "Проверка";
--
--
\connect mtest postgres - 6000
-- На основном сервере проводим необходимые настройки
-- Регистрируем расширение postgres_fdw -- FOREIGN DATA WRAPPER для PostgreSQL
CREATE EXTENSION postgres_fdw;
-- Создаём сервер, содержащий таблицу 'Проверка'
CREATE SERVER foreign_server FOREIGN DATA WRAPPER postgres_fdw OPTIONS(host '127.0.0.1',
                                                                       port '6001',
                                                                       dbname 'mtest');
-- Выдаём права на использование сервера пользователю PUBLIC
GRANT ALL ON FOREIGN SERVER foreign_server TO PUBLIC;
--
-- Создаём сопоставление пользователей
CREATE USER MAPPING FOR u_1_11 SERVER foreign_server OPTIONS (user 'u_2_11', password '123456');
CREATE USER MAPPING FOR u_2_11 SERVER foreign_server OPTIONS (user 'u_1_11', password '123456');
-- Создаем внешнюю таблицу
-- Ожидается успех
CREATE FOREIGN TABLE "Внешняя_таблица"("идентификатор" INTEGER, "данные" TEXT) SERVER foreign_server OPTIONS (table_name 'Проверка');
-- Выдаем права на таблицу пользователю public
GRANT ALL ON "Внешняя_таблица" TO PUBLIC;
--
--
-- Заходим как пользователь u_1_11
\connect - u_1_11

-- Просматриваем записи таблицы
-- Ожидается ошибка доступа, т.к. произведена попытка
-- работы с внешними таблицами под уровнем
SELECT * FROM "Внешняя_таблица";
--
-- Включаем работу dblink под уровнем
\i 'support/enable-dblink-mac.sql'
--
-- Заходим как пользователь u_1_11
\connect - u_1_11
-- Просматриваем записи таблицы
-- Ожидается ошибка, связанная с невозможностью установки метки сессии
-- (т.к. было установлено соответствие u_1_11 -> u_2_11 ({1,3} -> {2,3}))
SELECT * FROM "Внешняя_таблица";
--
-- Выдаем все права на внешнюю таблицу пользователю public
\connect - postgres
GRANT ALL ON "Внешняя_таблица" TO PUBLIC;
--
-- Подключаемся как пользователь u_2_11
\connect - u_2_11
-- Просматриваем записи таблицы
-- Ожидается ошибка доступа, т.к. произведена попытка
-- работы с внешними таблицами под уровнем
-- (т.к. было установлено соответствие u_2_11 -> u_1_11 ({2,3} -> {1,3}))
SELECT * FROM "Внешняя_таблица";
--
\connect - postgres
-- Удаляем сопоставления пользователей
DROP USER MAPPING FOR u_1_11 SERVER foreign_server;
DROP USER MAPPING FOR u_2_11 SERVER foreign_server;
-- Создаем новые сопоставления пользователей
CREATE USER MAPPING FOR u_1_11 SERVER foreign_server OPTIONS (user 'dbsa',   password '123456');
CREATE USER MAPPING FOR u_2_11 SERVER foreign_server OPTIONS (user 'u_2_11', password '123456');
-- Подключаемся как пользователь u_1_11
\connect - u_1_11
-- Просматриваем записи таблицы
-- Ожидается ошибка доступа,
-- т.к. указаны пользователи при создании
-- USER MAPPING с несовпадающими привилегиями
SELECT * FROM "Внешняя_таблица";
--
-- Подключаемся как пользователь postgres
\connect - postgres
-- Удаляем соответствие пользователей для u_1_11
DROP USER MAPPING FOR u_1_11 SERVER foreign_server;
-- Создаем соответстиве пользователей для u_1_11
CREATE USER MAPPING FOR u_1_11 SERVER foreign_server OPTIONS (user 'u_1_11', password '123456');
--
-- Подключаемся как пользователь u_1_11
\connect - u_1_11
-- Просматриваем метку внешней таблицы
-- Ожидается {0,0}
SELECT maclabel FROM pg_class WHERE relname = 'Внешняя_таблица';
-- Просматриваем записи таблицы
-- Ожидается ошибка доступа,
-- т.к. метка внешней таблицы должна совпадать с меткой удаленной таблицы
SELECT * FROM "Внешняя_таблица";
--
-- Подключаемся как пользователь postgres
\connect - postgres
-- Устанавливаем метку на внешнюю таблицу
MAC LABEL ON FOREIGN TABLE "Внешняя_таблица" IS '{2,3}';
--
-- Подключаемся как пользователь u_2_11
\connect - u_2_11
-- Просматриваем метку внешней таблицы
-- Ожидается {2,3}
SELECT maclabel FROM pg_class WHERE relname = 'Внешняя_таблица';
-- Просматриваем записи таблицы
-- Ожидается 7 записей
SELECT * FROM "Внешняя_таблица";
--
-- Выполняем попытку вставки данных (INSERT)
-- Ожидается успех
INSERT INTO "Внешняя_таблица" VALUES(8, 'Новая запись с меткой {2,3}');
-- Выполняем попытку обновления данных (UPDATE)
-- Ожидается успех
UPDATE "Внешняя_таблица" SET "данные" = 'Обновленная запись' WHERE "идентификатор" = 7;
-- Просматриваем таблицу
-- Ожидается 8 записей, запись с идентификатором 7
-- обновлена
SELECT * FROM "Внешняя_таблица";
--
-- Подключаемся к удаленному серверу
\connect - postgres - 6001
-- Просматриваем записи таблицы
-- Ожидается 8 записей
-- Запись с идентификатором 7 обновлена.
SELECT maclabel, * FROM "Проверка";
--
-- Подключаемся как пользователь postgres
\connect - postgres - 6000
-- Удаляем внешнюю таблицу
DROP FOREIGN TABLE "Внешняя_таблица";
--
-- Выключаем работу dblink под уровнем
\i 'support/disable-dblink-mac.sql'
--
-- Подключаемся к основному кластеру и проводим очистку
\connect postgres postgres - 6000
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'
-- Подключаемся к удаленному кластеру и проводим очистку
\connect postgres postgres - 6001
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'