-- RBT-TEST - МПРД для индексов
-- Проверка сохранения атрибутов безопасности при использовании индексов
--

\i 'support/init.sql'
\i 'support/init-mac.sql'
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
--
-- Подключаемся как пользователь postgres
\connect - postgres
--
-- Создаем проверочную таблицу без защищенных строк
CREATE TABLE "ПроверкаБезМеток"("идентификатор" INTEGER PRIMARY KEY, "данные" TEXT);
MAC LABEL ON TABLE "ПроверкаБезМеток" IS '{1,2}';
GRANT ALL ON "ПроверкаБезМеток" TO PUBLIC;
--
--
-- 1. Проверка базовых возможностей
--
--
-- Проверка создания индексов с опциями MACS с использованием
-- метода доступа btree для таблицы с защищенными записейами
-- Ожидается успех
CREATE INDEX "Индекс1" ON "Проверка" USING btree("идентификатор");
CREATE INDEX "Индекс2" ON "Проверка" USING btree("идентификатор") WITH (MACS=FALSE);
CREATE INDEX "Индекс3" ON "Проверка" USING btree("идентификатор") WITH (MACS=TRUE);
CREATE INDEX "Индекс4" ON "Проверка" USING btree("идентификатор") WITH (MACS=TRUE, fillfactor=10);
CREATE INDEX "Индекс5" ON "Проверка" USING btree("идентификатор", "данные") WITH (MACS=TRUE);
--
-- Проверяем мандатные атрибуты созданных индексов
-- Ожидается 5 записей
-- Индексы Индекс3, Индекс4, Индекс5 будут иметь relhasmacs=t
-- т.к. созданы с опциями MACS=TRUE,
-- и все индексы будут иметь relmacccr=f,
-- т.к. будут наследовать его с таблицы Проверка
-- а мандатная метка будет соответствовать метке таблицы Проверка -- {2,3}
SELECT maclabel, relname, relhasmacs, relmacccr FROM pg_class WHERE relname LIKE 'Индекс%' ORDER BY oid;
--
-- Просматриваем определение индексов
-- Ожидается 5 записей
-- Индексы Индекс3, Индекс4, Индекс5 будут иметь опцию macs='true' в столбце indexname
-- а мандатная метка будет соответствовать метке таблицы Проверка -- {2,3}
SELECT maclabel, indexname, indexdef FROM pg_indexes WHERE indexname LIKE 'Индекс%' ORDER BY indexname;
--
-- Удаляем созданные индексы
DROP INDEX "Индекс1";
DROP INDEX "Индекс2";
DROP INDEX "Индекс3";
DROP INDEX "Индекс4";
DROP INDEX "Индекс5";
--
-- Проверка создания индексов с опциями MACS с использованием
-- метода доступа btree для таблицы без защищенных строк
-- Ожидается успех
CREATE INDEX "Индекс1" ON "ПроверкаБезМеток" USING btree("идентификатор");
CREATE INDEX "Индекс2" ON "ПроверкаБезМеток" USING btree("идентификатор") WITH (MACS=FALSE);
CREATE INDEX "Индекс3" ON "ПроверкаБезМеток" USING btree("идентификатор") WITH (MACS=FALSE, fillfactor=10);
--
-- Ожидается ошибка
-- т.к. таблица без меток не может иметь индексов с метками
CREATE INDEX "Индекс4" ON "ПроверкаБезМеток" USING btree("идентификатор") WITH (MACS=TRUE);
CREATE INDEX "Индекс5" ON "ПроверкаБезМеток" USING btree("идентификатор") WITH (MACS=TRUE, fillfactor=10);
--
-- Проверяем мандатные атрибуты созданных индексов
-- Ожидается 3 записи
-- Индексы Индекс1, Индекс2, Индекс3 будут иметь relhasmacs=f
-- т.к. созданы с опциями MACS=FALSE,
-- и все индексы будут иметь relmacccr=t,
-- т.к. будут наследовать его с таблицы ПроверкаБезМеток,
-- а мандатная метка будет соответстовать метке таблицы ПроверкаБезМеток -- {1,2}
SELECT maclabel, relname, relhasmacs, relmacccr FROM pg_class WHERE relname LIKE 'Индекс%' ORDER BY oid;
--
-- Просматриваем определение индексов
-- Ожидается 3 записи
-- Индексы Индекс1, Индекс2, Индекс3 в столбце indexdef будут содержать их SQL создания
-- мандатные метки будут соответствовать метке таблицы ПроверкаБезМеток -- {1,2}
SELECT maclabel, indexname, indexdef FROM pg_indexes WHERE indexname LIKE 'Индекс%' ORDER BY indexname;
--
-- Удаляем созданные индексы
DROP INDEX "Индекс1";
DROP INDEX "Индекс2";
DROP INDEX "Индекс3";
--
--
-- 2. Проверка изменения признака защищенности записей для таблицы
--
--
-- Создаем индекс для таблицы Проверка с метками
CREATE INDEX bt_macindex ON "Проверка" ("идентификатор") WITH (MACS=TRUE);
-- Создаем индекс для таблицы Проверка без меток
CREATE INDEX bt_index ON "Проверка" ("идентификатор") WITH (MACS=FALSE);
--
-- Просматриваем его мандатные атрибуты
-- Ожидается maclabel = {2,3}, relmacccr = f и relhasmacs = t для bt_macindex,
-- maclabel = {2,3} relmaccr = f и relhasmacs = f для bt_index
SELECT relname, maclabel, relmacccr, relhasmacs FROM pg_class
WHERE oid = 'bt_macindex'::regclass::oid OR oid = 'bt_index'::regclass::oid ORDER BY oid;
--
-- Модифицируем таблицу Проверка -- убираем столбец maclabel
ALTER TABLE "Проверка" SET WITHOUT MACS;
--
-- Просматриваем описание индексов в представлении pg_indexes
-- Ожидается, что в описании индексов не будет присутствовать опция MACS=TRUE
SELECT indexname, indexdef FROM pg_indexes WHERE indexname = 'bt_macindex' OR indexname = 'bt_index';
--
-- Просматриваем мандатные атрибуты таблицы и индекса
-- Ожидается maclabel = {2,3}, relmacccr = f и relhasmacs = f для таблицы Проверка и индексов bt_macindex bt_index
SELECT relname, maclabel, relmacccr, relhasmacs FROM pg_class
WHERE oid = 'Проверка'::regclass::oid OR oid = 'bt_macindex'::regclass::oid OR oid = 'bt_index'::regclass::oid  ORDER BY oid;
--
-- Модифицируем таблицу Проверка -- добавляем столбец maclabel
ALTER TABLE "Проверка" SET WITH MACS;
--
-- Просматриваем мандатные атрибуты таблицы и индекса
-- Ожидается maclabel = {2,3}, relmacccr = f и relhasmacs = t для таблицы Проверка,
-- Индексы bt_macindex и bt_index будут иметь maclabel = {2,3}, relmacccr = f, relhasmacs = f
SELECT relname, maclabel, relmacccr, relhasmacs FROM pg_class
WHERE oid = 'Проверка'::regclass::oid OR oid = 'bt_macindex'::regclass::oid OR oid = 'bt_index'::regclass::oid ORDER BY oid;
--
--
-- 3. Проверка модификации флага CCR для таблицы
--
--
-- Устанавливаем флаг MAC CCR для таблицы Проверка
MAC CCR ON TABLE "Проверка" IS ON;
--
-- Просматриваем мандатные атрибуты таблицы и индекса
-- Ожидается maclabel = {2,3}, relmacccr = t и relhasmacs = t для таблицы Проверка
-- Индексы bt_macindex и bt_index будут иметь maclabel = {2,3} relhasmacs = f, relmacccr = t
SELECT relname, maclabel, relmacccr, relhasmacs FROM pg_class
WHERE oid = 'Проверка'::regclass::oid OR oid = 'bt_macindex'::regclass::oid OR oid = 'bt_index'::regclass::oid ORDER BY oid;
--
-- Снимаем флаг MAC CCR для таблицы Проверка
MAC CCR ON TABLE "Проверка" IS OFF;
--
-- Просматриваем мандатные атрибуты таблицы и индекса
-- Ожидается maclabel = {2,3}, relmacccr = t и relhasmacs = t для таблицы Проверка
-- Индексы bt_macindex и bt_index будут иметь maclabel = {2,3} relhasmacs = f, relmacccr = f
SELECT relname, maclabel, relmacccr, relhasmacs FROM pg_class
WHERE oid = 'Проверка'::regclass::oid OR oid = 'bt_macindex'::regclass::oid OR oid = 'bt_index'::regclass::oid ORDER BY oid;
--
--
-- 4. Проверка автоматического обновления индексов
--
--
-- Подключаемся как postgres
\connect - postgres
-- Пересоздаем индексы bt_macindex, bt_index
DROP INDEX bt_macindex;
DROP INDEX bt_index;
--
CREATE INDEX bt_macindex ON "Проверка" ("идентификатор") WITH (MACS=TRUE);
CREATE INDEX bt_index ON "Проверка" ("идентификатор") WITH (MACS=FALSE);
--
-- Создаём расширение pageinspect
CREATE EXTENSION pageinspect;
--
-- Создаем триггер BEFORE UPDATE
CREATE FUNCTION "ТрФункUPDATE"() RETURNS trigger LANGUAGE plpgsql AS $$
BEGIN
  NEW."данные" = NEW."данные" || '-ТрФункUPDATE';
  NEW.maclabel = '{0,2}';
  RETURN NEW;
END;$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФункUPDATE" () TO PUBLIC;
--
CREATE TRIGGER "ТриггерBU"
  BEFORE UPDATE
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункUPDATE"();
--
-- Просматривем физическое представление таблицы Проверка
-- Ожидается 10 записей, метка которых t_maclabel = {2,3}, t_hasmac = t
SELECT lp, t_xmax = 0 AS last, t_ctid, t_hasmac, t_maclabel FROM heap_page_items(get_raw_page('Проверка', 0));
--
-- Просматриваем физическое представление индекса bt_macindex
-- Ожидается 10 записей, hasmac = t maclabel = {2,3}
SELECT itemoffset, ctid, hasmac, maclabel FROM bt_page_items('bt_macindex', 1);
--
-- Просматриваем физическое представление индекса bt_index
-- Ожидается 10 записей, hasmac = f maclabel -- пустое
SELECT itemoffset, ctid, hasmac, maclabel FROM bt_page_items('bt_index', 1);
--
-- Выполняем модификацию идентификатора таблицы
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" % 2 = 0;
--
-- Просматриваем содержимое таблицы Проверка
-- Ожидается 10 записей с меткой {2,3}
SELECT maclabel, * FROM "Проверка";
--
-- Просматривем физическое представление таблицы Проверка
-- Ожидается 15 записей, 10 из которых будут иметь t_maclabel = {2,3} t_hasmac = t
-- остальные 5 -- t_maclabel = {0,2} и t_hasmac = t
-- 10 записей будут иметь t_xmax != 0 (last = f) -- эти записи не будут видны в данный момент времени
SELECT lp, t_xmax = 0 AS last, t_ctid, t_hasmac, t_maclabel FROM heap_page_items(get_raw_page('Проверка', 0));
--
-- Просматриваем физическое представление индекса bt_macindex
-- Ожидается 15 записей, 10 из которых будут иметь метку {2,3},
-- остальные 5 -- метку {0,2}
SELECT itemoffset, ctid, hasmac, maclabel FROM bt_page_items('bt_macindex', 1);
--
-- Просматриваем физическое представление индекса bt_index
-- Ожидается 15 записей, hasmac = f maclabel -- пустое
SELECT itemoffset, ctid, hasmac, maclabel FROM bt_page_items('bt_index', 1);
--
-- Вакуумизируемся
VACUUM FULL;
--
-- Удаляем триггер и триггерную функцию
DROP TRIGGER "ТриггерBU" ON "Проверка";
DROP FUNCTION "ТрФункUPDATE"();
--
-- Просматривем физическое представление таблицы Проверка
-- Ожидается 10 записей, метка 5 из которых {2,3}, других 5 -- {0,2}, t_hasmac = t
SELECT lp, t_xmax = 0 AS last, t_ctid, t_hasmac, t_maclabel FROM heap_page_items(get_raw_page('Проверка', 0));
--
-- Просматриваем физическое представление индекса bt_macindex
-- Ожидается 10 записей, hasmac = t метка 5 из которых {2,3}, других 5 -- {0,2}
SELECT itemoffset, ctid, hasmac, maclabel FROM bt_page_items('bt_macindex', 1);
--
-- Просматриваем физическое представление индекса bt_index
-- Ожидается 10 записей, hasmac = f maclabel -- пустое
SELECT itemoffset, ctid, hasmac, maclabel FROM bt_page_items('bt_index', 1);
--
-- Модифицируем мандатную метку записей
CHMAC "Проверка" SET maclabel = '{1,2}';
--
-- Просматривем физическое представление таблицы Проверка
-- Ожидается 20 записей, метка которых t_maclabel = {1,2}, t_hasmac = t
-- 10 записей будут иметь t_xmax != 0 (last = f) -- эти записи не будут видны в данный момент времени
SELECT lp, t_xmax = 0 AS last, t_ctid, t_hasmac, t_maclabel FROM heap_page_items(get_raw_page('Проверка', 0));
--
-- Просматриваем физическое представление индекса bt_macindex
-- Ожидается 10 записей, hasmac = t maclabel = {1,2}
SELECT itemoffset, ctid, hasmac, maclabel FROM bt_page_items('bt_macindex', 1);
--
-- Просматриваем физическое представление индекса bt_index
-- Ожидается 10 записей, hasmac = f maclabel -- пустое
SELECT itemoffset, ctid, hasmac, maclabel FROM bt_page_items('bt_index', 1);
--
-- Выполняем очистку
VACUUM;
--
-- Просматривем физическое представление таблицы Проверка
-- Ожидается 20 записей, 10 из которых будут пустыми, т.к. они помечены как старые t_xmax != 0 (last = f)
-- остальные 10 будут содержать метку {1,2}
SELECT lp, t_xmax = 0 AS last, t_ctid, t_hasmac, t_maclabel FROM heap_page_items(get_raw_page('Проверка', 0));
--
-- Просматриваем физическое представление индекса bt_macindex
-- Ожидается 20 записей, hasmac = t, 10 записей будут иметь метку {1,2} -- тех записей,
-- которые видны в текущей транзакции, остальные -- будут иметь метки {2,3} или {0,2} --
-- индексные записи, которые указывают на устаревшие данные таблицы Проверка
SELECT itemoffset, ctid, hasmac, maclabel FROM bt_page_items('bt_macindex', 1);
--
-- Просматриваем физическое представление индекса bt_index
-- Ожидается 20 записей, hasmac = f maclabel -- пустое
SELECT itemoffset, ctid, hasmac, maclabel FROM bt_page_items('bt_index', 1);
--
-- Выполняем переиндексацию индекса bt_macindex
REINDEX INDEX bt_macindex;
--
-- Просматриваем физическое представление индекса bt_macindex
-- Ожидается 10 записей с актуальными метками записи таблицы Проверка -- {1,2}
SELECT itemoffset, ctid, hasmac, maclabel FROM bt_page_items('bt_macindex', 1);
--
--
-- 5. Проверка МПРД для индексов btree
--
--
-- Подключаемся как postgres
--
\connect - postgres
--
-- Очищаем таблицу Проверка
TRUNCATE "Проверка";
--
-- Подключаем расширение pg_hint_plan
CREATE EXTENSION pg_hint_plan;
--
-- Вставляем в таблицу данные от разных пользователей
\i 'support/init-test-table.sql'
--
\connect - postgres
--
-- Просматриваем содержимое таблицы Проверка
-- Ожидается 10 записей
SELECT maclabel, * FROM "Проверка";
--
-- Вакуумизирумся
VACUUM FULL;
--
-- Кластеризуем таблицу Проверка
CLUSTER "Проверка" USING bt_macindex;
--
-- Подключаемся как пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
LOAD '$libdir/plugins/pg_hint_plan';
--
-- Просматриваем содержимое таблицы Проверка
-- Ожидается 2 записи с метками {0,0}
SELECT maclabel, * FROM "Проверка";
--
--
-- ========== Seq Scan ==========
--
--
-- Ожидаются 2 записи с метками {0,0}
EXPLAIN (COSTS FALSE, VERBOSE) /*+ SeqScan("Проверка") */ SELECT * FROM "Проверка";
/*+ SeqScan("Проверка") */ SELECT * FROM "Проверка";
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ SeqScan("Проверка") */ SELECT * FROM "Проверка" WHERE "идентификатор" < 60;
/*+ SeqScan("Проверка") */ SELECT * FROM "Проверка" WHERE "идентификатор" < 60;
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ SeqScan("Проверка") */ SELECT maclabel, * FROM "Проверка";
/*+ SeqScan("Проверка") */ SELECT maclabel, * FROM "Проверка";
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ SeqScan("Проверка") */ SELECT maclabel, * FROM "Проверка" WHERE "идентификатор" < 60;
/*+ SeqScan("Проверка") */ SELECT maclabel, * FROM "Проверка" WHERE "идентификатор" < 60;
--
-- Пытаемся просмотреть запись с идентификатором, заведомо не совпадающей с уровнем пользователя
-- Ожидается 0 записей
EXPLAIN (COSTS FALSE , VERBOSE) /*+ SeqScan("Проверка") */ SELECT "идентификатор" FROM "Проверка" WHERE "идентификатор" = 91;
/*+ SeqScan("Проверка") */ SELECT "идентификатор" FROM "Проверка" WHERE "идентификатор" = 91;
--
--
-- ========== Tid Scan ==========
--
--
-- Ожидаются 2 записи с метками {0,0}
EXPLAIN (COSTS FALSE, VERBOSE) /*+ TidScan("Проверка") */SELECT * FROM "Проверка" WHERE ctid = any(array(SELECT ctid FROM "Проверка" LIMIT 10));
/*+ TidScan("Проверка") */SELECT * FROM "Проверка" where ctid = any(array(SELECT ctid FROM "Проверка" LIMIT 10));
--
--
-- ========== Index Scan ==========
--
--
--
-- Ожидается 1 запись с меткой {0,0}
EXPLAIN (COSTS FALSE, VERBOSE) /*+ IndexScan("Проверка" bt_macindex) */ SELECT maclabel, * FROM "Проверка" WHERE "идентификатор" BETWEEN 10 AND 30;
/*+ IndexScan("Проверка" bt_macindex) */ SELECT maclabel, * FROM "Проверка" WHERE "идентификатор" BETWEEN 10 AND 30;
--
-- Ожидается 1 запись с меткой {0,0}
EXPLAIN (COSTS FALSE, VERBOSE) /*+ IndexScan("Проверка" bt_index) */ SELECT maclabel, * FROM "Проверка" WHERE "идентификатор" BETWEEN 10 AND 30;
/*+ IndexScan("Проверка" bt_index) */ SELECT maclabel, * FROM "Проверка" WHERE "идентификатор" BETWEEN 10 AND 30;
--
-- Пытаемся просмотреть запись с идентификатором, заведомо не совпадающей с уровнем пользователя
-- Ожидается 0 записей
EXPLAIN (COSTS FALSE, VERBOSE) /*+ IndexScan("Проверка" bt_macindex) */ SELECT maclabel, * FROM "Проверка" WHERE "идентификатор" < 19;
/*+ IndexScan("Проверка" bt_macindex) */ SELECT maclabel, * FROM "Проверка" WHERE "идентификатор" < 19;
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ IndexScan("Проверка" bt_index) */ SELECT maclabel, * FROM "Проверка" WHERE "идентификатор" < 19;
/*+ IndexScan("Проверка" bt_index) */ SELECT maclabel, * FROM "Проверка" WHERE "идентификатор" < 19;
--
--
-- ========== Index Only Scan ==========
--
--
--
-- Ожидается 1 запись с меткой {0,0}
EXPLAIN (COSTS FALSE, VERBOSE) /*+ IndexOnlyScan("Проверка" bt_macindex) */ SELECT "идентификатор" FROM "Проверка" WHERE "идентификатор" BETWEEN 10 AND 30;
/*+ IndexOnlyScan("Проверка" bt_macindex) */ SELECT "идентификатор" FROM "Проверка" WHERE "идентификатор" BETWEEN 10 AND 30;
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ IndexOnlyScan("Проверка" bt_index) */ SELECT "идентификатор" FROM "Проверка" WHERE "идентификатор" BETWEEN 10 AND 30;
/*+ IndexOnlyScan("Проверка" bt_index) */ SELECT "идентификатор" FROM "Проверка" WHERE "идентификатор" BETWEEN 10 AND 30;
--
-- Ожидается 0 записей
EXPLAIN (COSTS FALSE, VERBOSE) /*+ IndexOnlyScan("Проверка" bt_macindex) */ SELECT "идентификатор" FROM "Проверка" WHERE "идентификатор" BETWEEN 19 AND 30;
/*+ IndexOnlyScan("Проверка" bt_macindex) */ SELECT "идентификатор" FROM "Проверка" WHERE "идентификатор" BETWEEN 19 AND 30;
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ IndexOnlyScan("Проверка" bt_index) */ SELECT "идентификатор" FROM "Проверка" WHERE "идентификатор" BETWEEN 19 AND 30;
/*+ IndexOnlyScan("Проверка" bt_index) */ SELECT "идентификатор" FROM "Проверка" WHERE "идентификатор" BETWEEN 19 AND 30;
--
-- Пытаемся просмотреть запись с идентификатором, заведомо не совпадающей с уровнем пользователя
-- Ожидается 0 записей
EXPLAIN (COSTS FALSE, VERBOSE) /*+ IndexOnlyScan("Проверка" bt_macindex) */ SELECT "идентификатор" FROM "Проверка" WHERE "идентификатор" = 19;
/*+ IndexOnlyScan("Проверка" bt_macindex) */ SELECT "идентификатор" FROM "Проверка" WHERE "идентификатор" = 19;
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ IndexOnlyScan("Проверка" bt_index) */ SELECT "идентификатор" FROM "Проверка" WHERE "идентификатор" = 19;
/*+ IndexOnlyScan("Проверка" bt_index) */ SELECT "идентификатор" FROM "Проверка" WHERE "идентификатор" = 19;
--
--
-- ========== BitMap Scan ==========
--
--
-- Ожидается 1 запись с меткой {0,0}
EXPLAIN (COSTS FALSE, VERBOSE) /*+ BitmapScan("Проверка" bt_macindex) */ SELECT * FROM "Проверка" WHERE "идентификатор" BETWEEN 10 AND 30;
/*+ BitmapScan("Проверка" bt_macindex) */ SELECT * FROM "Проверка" WHERE "идентификатор" BETWEEN 10 AND 30;
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ BitmapScan("Проверка" bt_index) */ SELECT * FROM "Проверка" WHERE "идентификатор" BETWEEN 10 AND 30;
/*+ BitmapScan("Проверка" bt_index) */ SELECT * FROM "Проверка" WHERE "идентификатор" BETWEEN 10 AND 30;
--
-- Ожидается 0 записей
EXPLAIN (COSTS FALSE, VERBOSE) /*+ BitmapScan("Проверка" bt_macindex) */ SELECT * FROM "Проверка" WHERE "идентификатор" BETWEEN 19 AND 30;
/*+ BitmapScan("Проверка" bt_macindex) */ SELECT * FROM "Проверка" WHERE "идентификатор" BETWEEN 19 AND 30;
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ BitmapScan("Проверка" bt_index) */ SELECT * FROM "Проверка" WHERE "идентификатор" BETWEEN 19 AND 30;
/*+ BitmapScan("Проверка" bt_index) */ SELECT * FROM "Проверка" WHERE "идентификатор" BETWEEN 19 AND 30;
--
-- Пытаемся просмотреть запись с идентификатором, заведомо не совпадающей с уровнем пользователя
EXPLAIN (COSTS FALSE, VERBOSE) /*+ BitmapScan("Проверка" bt_macindex) */ SELECT * FROM "Проверка" WHERE "идентификатор" = 19;
/*+ BitmapScan("Проверка" bt_macindex) */ SELECT * FROM "Проверка" WHERE "идентификатор" = 19;
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ BitmapScan("Проверка" bt_index) */ SELECT * FROM "Проверка" WHERE "идентификатор" = 19;
/*+ BitmapScan("Проверка" bt_index) */ SELECT * FROM "Проверка" WHERE "идентификатор" = 19;
--
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'