-- RBT-TEST - МПРД при наследовании таблиц
-- Проверка наследования мандатных атрибутов при наследовании таблиц (INHERITS)
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
--
-- Создаём родительскую таблицу без мандатных меток
CREATE TABLE "Родитель"("Идентификатор" INTEGER PRIMARY KEY NOT NULL);
-- Назначаем мандатную метку на таблицу 'Родитель'
MAC LABEL ON TABLE "Родитель" IS '{2,3}';
-- Меняем владельца таблицы на u_2_11
-- Ожидается успех
ALTER TABLE "Родитель" OWNER TO u_2_11;
--
-- Подключаемся как u2_1_11
\connect - u_2_11
-- Убеждаемся в том, что флаг CCR таблицы 'Родитель' сброшен
-- Ожидается 1 запись
SELECT relname AS "Имя отношения", relhasmacs AS "Содержит метки на строках?" FROM pg_class WHERE relname = 'Родитель';
-- Наполняем таблицу данными
INSERT INTO "Родитель" VALUES(1);
INSERT INTO "Родитель" VALUES(2);
INSERT INTO "Родитель" VALUES(3);
INSERT INTO "Родитель" VALUES(4);
INSERT INTO "Родитель" VALUES(5);
-- Добавляем в таблицу 'Родитель' столбец maclabel
-- Ожидается ошибка: добавить столбец maclabel может только администратор
ALTER TABLE "Родитель" SET WITH MACS;
--
\connect - postgres
-- Добавляем в таблицу 'Родитель' столбец maclabel
-- Ожидается успех
ALTER TABLE "Родитель" SET WITH MACS;
--
\connect - u_2_11
-- Убеждаемся в том, что был добавлен столбец maclabel
-- и все записи таблицы имеют метку {2,3}
SELECT maclabel, * FROM ONLY "Родитель";
-- Удаляем столбец maclabel из таблицы 'Родитель'
-- Ожидается ошибка: удалить столбец maclabel может только администратор
ALTER TABLE "Родитель" SET WITHOUT MACS;
--
-- Подключаемся как 'postgres' и удаляем столбец maclabel
\connect - postgres
-- Ожидается успех
ALTER TABLE "Родитель" SET WITHOUT MACS;
--
\connect - u_2_11
-- Убеждаемся в том, что был отсутствует столбец maclabel
-- Ожидается ошибка доступа
SELECT maclabel, * FROM ONLY "Родитель";
--
-- Убеждаемся, что в таблице 5 записей
-- Ожидается 5 записей
SELECT * FROM "Родитель";
--
\connect - postgres
-- Создаём таблицу-потомок, родитель которой является таблица 'Родитель'
CREATE TABLE "Потомок"("Данные" TEXT) INHERITS ("Родитель");
-- Назначаем метку таблице 'Потомок' {1,0}
-- Ожидается успех
MAC LABEL ON TABLE "Потомок" IS '{1,0}';
-- Меняем владельца таблицы 'Потомок' на 'u_2_11'
ALTER TABLE "Потомок" OWNER TO u_2_11;
-- Убеждаемся, что в таблице 'Потомок' содержится столбец 'Идентификатор' и 'Данные'
-- Ожидается 0 записей
SELECT * FROM "Потомок";
-- Наполняем таблицу данными
INSERT INTO "Потомок" VALUES(1, 'Запись 1');
INSERT INTO "Потомок" VALUES(2, 'Запись 2');
INSERT INTO "Потомок" VALUES(3, 'Запись 3');
INSERT INTO "Потомок" VALUES(4, 'Запись 4');
INSERT INTO "Потомок" VALUES(5, 'Запись 5');
--
-- Убеждаемся, что в таблице 'Потомок' содержится 5 записей
SELECT * FROM ONLY "Потомок";
-- Модифицируем таблицу 'Родитель', добавив системное поле maclabel
-- Ожидается успех
ALTER TABLE "Родитель" SET WITH MACS;
-- Убеждаемся в добавлении столбца maclabel,
-- а также, что все записи имеют метку таблицы - {2,3}
-- Ожидается 5 записей с меткой {2,3}
SELECT maclabel, * FROM ONLY "Родитель";
-- Ожидается 5 записей с меткой {1,0}
SELECT maclabel, * FROM ONLY "Потомок";
--
-- Удаляем столбец maclabel из таблицы 'Родитель'
-- Ожидается удаление столбца maclabel как из таблицы 'Родитель', так и из таблицы 'Потомок'
ALTER TABLE "Родитель" SET WITHOUT MACS;
-- Ожидается ошибка доступа
SELECT maclabel, * FROM "Родитель";
SELECT maclabel, * FROM "Потомок";
--
-- Добавим столбец maclabel в таблице 'Потомок'
ALTER TABLE "Потомок" SET WITH MACS;
-- Попробуем добавить столбец в родительской таблице
-- Ожидается успех
ALTER TABLE "Родитель" SET WITH MACS;
-- Выводим мандатные метки и записи таблиц
-- Ожидаются 5 записей с метками {2,3}
SELECT maclabel, * FROM ONLY "Родитель";
-- Ожидаются 5 записей с метками {1,0}
SELECT maclabel, * FROM ONLY "Потомок";
--
DROP TABLE "Потомок";
DROP TABLE "Родитель";
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'