-- RBT-TEST - МПРД при вставке (INSERT)
-- Проверка работы МПРД при добавлении данных командой INSERT
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
\i 'support/create-test-table.sql'
--
-- Проверяем вставку данных в таблицу без признака MAC CCR
\i 'support/init-test-table.sql'
--
-- Проверяем выборку данных (правильность заполнения тестовой таблицы)
--
-- Выполняем запрос от имени суперпользователя postgres
-- Ожидается 10 записей с метками пользователей
\connect - postgres
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем вставку данных в таблицу с признаком MAC CCR
-- Создаем от имени пользователя u_1_01 таблицу с меткой {1, 1}
\connect - u_1_01
CREATE TABLE "Схема_1_01"."Проверка_1_01" ("идентификатор" INTEGER PRIMARY KEY, "данные" TEXT) WITH (MACS=TRUE);
GRANT ALL ON "Схема_1_01"."Проверка_1_01" TO PUBLIC;
--
-- Пытаемся вставить в таблицу данные от имени разных пользователей
-- Выполняем запрос от имени пользователя u_0_00 с меткой {0,0}
-- Ожидается ошибка доступа
\connect - u_0_00
INSERT INTO "Схема_1_01"."Проверка_1_01" VALUES (1, USER || '-insert');
--
-- Выполняем запрос от имени пользователя u_0_01 с меткой {0,1}
-- Ожидается ошибка доступа
\connect - u_0_01
INSERT INTO "Схема_1_01"."Проверка_1_01" VALUES (2, USER || '-insert');
--
-- Выполняем запрос от имени пользователя u_1_01 с меткой {1,1}
-- Ожидается успех
\connect - u_1_01
INSERT INTO "Схема_1_01"."Проверка_1_01" VALUES (3, USER || '-insert');
--
-- Выполняем запрос от имени пользователя u_0_10 с меткой {0,2}
-- Ожидается ошибка доступа
\connect - u_0_10
INSERT INTO "Схема_1_01"."Проверка_1_01" VALUES (4, USER || '-insert');
--
-- Выполняем запрос от имени пользователя u_1_10 с меткой {1,2}
-- Ожидается ошибка доступа
\connect - u_1_10
INSERT INTO "Схема_1_01"."Проверка_1_01" VALUES (5, USER || '-insert');
--
-- Выполняем запрос от имени пользователя u_1_11 с меткой {1,3}
-- Ожидается ошибка доступа
\connect - u_1_11
INSERT INTO "Схема_1_01"."Проверка_1_01" VALUES (6, USER || '-insert');
--
-- Выполняем запрос от имени пользователя u_2_11 с меткой {2,3}
-- Ожидается ошибка доступа
\connect - u_2_11
INSERT INTO "Схема_1_01"."Проверка_1_01" VALUES (7, USER || '-insert');
--
-- Выполняем запрос от имени пользователя dbsa
-- Ожидается успех
\connect - dbsa
SET ac_session_maclabel = '{0, 0}';
INSERT INTO "Схема_1_01"."Проверка_1_01" VALUES (8, USER || '-insert-0_00');
SET ac_session_maclabel = '{1, 0}';
INSERT INTO "Схема_1_01"."Проверка_1_01" VALUES (9, USER || '-insert-1_00');
--
-- Ожидается ошибка доступа (метка записи превышает метку таблицы)
SET ac_session_maclabel = '{1, 3}';
INSERT INTO "Схема_1_01"."Проверка_1_01" VALUES (10, USER || '-insert-1_11');
--
-- Выполняем запрос от имени суперпользователя postgres
-- Ожидается 3 записи
\connect - postgres
SELECT maclabel,* FROM "Схема_1_01"."Проверка_1_01" ORDER BY "идентификатор";
--
-- Проверяем возможность вставки метки пользователем с привилегией назначения метки CHMAC
\connect - dbsa
--
-- Ожидается успех
INSERT INTO "Схема_1_01"."Проверка_1_01"(maclabel, "идентификатор", "данные") VALUES ('{1,0}', 11, USER || '-insert-1_00');
--
-- Ожидается ошибка доступа (метка записи превышает метку таблицы)
INSERT INTO "Схема_1_01"."Проверка_1_01"(maclabel, "идентификатор", "данные") VALUES ('{2,0}', 11, USER || '-insert-2_00');
--
-- Проверяем, что была вставлена одна строка
-- Ожидается 4 записи
SELECT maclabel,* FROM "Схема_1_01"."Проверка_1_01" ORDER BY "идентификатор";
--
-- Проверяем невозможность вставки метки пользователем без привилегии назначения метки CHMAC
-- Ожидается ошибка доступа
\connect - u_1_01
INSERT INTO "Схема_1_01"."Проверка_1_01"(maclabel, "идентификатор", "данные") VALUES ('{1,1}', 12, USER || '-insert-0_00');
--
-- Проверяем, что строка не была вставлена
-- Ожидается 4 записи
\connect - dbsa
SELECT maclabel,* FROM "Схема_1_01"."Проверка_1_01" ORDER BY "идентификатор";
--
-- Подключаемся пользователем u_1_01 с меткой {1,1}
\connect - u_1_01
--
-- Создаем таблицу на основе существующей с помощью CREATE ... (LIKE ...)
CREATE TABLE "Проверка2" (LIKE "Проверка")  WITH(MACS=TRUE);
--
-- Проверяем метку созданной таблицы
-- Ожидается, что метка созданной таблицы соответствует метке пользователя {1,1}
-- и записи таблицы защищены, признак MAC CCR установлен по умолчанию
SELECT relname AS "Наименование таблицы", maclabel AS "Метка",
       relhasmacs AS "Записи имеют метки", relmacccr AS "MAC CCR"
FROM pg_class WHERE oid = 'Проверка2'::regclass::oid;
--
-- Проверяем содержимое таблицы
-- Ожидается что таблица пуста
SELECT * FROM "Проверка2" ORDER BY "идентификатор";
--
-- Выполняем вставку данных в таблицу выборкой из другой таблицы
INSERT INTO "Проверка2" SELECT "идентификатор", "данные" FROM "Проверка";
--
-- Проверяем содержимое таблицы
-- Ожидается 5 записей, доступные по МПРД, с метками пользователя {1,1}
SELECT maclabel, * FROM "Проверка2" ORDER BY "идентификатор";
--
-- Очищаем таблицу
DELETE FROM "Проверка2";
--
-- Выполняем вставку данных в таблицу выборкой из другой таблицы
-- с помощью DISTINCT
INSERT INTO "Проверка2" SELECT DISTINCT "идентификатор", "данные" FROM "Проверка";
--
-- Проверяем содержимое таблицы
-- Ожидается 5 записей, доступные по МПРД, с метками пользователя {1,1}
SELECT maclabel, * FROM "Проверка2" ORDER BY "идентификатор";
--
-- Очищаем таблицу
DELETE FROM "Проверка2";
--
-- Выполняем вставку данных в таблицу выборкой из другой таблицы
-- с помощью DISTINCT ON
INSERT INTO "Проверка2" SELECT DISTINCT ON ("идентификатор") "идентификатор", "данные" FROM "Проверка";
--
-- Проверяем содержимое таблицы
-- Ожидается 5 записей, доступные по МПРД, с метками пользователя {1,1}
SELECT maclabel, * FROM "Проверка2" ORDER BY "идентификатор";
--
--
-- Проверяем выполнение ON CONFLICT DO NOTHING и DO UPDATE SET
-- Создаем таблицу {1,1} от имени u_1_01 {1,1}
\connect - u_1_01
CREATE TABLE "Схема_1_01"."Проверка3_1_01" ("идентификатор" INTEGER PRIMARY KEY, "данные" TEXT) WITH (MACS=TRUE);
GRANT ALL ON "Схема_1_01"."Проверка3_1_01" TO PUBLIC;
SELECT relname, relhasmacs, maclabel, relmacccr from PG_CLASS WHERE relname LIKE 'Проверка3_1_01';
-- Создаем в таблице запись с меткой {1,1} с условием ON CONFLICT DO NOTHING
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (1, USER || '-insert') ON CONFLICT DO NOTHING;
SELECT *, maclabel FROM "Схема_1_01"."Проверка3_1_01";
-- Повторяем вставку с наличием конфликта и с условием ON CONFLICT DO NOTHING. Не ожидаем изменений и сообщений об ошибке
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (1, USER || '-insert') ON CONFLICT DO NOTHING;
SELECT *, maclabel FROM "Схема_1_01"."Проверка3_1_01";
-- Проверяем UPDATE SET. Ожидаем изменение поля "идентификатор" с 1 на 2
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (1, USER || '-insert') ON CONFLICT ("идентификатор") DO UPDATE SET "идентификатор"=2, "данные"=USER || '-insert';
-- Пересоздаем запись "идентификатор"=1
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (1, USER || '-insert') ON CONFLICT ("идентификатор") DO NOTHING;
SELECT *, maclabel FROM "Схема_1_01"."Проверка3_1_01";
--
-- Выключаем флаг CCR и передаем права на таблицу пользователю с меньшим уровнем метки {0,0}
\connect - postgres
MAC CCR ON TABLE "Схема_1_01"."Проверка3_1_01" IS OFF;
ALTER TABLE "Схема_1_01"."Проверка3_1_01" OWNER TO u_0_00;
-- Пытаемся вставить в таблицу данные от имени пользователя u_0_00 с условием ON CONFLICT DO NOTHING. Не ожидаем сообщений об ошибке
\connect - u_0_00
-- Вставка с конфликтом на запись высокоуровневого пользователя
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (2, USER || '-insert') ON CONFLICT DO NOTHING;
-- Вставка в свободное место
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (3, USER || '-insert') ON CONFLICT DO NOTHING;
-- Ожидаем появление "идентификатор" 3 от имени пользователя u_0_00 {0,0} ("идентификатор" 2 вне видимости, кортеж не меняется)
SELECT *, maclabel FROM "Схема_1_01"."Проверка3_1_01";
-- Проверяем UPDATE SET. Не ожидаем замены записи высокоуровневого пользователя, но ожидаем замены собственной записи u_0_00
-- Вставка при конфликте с высокоуровневой записью и с условием ON CONFLICT DO UPDATE SET
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (2, USER || '-insert') ON CONFLICT ("идентификатор") DO UPDATE SET "идентификатор"=4, "данные"=USER || '-insert';
-- Вставка при конфликте с высокоуровневой записью и с условием ON CONFLICT DO UPDATE SET на высокоуровневую конфликтную запись
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (1, USER || '-insert') ON CONFLICT ("идентификатор") DO UPDATE SET "идентификатор"=2, "данные"=USER || '-insert';
-- Вставка при конфликте с собственной записью и с условием ON CONFLICT DO UPDATE SET
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (3, USER || '-insert') ON CONFLICT ("идентификатор") DO UPDATE SET "идентификатор"=5, "данные"=USER || '-insert';
SELECT *, maclabel FROM "Схема_1_01"."Проверка3_1_01";
-- Передаем права на таблицу пользователю u_2_11 с самым высоким уровнем метки {2,2}
\connect - postgres
ALTER TABLE "Схема_1_01"."Проверка3_1_01" OWNER TO u_2_11;
-- Пытаемся вставить в таблицу данные от имени пользователя u_2_11
-- Ожидается ошибка отсутствия прав доступа на добавление записи
\connect - u_2_11
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (1, USER || '-insert') ON CONFLICT DO NOTHING;
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (2, USER || '-insert') ON CONFLICT DO NOTHING;
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (3, USER || '-insert') ON CONFLICT DO NOTHING;
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (4, USER || '-insert') ON CONFLICT DO NOTHING;
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (5, USER || '-insert') ON CONFLICT DO NOTHING;
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (6, USER || '-insert') ON CONFLICT DO NOTHING;
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (1, USER || '-insert') ON CONFLICT ("идентификатор") DO UPDATE SET "идентификатор"=6, "данные"=USER || '-insert';
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (2, USER || '-insert') ON CONFLICT ("идентификатор") DO UPDATE SET "идентификатор"=7, "данные"=USER || '-insert';
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (3, USER || '-insert') ON CONFLICT ("идентификатор") DO UPDATE SET "идентификатор"=8, "данные"=USER || '-insert';
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (4, USER || '-insert') ON CONFLICT ("идентификатор") DO UPDATE SET "идентификатор"=9, "данные"=USER || '-insert';
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (5, USER || '-insert') ON CONFLICT ("идентификатор") DO UPDATE SET "идентификатор"=10, "данные"=USER || '-insert';
INSERT INTO "Схема_1_01"."Проверка3_1_01" VALUES (6, USER || '-insert') ON CONFLICT ("идентификатор") DO UPDATE SET "идентификатор"=11, "данные"=USER || '-insert';
-- Проверяем все записи в таблице
\connect - postgres
SELECT *, maclabel FROM "Схема_1_01"."Проверка3_1_01";
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'
