-- RBT-TEST - МПРД для соединений (joins) таблиц
-- Проверка МПРД при соединениях таблиц (NestLoop, HashJoin, MergeJoin)
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
--
-- Cоздаем расширение pg_hint_plan
CREATE EXTENSION pg_hint_plan;
--
-- Создаем таблицы
CREATE TABLE "ПроверкаСМетками"("идентификатор" INTEGER, "данные" TEXT) WITH (MACS=TRUE);
CREATE TABLE "ПроверкаСМетками2"("идентификатор" INTEGER, "данные" TEXT) WITH (MACS=TRUE);
CREATE TABLE "ПроверкаБезМеток"("идентификатор" INTEGER, "данные" TEXT) WITH (MACS=FALSE);
CREATE TABLE "ПроверкаБезМеток2"("идентификатор" INTEGER, "данные" TEXT) WITH (MACS=FALSE);
--
-- Выдаем права на таблицы
GRANT ALL ON TABLE "ПроверкаСМетками" TO PUBLIC;
GRANT ALL ON TABLE "ПроверкаСМетками2" TO PUBLIC;
GRANT ALL ON TABLE "ПроверкаБезМеток" TO PUBLIC;
GRANT ALL ON TABLE "ПроверкаБезМеток2" TO PUBLIC;
--
-- Назначаем мандатные атрибуты на созданные таблицы
MAC LABEL ON TABLE "ПроверкаСМетками" IS '{2,3}';
MAC CCR ON TABLE "ПроверкаСМетками" IS OFF;
--
MAC LABEL ON TABLE "ПроверкаСМетками2" IS '{1,1}';
MAC CCR ON TABLE "ПроверкаСМетками2" IS OFF;
--
MAC LABEL ON TABLE "ПроверкаБезМеток" IS '{1,3}';
--
MAC LABEL ON TABLE "ПроверкаБезМеток2" IS '{0,1}';
--
-- Наполняем таблицы
--
\connect - u_0_00
INSERT INTO "ПроверкаСМетками" VALUES(1, 'данные ' || USER);
\connect - u_0_01
INSERT INTO "ПроверкаСМетками" VALUES(2, 'данные ' || USER);
\connect - u_1_01
INSERT INTO "ПроверкаСМетками" VALUES(3, 'данные ' || USER);
\connect - u_0_10
INSERT INTO "ПроверкаСМетками" VALUES(4, 'данные ' || USER);
\connect - u_1_10
INSERT INTO "ПроверкаСМетками" VALUES(5, 'данные ' || USER);
\connect - u_1_11
INSERT INTO "ПроверкаСМетками" VALUES(6, 'данные ' || USER);
\connect - u_2_11
INSERT INTO "ПроверкаСМетками" VALUES(7, 'данные ' || USER);
--
\connect - dbsa
SET ac_session_maclabel = '{0,0}';
INSERT INTO "ПроверкаСМетками" VALUES (8, 'данные ' || USER);
SET ac_session_maclabel = '{1,0}';
INSERT INTO "ПроверкаСМетками" VALUES (9, 'данные ' || USER);
SET ac_session_maclabel = '{1,3}';
INSERT INTO "ПроверкаСМетками" VALUES (10, 'данные ' || USER);
--
--
\connect - u_0_00
INSERT INTO "ПроверкаСМетками2" VALUES(1, 'данные ' || USER);
\connect - u_0_01
INSERT INTO "ПроверкаСМетками2" VALUES(2, 'данные ' || USER);
\connect - u_1_01
INSERT INTO "ПроверкаСМетками2" VALUES(3, 'данные ' || USER);
--
\connect - dbsa
SET ac_session_maclabel = '{0,0}';
INSERT INTO "ПроверкаСМетками2" VALUES (4, 'данные ' || USER);
SET ac_session_maclabel = '{0,1}';
INSERT INTO "ПроверкаСМетками2" VALUES (5, 'данные ' || USER);
SET ac_session_maclabel = '{1,1}';
INSERT INTO "ПроверкаСМетками2" VALUES (6, 'данные ' || USER);
--
--
-- Вставку в таблицу от пользователей, метка которой
-- не равна {1,3} бессмысленно, т.к. если метка пользователя
-- будет меньше {1,3} -- то пользователь не увидит таблицу,
-- а если метка пользователя будет превосходить {1,3} -- 
-- то отказ доступа на запись
\connect - u_1_11
INSERT INTO "ПроверкаБезМеток" VALUES(1, 'данные ' || USER);
--
\connect - dbsa
SET ac_session_maclabel = '{2,3}';
INSERT INTO "ПроверкаБезМеток" VALUES(2, 'данные ' || USER);
--
--
\connect - u_0_01
INSERT INTO "ПроверкаБезМеток2" VALUES(1, 'данные ' || USER);
--
\connect - dbsa
SET ac_session_maclabel = '{0,1}';
INSERT INTO "ПроверкаБезМеток2" VALUES(2, 'данные ' || USER);
--
--
-- Просматриваем данные таблиц
--
-- Ожидается 10 записей с мандатными метками пользователей,
-- которые проводили вставку
SELECT maclabel, * FROM "ПроверкаСМетками";
-- Ожидается 6 записей с мандатными метками пользователей,
-- которые проводили вставку
SELECT maclabel, * FROM "ПроверкаСМетками2";
--
-- Ожидается 2 записи пользователей u_1_11 и dbsa
SELECT * FROM "ПроверкаБезМеток";
-- Ожидается 2 записи пользователей u_0_01 и dbsa
SELECT * FROM "ПроверкаБезМеток2";
--
--
-- ==================================================
-- I. Соединения мандатных таблиц
-- ==================================================
--
--
-- 1. NextLoop
--
--
-- Заходим пользователем u_0_00 с меткой {0,0}
--
\connect - u_0_00
EXPLAIN (COSTS FALSE, VERBOSE) /*+ NestLoop(ПроверкаСМетками ПроверкаСМетками2) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.maclabel AS maclabel2, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаСМетками2" t2
ON t1."идентификатор" < t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- Ожидается 1 запись с меткой {0,0}
/*+ NestLoop(ПроверкаСМетками ПроверкаСМетками2) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.maclabel AS maclabel2, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаСМетками2" t2
ON t1."идентификатор" < t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- Заходим пользователем u_1_01 с меткой {1,1}
--
\connect - u_1_01
EXPLAIN (COSTS FALSE, VERBOSE) /*+ NestLoop(ПроверкаСМетками ПроверкаСМетками2) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.maclabel AS maclabel2, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаСМетками2" t2
ON t1."идентификатор" < t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- Ожидается 12 записей с меткой <= {1,1}
/*+ NestLoop(ПроверкаСМетками ПроверкаСМетками2) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.maclabel AS maclabel2, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаСМетками2" t2
ON t1."идентификатор" < t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- Заходим пользователем с игнорированием МПРД
--
\connect - dbsa
EXPLAIN (COSTS FALSE, VERBOSE) /*+ NestLoop(ПроверкаСМетками ПроверкаСМетками2) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.maclabel AS maclabel2, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаСМетками2" t2
ON t1."идентификатор" < t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- Ожидается 15 записей
/*+ NestLoop(ПроверкаСМетками ПроверкаСМетками2) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.maclabel AS maclabel2, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаСМетками2" t2
ON t1."идентификатор" < t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- 2. HashJoin
--
-- Заходим пользователем u_0_00 с меткой {0,0}
--
\connect - u_0_00
EXPLAIN (COSTS FALSE, VERBOSE) /*+ HashJoin(ПроверкаСМетками ПроверкаСМетками2) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.maclabel AS maclabel2, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаСМетками2" t2
ON t1."идентификатор" = t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- Ожидается 1 запись с меткой {0,0}
/*+ HashJoin(ПроверкаСМетками ПроверкаСМетками2) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.maclabel AS maclabel2, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаСМетками2" t2
ON t1."идентификатор" = t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- Заходим пользователем u_1_01 с меткой {1,1}
--
\connect - u_1_01
EXPLAIN (COSTS FALSE, VERBOSE) /*+ HashJoin(ПроверкаСМетками ПроверкаСМетками2) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.maclabel AS maclabel2, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаСМетками2" t2
ON t1."идентификатор" = t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- Ожидается 3 записи с меткой <= {1,1}
/*+ HashJoin(ПроверкаСМетками ПроверкаСМетками2) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.maclabel AS maclabel2, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаСМетками2" t2
ON t1."идентификатор" = t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- Заходим пользователем с игнорированием МПРД
--
\connect - dbsa
EXPLAIN (COSTS FALSE, VERBOSE) /*+ HashJoin(ПроверкаСМетками ПроверкаСМетками2) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.maclabel AS maclabel2, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаСМетками2" t2
ON t1."идентификатор" = t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- Ожидается 6 записей
/*+ HashJoin(ПроверкаСМетками ПроверкаСМетками2) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.maclabel AS maclabel2, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаСМетками2" t2
ON t1."идентификатор" = t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- 3. MergeJoin
--
-- Заходим пользователем u_0_00 с меткой {0,0}
--
\connect - u_0_00
EXPLAIN (COSTS FALSE, VERBOSE) /*+ MergeJoin(ПроверкаСМетками ПроверкаСМетками2) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.maclabel AS maclabel2, t2.*
FROM "ПроверкаСМетками" t1
JOIN "ПроверкаСМетками2" t2
ON t1."идентификатор" = t2."идентификатор";
--
-- Ожидается 1 запись с меткой {0,0}
/*+ MergeJoin(ПроверкаСМетками ПроверкаСМетками2) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.maclabel AS maclabel2, t2.*
FROM "ПроверкаСМетками" t1
JOIN "ПроверкаСМетками2" t2
ON t1."идентификатор" = t2."идентификатор";
--
-- Заходим пользователем u_1_01 с меткой {1,1}
--
\connect - u_1_01
EXPLAIN (COSTS FALSE, VERBOSE) /*+ MergeJoin(ПроверкаСМетками ПроверкаСМетками2) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.maclabel AS maclabel2, t2.*
FROM "ПроверкаСМетками" t1
JOIN "ПроверкаСМетками2" t2
ON t1."идентификатор" = t2."идентификатор";
--
-- Ожидается 3 записи с меткой <= {1,1}
/*+ MergeJoin(ПроверкаСМетками ПроверкаСМетками2) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.maclabel AS maclabel2, t2.*
FROM "ПроверкаСМетками" t1
JOIN "ПроверкаСМетками2" t2
ON t1."идентификатор" = t2."идентификатор";
--
-- Заходим пользователем с игнорированием МПРД
--
\connect - dbsa
EXPLAIN (COSTS FALSE, VERBOSE) /*+ MergeJoin(ПроверкаСМетками ПроверкаСМетками2) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.maclabel AS maclabel2, t2.*
FROM "ПроверкаСМетками" t1
JOIN "ПроверкаСМетками2" t2
ON t1."идентификатор" = t2."идентификатор";
--
-- Ожидается 6 записей
/*+ MergeJoin(ПроверкаСМетками ПроверкаСМетками2) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.maclabel AS maclabel2, t2.*
FROM "ПроверкаСМетками" t1
JOIN "ПроверкаСМетками2" t2
ON t1."идентификатор" = t2."идентификатор";
--
--
-- ==================================================
-- II. Соединения мандатных и немандатных таблиц
-- ==================================================
--
--
-- 1. NextLoop
--
--
-- Заходим пользователем u_1_11 с меткой {1,3}
--
\connect - u_1_11
EXPLAIN (COSTS FALSE, VERBOSE) /*+ NestLoop(ПроверкаСМетками ПроверкаБезМеток) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаБезМеток" t2
ON t1."идентификатор" < t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- Ожидается 1 запись с меткой {0,0}
/*+ NestLoop(ПроверкаСМетками ПроверкаБезМеток) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаБезМеток" t2
ON t1."идентификатор" < t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- Заходим пользователем с игнорированием МПРД
--
\connect - dbsa
EXPLAIN (COSTS FALSE, VERBOSE) /*+ NestLoop(ПроверкаСМетками ПроверкаБезМеток) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаБезМеток" t2
ON t1."идентификатор" < t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- Ожидается 1 запись
/*+ NestLoop(ПроверкаСМетками ПроверкаБезМеток) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаБезМеток" t2
ON t1."идентификатор" < t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- 2. HashJoin
--
-- Заходим пользователем u_1_11 с меткой {1,3}
--
\connect - u_1_11
EXPLAIN (COSTS FALSE, VERBOSE) /*+ HashJoin(ПроверкаСМетками ПроверкаБезМеток) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаБезМеток" t2
ON t1."идентификатор" = t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- Ожидается 2 записи с меткой <= {1,3}
/*+ HashJoin(ПроверкаСМетками ПроверкаБезМеток) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаБезМеток" t2
ON t1."идентификатор" = t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- Заходим пользователем с игнорированием МПРД
--
\connect - dbsa
EXPLAIN (COSTS FALSE, VERBOSE) /*+ HashJoin(ПроверкаСМетками ПроверкаБезМеток) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаБезМеток" t2
ON t1."идентификатор" = t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- Ожидается 2 записи
/*+ HashJoin(ПроверкаСМетками ПроверкаБезМеток) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.*
FROM "ПроверкаСМетками" t1
LEFT JOIN "ПроверкаБезМеток" t2
ON t1."идентификатор" = t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- 3. MergeJoin
--
-- Заходим пользователем u_1_11 с меткой {1,3}
--
\connect - u_1_11
EXPLAIN (COSTS FALSE, VERBOSE) /*+ MergeJoin(ПроверкаСМетками ПроверкаБезМеток) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.*
FROM "ПроверкаСМетками" t1
JOIN "ПроверкаБезМеток" t2
ON t1."идентификатор" = t2."идентификатор";
--
-- Ожидается 2 записи с меткой <= {1,3}
/*+ MergeJoin(ПроверкаСМетками ПроверкаБезМеток) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.*
FROM "ПроверкаСМетками" t1
JOIN "ПроверкаБезМеток" t2
ON t1."идентификатор" = t2."идентификатор";
--
-- Заходим пользователем с игнорированием МПРД
--
\connect - dbsa
EXPLAIN (COSTS FALSE, VERBOSE) /*+ MergeJoin(ПроверкаСМетками ПроверкаБезМеток) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.*
FROM "ПроверкаСМетками" t1
JOIN "ПроверкаБезМеток" t2
ON t1."идентификатор" = t2."идентификатор";
--
-- Ожидается 2 записи
/*+ MergeJoin(ПроверкаСМетками ПроверкаБезМеток) */
SELECT t1.maclabel AS maclabel1, t1.*, t2.*
FROM "ПроверкаСМетками" t1
JOIN "ПроверкаБезМеток" t2
ON t1."идентификатор" = t2."идентификатор";
--
--
-- ==================================================
-- III. Соединения немандатных таблиц
-- ==================================================
--
--
-- 1. NextLoop
--
-- Заходим пользователем с игнорированием МПРД
--
\connect - dbsa
EXPLAIN (COSTS FALSE, VERBOSE) /*+ NestLoop(ПроверкаБезМеток ПроверкаБезМеток2) */
SELECT t1.*, t2.*
FROM "ПроверкаБезМеток" t1
LEFT JOIN "ПроверкаБезМеток2" t2
ON t1."идентификатор" < t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- Ожидается 1 запись
/*+ NestLoop(ПроверкаБезМеток ПроверкаБезМеток2) */
SELECT t1.*, t2.*
FROM "ПроверкаБезМеток" t1
LEFT JOIN "ПроверкаБезМеток2" t2
ON t1."идентификатор" < t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- 2. HashJoin
--
\connect - dbsa
EXPLAIN (COSTS FALSE, VERBOSE) /*+ HashJoin(ПроверкаБезМеток ПроверкаБезМеток2) */
SELECT t1.*, t2.*
FROM "ПроверкаБезМеток" t1
LEFT JOIN "ПроверкаБезМеток2" t2
ON t1."идентификатор" = t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- Ожидается 2 записи
/*+ HashJoin(ПроверкаСМетками ПроверкаБезМеток) */
SELECT t1.*, t2.*
FROM "ПроверкаБезМеток" t1
LEFT JOIN "ПроверкаБезМеток2" t2
ON t1."идентификатор" = t2."идентификатор"
WHERE t2."идентификатор" BETWEEN 1 AND 10;
--
-- 3. MergeJoin
--
-- Заходим пользователем с игнорированием МПРД
--
\connect - dbsa
EXPLAIN (COSTS FALSE, VERBOSE) /*+ MergeJoin(ПроверкаБезМеток ПроверкаБезМеток2) */
SELECT t1.*, t2.*
FROM "ПроверкаБезМеток" t1
JOIN "ПроверкаБезМеток2" t2
ON t1."идентификатор" = t2."идентификатор";
--
-- Ожидается 2 записи
/*+ MergeJoin(ПроверкаСМетками ПроверкаБезМеток) */
SELECT t1.*, t2.*
FROM "ПроверкаБезМеток" t1
JOIN "ПроверкаБезМеток2" t2
ON t1."идентификатор" = t2."идентификатор";
--
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'