-- RBT-TEST - МПРД при доступе к материализованным видам
-- Проверка работы МПРД при доступе к материализованным видам
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
\i 'support/select-test-table.sql'
--
-- Подключаемся пользователем u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Виды и материализованные виды не могут быть созданы
-- с явным указанием защиты (отсутствия) меток на строках,
-- т.к. они создаются изначально *без* меток на строках
-- Ожидается ошибка
CREATE MATERIALIZED VIEW u_1_11_matview WITH (MACS=TRUE) AS SELECT * FROM "Проверка";
-- Ожидается ошибка
CREATE MATERIALIZED VIEW u_1_11_matview_m WITH (MACS=FALSE) AS SELECT * FROM "Проверка";
-- Создаем материализованный вид
-- Ожидается успех
CREATE MATERIALIZED VIEW u_1_11_matview AS SELECT * FROM "Проверка";
--
-- Проверяем метку созданного вида
-- Ожидается, что метка созданного вида соответствует метке пользователя {1,3}
-- и записи вида не защищены (меток не имеют)
-- признак MAC CCR установлен по умолчанию
SELECT relname AS "Наименование вида", maclabel AS "Метка",
       relhasmacs AS "Записи имеют метки", relmacccr AS "MAC CCR"
FROM pg_class WHERE oid = 'u_1_11_matview'::regclass::oid;
--
-- Проверяем содержимое вида
-- Ожидается 9 записей, попали только доступные по МПРД записи
SELECT * FROM u_1_11_matview ORDER BY "идентификатор";
--
-- Подключаемся пользователем u_1_01 с меткой {1,1}
\connect - u_1_01
--
-- Добавляем запись в таблицу
INSERT INTO "Проверка" VALUES (84, USER || '-new');
--
-- Подключаемся пользователем u_0_00 с меткой {0,0}
\connect - u_0_00
--
-- Удаляем записи из таблицы
DELETE FROM "Проверка";
--
-- Подключаемся пользователем u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Просматриваем доступные по МПРД записи в таблице "Проверка"
-- Ожидается 8 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Обновляем виды
REFRESH MATERIALIZED VIEW u_1_11_matview;
--
-- Проверяем содержимое вида
-- Ожидается 8, попали только доступные по МПРД записи
SELECT * FROM u_1_11_matview ORDER BY "идентификатор";
--
-- Подключаемся пользователем u_1_01 с меткой {1,1}
\connect - u_1_01
--
-- Попытка обновления вида
-- Ожидается, что вид недоступен для пользователя
REFRESH MATERIALIZED VIEW u_1_11_matview;
--
-- Подключаемся пользователем u_2_11 с меткой {2,3}
\connect - u_2_11
--
-- Попытка обновления вида
-- Ожидается ошибка доступа
REFRESH MATERIALIZED VIEW u_1_11_matview;
--
-- Подключаемся пользователем dbsa с привилегией CHMAC
\connect - dbsa
--
-- Просматриваем записи в таблице "Проверка"
-- Ожидается 9 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Создаем материализованный вид
CREATE MATERIALIZED VIEW dbsa_matview AS SELECT * FROM "Проверка";
GRANT ALL ON dbsa_matview TO PUBLIC;
--
-- Проверяем метку созданного вида
-- Ожидается, что метка созданного вида соответствует метке пользователя {2,3}
-- и записи вида незащищены (меток не имеют)
-- признак MAC CCR установлен по умолчанию
SELECT relname AS "Наименование вида", maclabel AS "Метка",
       relhasmacs AS "Записи имеют метки", relmacccr AS "MAC CCR"
FROM pg_class WHERE oid = 'dbsa_matview'::regclass::oid;
--
-- Проверяем содержимое вида
-- Ожидается 9 записей, попали все записи
SELECT * FROM dbsa_matview ORDER BY "идентификатор";
--
-- Подключаемся как пользователь u_0_00
\connect - u_0_00
-- Пытаемся прочитать содержимое материализованного представления dbsa_matview
-- Ожидается ошибка доступа
SELECT * FROM dbsa_matview;
--
-- Подключаемся как пользователь postgres
\connect - postgres
-- Снимаем флаг MAC CCR с dbsa_matview
MAC CCR ON MATERIALIZED VIEW dbsa_matview IS OFF;
--
-- Подключаемся как пользователь u_0_00
\connect - u_0_00
-- Повторяем попытку прочтения информации из вида,
-- метка которого больше метки пользователя
-- Ожидается 0 строк,
-- т.к. при создании вида данные аггрегировались до уровня пользователя dbsa -- {2,3}
SELECT * FROM dbsa_matview;
-- Подключаемся как пользователь u_2_11
\connect - u_2_11
-- Читаем содержимое вида
-- Ожидается 9 записей
SELECT * FROM dbsa_matview;
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'