-- RBT-TEST - МПРД при выборке (SELECT)
-- Проверка работы МПРД при выборке данных командой SELECT
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
\i 'support/select-test-table.sql'
--
-- Проверка аггрегатных функций для таблицы без признака MAC CCR
--
-- Выполняем запрос от имени суперпользователя postgres
\connect - postgres
DELETE FROM "Проверка" WHERE maclabel = '{0,0}';
-- Ожидается 8
SELECT count(*) from "Проверка";
-- Ожидается 1
SELECT count("идентификатор") from "Проверка" where "идентификатор" = 82;
-- Ожидается true
SELECT exists(select "идентификатор" from "Проверка" where "идентификатор" = 82);
--
-- Выполняем запрос от имени пользователя u_0_00 с меткой {0,0}
\connect - u_0_00
-- Ожидается 0
SELECT count(*) from "Проверка";
-- Ожидается 0
SELECT count("идентификатор") from "Проверка" where "идентификатор" = 82;
-- Ожидается false
SELECT exists(select "идентификатор" from "Проверка" where "идентификатор" = 82);
--
-- Выполняем запрос от имени пользователя u_1_01 с меткой {1,1}
\connect - u_1_01
-- Ожидается 3
SELECT count(*) from "Проверка";
-- Ожидается 1
SELECT count("идентификатор") from "Проверка" where "идентификатор" = 82;
-- Ожидается true
SELECT exists(select "идентификатор" from "Проверка" where "идентификатор" = 82);
--
-- Администратор postgres устанавливает метку {1,1} на таблицу
-- и устанавливает флаг MAC CCR
\connect - postgres
DELETE FROM "Проверка" WHERE NOT maclabel <= '{1,1}';
MAC LABEL ON TABLE "Проверка" IS '{1,1}';
MAC CCR ON TABLE "Проверка" IS ON;
--
-- Проверяем выборку данных (правильность заполнения тестовой таблицы)
--
-- Выполняем запрос от имени суперпользователя postgres
-- Ожидается 3 записи
\connect - postgres
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Выполняем запрос от имени пользователя u_0_00 с меткой {0,0}
-- Ожидается ошибка доступа
\connect - u_0_00
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Выполняем запрос от имени пользователя u_0_01 с меткой {0,1}
-- Ожидается ошибка доступа
\connect - u_0_01
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Выполняем запрос от имени пользователя u_1_01 с меткой {1,1}
-- Ожидается 3 записей
\connect - u_1_01
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Выполняем запрос от имени пользователя u_0_10 с меткой {0,2}
-- Ожидается ошибка доступа
\connect - u_0_10
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Выполняем запрос от имени пользователя u_1_10 с меткой {1,2}
-- Ожидается ошибка доступа
\connect - u_1_10
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Выполняем запрос от имени пользователя u_1_11 с меткой {1,3}
-- Ожидается 3 записи
\connect - u_1_11
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Выполняем запрос от имени пользователя u_2_11 с меткой {2,3}
-- Ожидается 3 записи
\connect - u_2_11
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Выполняем запрос от имени администратора dbsa
-- Ожидается 3 записи
\connect - dbsa
SELECT * FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверка аггрегатных функций для таблицы с признаком MAC CCR
--
-- Выполняем запрос от имени суперпользователя postgres
\connect - postgres
-- Ожидается 3 записи
SELECT maclabel, * FROM "Проверка";
SELECT count(*) FROM "Проверка";
-- Ожидается 1
SELECT count(relname) FROM pg_class where relname = 'Проверка';
-- Ожидается true
SELECT EXISTS(select relname FROM pg_class where relname = 'Проверка');
--
-- Выполняем запрос от имени пользователя u_0_00 с меткой {0,0}
\connect - u_0_00
-- Ожидается ошибка доступа
SELECT count(*) FROM "Проверка";
-- Ожидается 0
SELECT count(relname) FROM pg_class where relname = 'Проверка';
-- Ожидается false
SELECT EXISTS(select relname FROM pg_class where relname = 'Проверка');
--
-- Выполняем запрос от имени пользователя u_1_01 с меткой {1,1}
\connect - u_1_01
-- Ожидается 3
SELECT count(*) FROM "Проверка";
-- Ожидается 1
SELECT count(relname) FROM pg_class where relname = 'Проверка';
-- Ожидается true
SELECT EXISTS(select relname FROM pg_class where relname = 'Проверка');
--
-- Проверка запросов к системному каталогу
--
-- Выполняем запрос от имени суперпользователя postgres
\connect - postgres
-- Ожидается 1 запись
SELECT maclabel, relname FROM pg_class where relname = 'Проверка';
SELECT relname FROM pg_class where relname = 'Проверка';
-- Ожидается true
SELECT EXISTS(SELECT * FROM pg_class where relname = 'Проверка');
--
-- Выполняем запрос от имени пользователя u_0_00 с меткой {0,0}
\connect - u_0_00
-- Ожидается пустая выборка
SELECT maclabel, relname FROM pg_class where relname = 'Проверка';
SELECT relname FROM pg_class where relname = 'Проверка';
-- Ожидается false
SELECT EXISTS(SELECT * FROM pg_class where relname = 'Проверка');
--
-- Выполняем запрос от имени пользователя u_1_01 с меткой {1,1}
\connect - u_1_01
-- Ожидается 1 запись
SELECT maclabel, relname FROM pg_class where relname = 'Проверка';
SELECT relname FROM pg_class where relname = 'Проверка';
-- Ожидается true
SELECT EXISTS(SELECT * FROM pg_class where relname = 'Проверка');
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'