-- RBT-TEST - МПРД при доступе к таблицам и видам
-- Проверка работы МПРД при доступе к защищенным таблицам и видам
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
--
-- Подключаемся как пользователь с меткой {0,0}
\connect - u_0_00;
--
-- Создаем таблицу с меткой по умолчанию ({0,0}) и без меток записей
CREATE TABLE u_0_00_table (
    id   INTEGER PRIMARY KEY,
    data TEXT
);
--
-- Отключаем дискреционный контроль доступа к таблице
GRANT ALL ON u_0_00_table TO PUBLIC;
--
-- Виды и материализованные виды не могут быть созданы
-- с явным указанием защиты (отсутствия) меток на строках,
-- т.к. они создаются изначально *без* меток на строках
-- Ожидается ошибка
CREATE VIEW u_0_00_view WITH (MACS=TRUE) AS SELECT id, data FROM u_0_00_table;
-- Ожидается ошибка
CREATE VIEW u_0_00_view WITH (MACS=FALSE) AS SELECT id, data FROM u_0_00_table;
-- Создаем вид u_0_00_view для таблицы с меткой по умолчанию ({0,0}) и правила insert, update и delete для него
-- Ожидается успех
CREATE VIEW u_0_00_view AS SELECT id, data FROM u_0_00_table;
CREATE RULE u_0_00_view_insert AS
    ON INSERT TO u_0_00_view
        DO INSTEAD INSERT INTO u_0_00_table
            VALUES (COALESCE ((SELECT max (id) + 1 FROM u_0_00_table), 0), new.data);
CREATE RULE u_0_00_view_update AS
    ON UPDATE TO u_0_00_view
        DO INSTEAD UPDATE u_0_00_table SET data = new.data WHERE id = old.id;
CREATE RULE u_0_00_view_delete AS
    ON DELETE TO u_0_00_view
        DO INSTEAD DELETE FROM u_0_00_table WHERE id = old.id;
--
-- Отключаем дискреционный контроль доступа к виду
GRANT ALL ON u_0_00_view TO PUBLIC;
--
-- Проверяем, что сам пользователь имеет полный доступ к таблице и виду
-- Вставка данных
-- Ожидается успех
INSERT INTO u_0_00_table VALUES (1, USER);
INSERT INTO u_0_00_view (data) VALUES (USER || '-view');
--
-- Проверяем, что записи были добавлены
-- И в таблице и в виде ожидается естественно 2 записи
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Обновление данных через таблицу
-- Ожидается успех
UPDATE u_0_00_table SET data = data || '+table+' || USER;
--
-- Проверяем, что записи были изменены
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Обновление данных через вид
UPDATE u_0_00_view SET data = data || '+view+' || USER;
--
-- Проверяем, что записи были изменены
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Проверка удаления данных из таблицы
-- Ожидается успех
DELETE FROM u_0_00_table WHERE data !~ '-view';
--
-- Проверяем удаление одной записи
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Проверка удаления данных через вид
DELETE FROM u_0_00_view WHERE data ~ '-view';
--
-- Проверяем, что все записи были удалены
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Вставим еще раз данные
-- Ожидается успех
INSERT INTO u_0_00_table VALUES (1, USER);
INSERT INTO u_0_00_view (data) VALUES (USER || '-view');
--
-- Проверяем, что записи были добавлены
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Подключаемся как пользователь с меткой {1,3}
\connect - u_1_11;
--
-- Проверяем, что его доступ к таблице и виду ограничены правилами мандатного контроля доступа
-- Вставка данных
-- Ожидается ошибка доступа
INSERT INTO u_0_00_table VALUES (1, USER);
INSERT INTO u_0_00_view (data) VALUES (USER || '-view');
--
-- Проверяем, что записи добавлены не были
-- Ожидается 2 записи, созданные пользователем u_0_00
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Обновление данных через таблицу
-- Ожидается ошибка доступа
UPDATE u_0_00_table SET data = data || '+table+' || USER;
--
-- Проверяем, что записи изменены не были
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Обновление данных через вид
-- Ожидается ошибка доступа
UPDATE u_0_00_view SET data = data || '+view+' || USER;
--
-- Проверяем, что записи изменены не были
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Проверка удаления данных из таблицы
-- Ожидается ошибка доступа
DELETE FROM u_0_00_table WHERE data !~ '-view';
--
-- Проверяем, что записи удалены не были
-- Ожидается 2 записи, созданные пользователем u_0_00
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Проверка удаления данных через вид
-- Ожидается ошибка доступа
DELETE FROM u_0_00_view WHERE data ~ '-view';
--
-- Проверяем, что записи удалены не были
-- Ожидается 2 записи, созданные пользователем u_0_00
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Изменяем метку вида
\connect - postgres
MAC LABEL ON VIEW u_0_00_view IS '{1,3}';
--
-- Снова проверяем доступ к данным пользователя с меткой {1,3}
-- Характер доступа не изменился, т.к. таблица все еще защищена
\connect - u_1_11
--
-- Вставка данных
-- Ожидается ошибка доступа
INSERT INTO u_0_00_table VALUES (1, USER);
INSERT INTO u_0_00_view (data) VALUES (USER || '-view');
--
-- Проверяем, что записи добавлены не были
-- Ожидается 2 записи, созданные пользователем u_0_00
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Обновление данных через таблицу
-- Ожидается ошибка доступа
UPDATE u_0_00_table SET data = data || '+table+' || USER;
--
-- Проверяем, что записи изменены не были
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Обновление данных через вид
-- Ожидается ошибка доступа
UPDATE u_0_00_view SET data = data || '+view+' || USER;
--
-- Проверяем, что записи изменены не были
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Проверка удаления данных из таблицы
-- Ожидается ошибка доступа
DELETE FROM u_0_00_table WHERE data !~ '-view';
--
-- Проверяем, что записи удалены не были
-- Ожидается 2 записи, созданные пользователем u_0_00
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Проверка удаления данных через вид
-- Ожидается ошибка доступа
DELETE FROM u_0_00_view WHERE data ~ '-view';
--
-- Проверяем, что записи удалены не были
-- Ожидается 2 записи, созданные пользователем u_0_00
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Создаем вид от имени пользователя с атрибутами игнорирования мандатного контроля и отключаем мандатную защиту для него
\connect - dbsa
CREATE VIEW su_0_00_view AS SELECT id, data FROM u_0_00_table;
CREATE RULE su_0_00_view_insert AS
    ON INSERT TO su_0_00_view
        DO INSTEAD INSERT INTO u_0_00_table
            VALUES (COALESCE ((SELECT max (id) + 1 FROM u_0_00_table), 0), new.data);
CREATE RULE su_0_00_view_update AS
    ON UPDATE TO su_0_00_view
        DO INSTEAD UPDATE u_0_00_table SET data = new.data WHERE id = old.id;
CREATE RULE su_0_00_view_delete AS
    ON DELETE TO su_0_00_view
        DO INSTEAD DELETE FROM u_0_00_table WHERE id = old.id;
--
-- Отключаем дискреционный контроль доступа
GRANT ALL ON su_0_00_view TO PUBLIC;
--
-- Снова проверяем доступ к данным пользователя с меткой {1,3}
-- Характер доступа не изменился, т.к. таблица все еще защищена
\connect - u_1_11
--
-- Вставка данных
-- Ожидается ошибка доступа
INSERT INTO u_0_00_table VALUES (1, USER);
INSERT INTO u_0_00_view (data) VALUES (USER || '-view');
--
-- Проверяем, что записи добавлены не были
-- Ожидается 2 записи, созданные пользователем u_0_00
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Обновление данных через таблицу
-- Ожидается ошибка доступа
UPDATE u_0_00_table SET data = data || '+table+' || USER;
--
-- Проверяем, что записи изменены не были
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Обновление данных через вид
-- Ожидается ошибка доступа
UPDATE u_0_00_view SET data = data || '+view+' || USER;
--
-- Проверяем, что записи изменены не были
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Проверка удаления данных из таблицы
-- Ожидается ошибка доступа
DELETE FROM u_0_00_table WHERE data !~ '-view';
--
-- Проверяем, что записи удалены не были
-- Ожидается 2 записи, созданные пользователем u_0_00
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Проверка удаления данных через вид
-- Ожидается ошибка доступа
DELETE FROM u_0_00_view WHERE data ~ '-view';
--
-- Проверяем, что записи удалены не были
-- Ожидается 2 записи, созданные пользователем u_0_00
SELECT * FROM u_0_00_table;
SELECT * FROM u_0_00_view;
--
-- Подключаемся как пользователь с меткой {1,3}
\connect - u_1_11;
--
-- Создаем таблицу с меткой по умолчанию {1,3}
CREATE TABLE u_1_11_table (
    id   INTEGER PRIMARY KEY,
    data TEXT
) WITH (MACS=TRUE);
--
-- Отключаем дискреционный контроль доступа к таблице
GRANT ALL ON u_1_11_table TO PUBLIC;
--
-- Создаем вид u_1_11_view для таблицы с меткой по умолчанию {1,3} и правила insert, update и delete для него
CREATE VIEW u_1_11_view AS SELECT id, data FROM u_1_11_table;
CREATE RULE u_1_11_view_insert AS
    ON INSERT TO u_1_11_view
        DO INSTEAD INSERT INTO u_1_11_table
            VALUES (COALESCE ((SELECT max (id) + 1 FROM u_1_11_table), 0), new.data);
CREATE RULE u_1_11_view_update AS
    ON UPDATE TO u_1_11_view
        DO INSTEAD UPDATE u_1_11_table SET data = new.data WHERE id = old.id;
CREATE RULE u_1_11_view_delete AS
    ON DELETE TO u_1_11_view
        DO INSTEAD DELETE FROM u_1_11_table WHERE id = old.id;
--
-- Отключаем дискреционный контроль доступа к виду
GRANT ALL ON u_1_11_view TO PUBLIC;
--
-- Проверяем, что сам пользователь имеет полный доступ к таблице и виду
-- Вставка данных
-- Ожидается успех
INSERT INTO u_1_11_table VALUES (1, USER);
INSERT INTO u_1_11_view (data) VALUES (USER || '-view');
--
-- Проверяем, что записи были добавлены
-- И в таблице и в виде ожидается естественно 2 записи
SELECT * FROM u_1_11_table;
SELECT * FROM u_1_11_view;
--
-- Обновление данных через таблицу
-- Ожидается успех
UPDATE u_1_11_table SET data = data || '+table+' || USER;
--
-- Проверяем, что записи были изменены
SELECT * FROM u_1_11_table;
SELECT * FROM u_1_11_view;
--
-- Обновление данных через вид
UPDATE u_1_11_view SET data = data || '+view+' || USER;
--
-- Проверяем, что записи были изменены
SELECT * FROM u_1_11_table;
SELECT * FROM u_1_11_view;
--
-- Проверка удаления данных из таблицы
-- Ожидается успех
DELETE FROM u_1_11_table WHERE data !~ '-view';
--
-- Проверяем удаление одной записи
SELECT * FROM u_1_11_table;
SELECT * FROM u_1_11_view;
--
-- Проверка удаления данных через вид
DELETE FROM u_1_11_view WHERE data ~ '-view';
--
-- Проверяем, что все записи были удалены
SELECT * FROM u_1_11_table;
SELECT * FROM u_1_11_view;
--
-- Вставка данных
INSERT INTO u_1_11_table VALUES (0, USER);
--
-- Подключаемся как пользователь с меткой {0,0}
\connect - u_0_00;
--
-- Проверяем, что его доступ к таблице и виду ограничены правилами мандатного контроля доступа
-- ожидается, что объекты недоступны для пользователя
INSERT INTO u_1_11_table VALUES (1, USER);
INSERT INTO u_1_11_view (data) VALUES (USER || '-view');
SELECT * FROM u_1_11_table;
SELECT * FROM u_1_11_view;
UPDATE u_1_11_table SET data = data || '+table+' || USER;
UPDATE u_1_11_view SET data = data || '+view+' || USER;
DELETE FROM u_1_11_table WHERE data !~ '-view';
DELETE FROM u_1_11_view WHERE data ~ '-view';
--
--
-- Подключаемся как пользователь администратор кластера
\connect - postgres;
--
-- Сбрасываем признак MAC CCR у таблицы u_1_11_table
MAC CCR ON TABLE u_1_11_table IS OFF;
--
-- Подключаемся как пользователь с меткой {0,0}
\connect - u_0_00;
--
-- Вставка данных через таблицу
-- Ожидается успех
INSERT INTO u_1_11_table VALUES (1, USER);
--
-- Вставка данных через вид
-- Ожидается ошибка доступа, так как вид недоступен для пользователя
INSERT INTO u_1_11_view (data) VALUES (USER || '-view');
--
-- Проверяем, что пользователь с меткой {0,0} может выбрать данные из таблицы
-- Ожидается одна запись
SELECT * FROM u_1_11_table;
--
-- вид недоступен
SELECT * FROM u_1_11_view;
--
-- Обновление данных через таблицу
-- Ожидается успех
UPDATE u_1_11_table SET data = data || '+table+' || USER;
SELECT * FROM u_1_11_table;
--
-- Обновление данных через вид
-- Ожидается ошибка доступа
UPDATE u_1_11_view SET data = data || '+view+' || USER;
--
-- Проверка удаления данных через вид
-- Ожидается ошибка доступа
DELETE FROM u_1_11_view WHERE data ~ '-view';
--
-- Проверка удаления данных из таблицы
-- Ожидается успех
DELETE FROM u_1_11_table WHERE data !~ '-view';
--
-- Проверяем, что записи пользователя u_0_00 удалены
\connect - u_1_11;
SELECT * FROM u_1_11_table;
SELECT * FROM u_1_11_view;
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'