-- RBT-TEST - МПРД в триггерах на Untrusted PL/Tcl
-- Проверка работы МПРД в триггерах на языке Untrusted PL/Tcl
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
\i 'support/init-pltclu.sql'
--
-- =========================================================
-- Тестируем триггеры, создаваемые от пользователя postgres
-- =========================================================
--
-- Проверяем работу триггера в таблице без признака MAC CCR
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Подключается суперпользователь postgres
\connect - postgres
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами вызывающего пользователя
CREATE FUNCTION "ТрФункUpdate_postgres"() RETURNS trigger LANGUAGE pltclu AS $$
  set NEW(данные) [join [list $NEW(данные) ТрФункUpdate] -]
  return [array get NEW]
$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФункUpdate_postgres" () TO PUBLIC;
--
-- Создаем триггер
CREATE TRIGGER "ТриггерBU_postgres"
  BEFORE UPDATE
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункUpdate_postgres"();
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами вызывающего пользователя
CREATE FUNCTION "ТрФункChmac_postgres"() RETURNS trigger LANGUAGE pltclu AS $$
  set NEW(данные) [join [list $NEW(данные) ТрФункChmac] -]
  return [array get NEW]
$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФункChmac_postgres" () TO PUBLIC;
--
-- Создаем триггер
CREATE TRIGGER "ТриггерBC_postgres"
  BEFORE CHMAC
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункChmac_postgres"();
--
-- Просматриваем метку сессии
-- Ожидается {0,0}
SELECT session_maclabel;
-- Просматриваем метки созданных объектов
-- Ожидается метки функций -- {0,0}
-- Метка триггеров -- {2,3}
SELECT maclabel, proname FROM pg_proc WHERE proname LIKE 'ТрФунк%' ORDER BY oid;
SELECT maclabel, tgname FROM pg_trigger WHERE tgname LIKE 'Триггер%' ORDER BY oid;
--
-- Проверяем модификацию данных в таблице без признака MAC CCR
-- Пользователи меняют данные
-- Ожидается, что метки записей не изменятся
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_2_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем модификацию данных администратором
-- и пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
\connect - postgres
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что обновление данных никак не повлияло на их зашиту
-- Ожидается, что суперюзер postgres и пользователь dbsa с флагами игнорирования МПРД
-- CAP_IGNMACLVL и CAP_IGNMACCAT смогли обновить все записи,
-- остальные пользователи смогли обновить только записи своего уровня,
-- т.е. не смогли осуществить запись выше своего уровня или ниже
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем модификацию метки пользователем с атрибутами игнорирования МПРД
-- Ожидается ошибка
UPDATE "Проверка" SET maclabel = '{1,2}', "данные" = "данные" || '+CHMAC' WHERE maclabel > '{0,0}';
--
-- Проверяем модификацию метки пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
CHMAC "Проверка" SET maclabel = '{1,2}' WHERE maclabel > '{0,0}';
--
\connect - postgres
-- Проверяем, что метки у строк с maclabel > '{1,2}' изменились
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Удаляем таблицу Проверка
DROP TABLE "Проверка" CASCADE;
-- Удаляем триггерные функции
DROP FUNCTION "ТрФункUpdate_postgres"();
DROP FUNCTION "ТрФункChmac_postgres"();
--
-- =====================================================
-- Тестируем триггеры, создаваемые от пользователя dbsa
-- =====================================================
--
\connect - dbsa
-- Создаем тестовую таблицу
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Устанавливаем метку сессии
SET ac_session_maclabel = '{1,3}';
-- Просматриваем метку сессии
-- Ожидается {1,3}
SELECT session_maclabel;
--
\connect - postgres
--
-- Создаем триггерные функции, выполняющующиеся с атрибутами создавшего пользователя
CREATE FUNCTION "ТрФункUpdate_dbsa"() RETURNS trigger LANGUAGE pltclu AS $$
  set NEW(данные) [join [list $NEW(данные) ТрФункUpdate] -]
  return [array get NEW]
$$ SECURITY DEFINER;
--
CREATE FUNCTION "ТрФункChmac_dbsa"() RETURNS trigger LANGUAGE pltclu AS $$
  set NEW(данные) [join [list $NEW(данные) ТрФункChmac] -]
  return [array get NEW]
$$ SECURITY DEFINER;
--
ALTER FUNCTION "ТрФункUpdate_dbsa"() OWNER TO dbsa;
ALTER FUNCTION "ТрФункChmac_dbsa"() OWNER TO dbsa;
--
-- Устанавливаем мандатные метки функций согласно уровню пользователя dbsa
MAC LABEL ON FUNCTION "ТрФункUpdate_dbsa"() IS '{1,3}';
MAC LABEL ON FUNCTION "ТрФункChmac_dbsa"() IS '{1,3}';
--
\connect - dbsa
--
-- Создаем триггер
-- Ожидается предупреждение о нарушении работы триггеров
CREATE TRIGGER "ТриггерBU_dbsa"
  BEFORE UPDATE
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункUpdate_dbsa"();
--
--
-- Создаем триггер
-- Ожидается предупреждение о нарушении работы триггеров
CREATE TRIGGER "ТриггерBC_dbsa"
  BEFORE CHMAC
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункChmac_dbsa"();
--
-- Просматриваем метки созданных объектов
-- Ожидаются метки функций -- {1,3}
-- Метки триггеров -- {2,3}
SELECT maclabel, proname FROM pg_proc WHERE proname LIKE 'ТрФунк%' ORDER BY oid;
SELECT maclabel, tgname FROM pg_trigger WHERE tgname LIKE 'Триггер%' ORDER BY oid;
--
-- Назначаем права на триггерные функции
\connect - postgres
GRANT EXECUTE ON FUNCTION "ТрФункUpdate_dbsa" () TO PUBLIC;
GRANT EXECUTE ON FUNCTION "ТрФункChmac_dbsa" () TO PUBLIC;
--
-- Пытаемся обновить записи таблицы
-- Ожидается сообщение с cache lookup failed для всех
-- пользователей, метка которых меньше {1,3} -- т.к. триггерная функция
-- имеет метку {1,3}
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_2_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем модификацию данных администратором
-- и пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
\connect - postgres
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что обновление данных никак не повлияло на их зашиту
-- Ожидается, что суперюзер postgres и пользователь dbsa с флагами игнорирования МПРД
-- CAP_IGNMACLVL и CAP_IGNMACCAT смогли обновить все записи,
-- остальные пользователи смогли обновить только записи своего уровня,
-- т.е. не смогли осуществить запись выше своего уровня или ниже
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем модификацию метки пользователем с атрибутами игнорирования МПРД
-- Ожидается ошибка
UPDATE "Проверка" SET maclabel = '{1,2}', "данные" = "данные" || '+CHMAC' WHERE maclabel > '{0,0}';
--
-- Проверяем модификацию метки пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
CHMAC "Проверка" SET maclabel = '{1,2}' WHERE maclabel > '{0,0}';
--
\connect - postgres
-- Проверяем, что метки у строк с maclabel > '{1,2}' изменились
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
-- Удаляем триггеры
DROP TRIGGER "ТриггерBU_dbsa" ON "Проверка";
DROP TRIGGER "ТриггерBC_dbsa" ON "Проверка";
-- Удаляем триггерные функции
DROP FUNCTION "ТрФункUpdate_dbsa"();
DROP FUNCTION "ТрФункChmac_dbsa"();
--
-- =====================================================
-- Тестируем триггеры, созданные от пользователя u_0_00
-- =====================================================
--
\connect - u_0_00
-- Просматриваем метку пользователя
-- Ожидается {0,0}
SELECT session_maclabel;
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами создавшего пользователя
-- Ожидается успех
\connect - postgres
--
CREATE FUNCTION "ТрФункUpdate_u00"() RETURNS trigger LANGUAGE pltclu AS $$
  set NEW(данные) [join [list $NEW(данные) ТрФункUpdate] -]
  return [array get NEW]
$$ SECURITY DEFINER;
--
ALTER FUNCTION "ТрФункUpdate_u00"() OWNER TO u_0_00;
--
\connect - u_0_00
--
-- Создаем триггер
-- Ожидается ошибка доступа,
-- т.к. метка пользователя не совпадает с меткой таблицы Проверка
CREATE TRIGGER "ТриггерBU_u00"
  BEFORE UPDATE
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункUpdate_u00"();
--
-- Удаляем триггерную функцию
DROP FUNCTION "ТрФункUpdate_u00"();
--
-- =====================================================
-- Тестируем триггеры, созданные от пользователя u_2_11
-- =====================================================
--
\connect - u_2_11
-- Просматриваем метку пользователя
-- Ожидается {2,3}
SELECT session_maclabel;
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами создавшего пользователя
-- Ожидается успех
\connect - postgres
--
CREATE FUNCTION "ТрФункUpdate_u23"() RETURNS trigger LANGUAGE pltclu AS $$
  set NEW(данные) [join [list $NEW(данные) ТрФункUpdate] -]
  return [array get NEW]
$$ SECURITY DEFINER;
--
ALTER FUNCTION "ТрФункUpdate_u23"() OWNER TO u_2_11;
--
MAC LABEL ON FUNCTION "ТрФункUpdate_u23"() IS '{2,3}';
--
\connect - u_2_11
-- Создаем триггер
-- Ожидается успех
-- Ожидается предупреждение о нарушении работы триггеров
CREATE TRIGGER "ТриггерBU_u23"
  BEFORE UPDATE
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункUpdate_u23"();
--
\connect - postgres
-- Удаляем триггер и триггерную функцию
DROP TRIGGER "ТриггерBU_u23" ON "Проверка";
DROP FUNCTION "ТрФункUpdate_u23"();
--
-- =======================================================================
-- Проверяем BEFORE INSERT триггеры (без явной установки мандатной метки)
-- =======================================================================
--
\connect - postgres
-- Очищаем таблицу Проверка
--
DELETE FROM "Проверка";
--
-- Создаем триггер BEFORE INSERT
--
CREATE FUNCTION "ТрФункINSERT"() RETURNS trigger LANGUAGE pltclu AS $$
  set NEW(данные) [join [list $NEW(данные) ТрФункINSERT] -]
  return [array get NEW]
$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФункINSERT" () TO PUBLIC;
--
CREATE TRIGGER "ТриггерBI"
  BEFORE INSERT
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункINSERT"();
--
-- Выполняем вставку данных в таблицу
-- Ожидается успех
\connect - u_0_00
INSERT INTO "Проверка" VALUES(1, 'данные ' || USER);
\connect - u_0_01
INSERT INTO "Проверка" VALUES(2, 'данные ' || USER);
\connect - u_1_01
INSERT INTO "Проверка" VALUES(3, 'данные ' || USER);
\connect - u_0_10
INSERT INTO "Проверка" VALUES(4, 'данные ' || USER);
\connect - u_1_10
INSERT INTO "Проверка" VALUES(5, 'данные ' || USER);
\connect - u_1_11
INSERT INTO "Проверка" VALUES(6, 'данные ' || USER);
\connect - u_2_11
INSERT INTO "Проверка" VALUES(7, 'данные ' || USER);
--
-- Проверяем вставку данных администратором
-- и пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
\connect - postgres
INSERT INTO "Проверка" VALUES(8, 'данные ' || USER);
\connect - dbsa
INSERT INTO "Проверка" VALUES(9, 'данные ' || USER);
--
-- Проверяем срабатывание триггера при добавлении записей от непривилегированного пользователя
\connect - u_1_01
-- Ожидается ошибка доступа
INSERT INTO "Проверка"(maclabel, "идентификатор", "данные") VALUES(session_maclabel, 10, 'данные ' || USER);
-- Ожидается ошибка доступа
INSERT INTO "Проверка"(maclabel, "идентификатор", "данные") VALUES('{0,0}', 11, 'данные ' || USER);
-- Ожидается ошибка доступа
INSERT INTO "Проверка"(maclabel, "идентификатор", "данные") VALUES('{1,1}', 12, 'данные ' || USER);
-- Ожидается ошибка доступа
INSERT INTO "Проверка"(maclabel, "идентификатор", "данные") VALUES('{1,2}', 13, 'данные ' || USER);
--
\connect - postgres
-- Проверяем, что записи были успешно вставлены в таблицу Проверка
-- с теми метками, под которыми зашли пользователи
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Удаляем триггер и триггерную функцию
DROP TRIGGER "ТриггерBI" ON "Проверка";
DROP FUNCTION "ТрФункINSERT"();
-- Очищаем таблицу проверка
DELETE FROM "Проверка";
--
-- =======================================================================
-- Проверяем BEFORE INSERT триггеры (c явной установкой мандатной метки)
-- =======================================================================
--
\connect - postgres
-- Очищаем таблицу Проверка
--
DELETE FROM "Проверка";
--
-- Создаем триггер BEFORE INSERT
--
CREATE FUNCTION "ТрФункINSERT"() RETURNS trigger LANGUAGE pltclu AS $$
  set NEW(данные) [join [list $NEW(данные) ТрФункINSERT] -]
  set NEW(maclabel) $AC_SESSION_MACLABEL
  return [array get NEW]
$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФункINSERT" () TO PUBLIC;
--
CREATE TRIGGER "ТриггерBI"
  BEFORE INSERT
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункINSERT"();
--
-- Выполняем вставку данных в таблицу
-- Ожидается успех
\connect - u_0_00
INSERT INTO "Проверка" VALUES(1, 'данные ' || USER);
\connect - u_0_01
INSERT INTO "Проверка" VALUES(2, 'данные ' || USER);
\connect - u_1_01
INSERT INTO "Проверка" VALUES(3, 'данные ' || USER);
\connect - u_0_10
INSERT INTO "Проверка" VALUES(4, 'данные ' || USER);
\connect - u_1_10
INSERT INTO "Проверка" VALUES(5, 'данные ' || USER);
\connect - u_1_11
INSERT INTO "Проверка" VALUES(6, 'данные ' || USER);
\connect - u_2_11
INSERT INTO "Проверка" VALUES(7, 'данные ' || USER);
--
-- Проверяем вставку данных администратором
-- и пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
\connect - postgres
INSERT INTO "Проверка" VALUES(8, 'данные ' || USER);
\connect - dbsa
INSERT INTO "Проверка" VALUES(9, 'данные ' || USER);
--
\connect - postgres
-- Проверяем, что записи были успешно вставлены в таблицу Проверка
-- с теми метками, под которыми зашли пользователи
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Удаляем триггер и триггерную функцию
DROP TRIGGER "ТриггерBI" ON "Проверка";
DROP FUNCTION "ТрФункINSERT"();
-- Очищаем таблицу проверка
DELETE FROM "Проверка";
--
-- Проверяем возможность маркировки записей триггером определенной меткой
--
-- Создаем триггер BEFORE INSERT
--
CREATE FUNCTION "ТрФункINSERT"() RETURNS trigger LANGUAGE pltclu AS $$
  set NEW(данные) [join [list $NEW(данные) ТрФункINSERT] -]
  set NEW(maclabel) "{1,2}"
  return [array get NEW]
$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФункINSERT" () TO PUBLIC;
--
CREATE TRIGGER "ТриггерBI"
  BEFORE INSERT
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункINSERT"();
--
-- Выполняем вставку данных в таблицу
-- Ожидается успех для всех пользователей, метка которых совпадает с {1,2}
\connect - u_0_00
INSERT INTO "Проверка" VALUES(1, 'данные ' || USER);
\connect - u_0_01
INSERT INTO "Проверка" VALUES(2, 'данные ' || USER);
\connect - u_1_01
INSERT INTO "Проверка" VALUES(3, 'данные ' || USER);
\connect - u_0_10
INSERT INTO "Проверка" VALUES(4, 'данные ' || USER);
\connect - u_1_10
INSERT INTO "Проверка" VALUES(5, 'данные ' || USER);
\connect - u_1_11
INSERT INTO "Проверка" VALUES(6, 'данные ' || USER);
\connect - u_2_11
INSERT INTO "Проверка" VALUES(7, 'данные ' || USER);
--
-- Проверяем вставку данных администратором
-- и пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
\connect - postgres
INSERT INTO "Проверка" VALUES(8, 'данные ' || USER);
\connect - dbsa
INSERT INTO "Проверка" VALUES(9, 'данные ' || USER);
--
\connect - postgres
-- Проверяем, что записи были успешно вставлены в таблицу Проверка
-- ожидается 3 записи
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Удаляем триггер и триггерную функцию
DROP TRIGGER "ТриггерBI" ON "Проверка";
DROP FUNCTION "ТрФункINSERT"();
-- Очищаем таблицу проверка
DELETE FROM "Проверка";
--
-- =================================
-- Проверяем BEFORE DELETE триггеры
-- =================================
--
-- Наполняем таблицу Проверка данными
\i 'support/init-test-table.sql'
--
-- Создаем триггер BEFORE DELETE
--
\connect - postgres
-- Создаем временную таблицу для результатов удаления
CREATE TABLE "Результаты" ("идентификатор" INTEGER, "пользователь" TEXT, "данные" TEXT) WITH (MACS=TRUE);
MAC LABEL ON TABLE "Результаты" IS '{2,3}';
MAC CCR ON TABLE "Результаты" IS OFF;
GRANT ALL ON TABLE "Результаты" TO PUBLIC;
--
CREATE FUNCTION "ТрФункDELETE"() RETURNS trigger LANGUAGE pltclu AS $$
  set OLD(данные) [join [list $OLD(данные) ТрФункDELETE] -]
  set query [join [list "INSERT INTO \"Результаты\" VALUES(" $OLD(идентификатор) ", '" $AC_USER_NAME "', '" $OLD(данные) "');"]]
  spi_exec $query
  return [array get OLD]
$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФункDELETE" () TO PUBLIC;
--
CREATE TRIGGER "ТриггерBD"
  BEFORE DELETE
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункDELETE"();
--
-- Выполняем удаление данных из таблицы
-- Ожидается успех
\connect - u_0_00
DELETE FROM "Проверка";
\connect - u_0_01
DELETE FROM "Проверка";
\connect - u_1_01
DELETE FROM "Проверка";
\connect - u_0_10
DELETE FROM "Проверка";
\connect - u_1_10
DELETE FROM "Проверка";
\connect - u_1_11
DELETE FROM "Проверка";
\connect - u_2_11
DELETE FROM "Проверка";
--
-- Проверяем удаление данных администратором
-- и пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
\connect - postgres
DELETE FROM "Проверка";
\connect - dbsa
DELETE FROM "Проверка";
--
\connect - postgres
-- Проверяем, что записи были успешно удалены
-- Ожидается 0 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Ожидается, что записи удаляются согласно уровню и привилегиям пользователя
SELECT maclabel, * FROM "Результаты" ORDER BY "идентификатор";
-- Удаляем триггер и триггерную функцию
DROP TRIGGER "ТриггерBD" ON "Проверка";
DROP FUNCTION "ТрФункDELETE"();
--
-- =================================
-- Проверяем BEFORE TRUNCATE триггеры
-- =================================
--
-- Наполняем таблицу Проверка данными
\i 'support/init-test-table.sql'
--
\connect - postgres
-- Очищаем таблицу результатов
DELETE FROM "Результаты";
--
-- Создаем триггер BEFORE TRUNCATE
--
\connect - postgres
--
CREATE FUNCTION "ТрФункTRUNCATE"() RETURNS trigger LANGUAGE pltclu AS $$
  set message [join [list "Триггер:" $TG_name "пользователь:" $AC_USER_NAME "таблица:" $TG_table_schema"."$TG_table_name]]
  elog NOTICE $message
  return OK;
$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФункTRUNCATE" () TO PUBLIC;
--
CREATE TRIGGER "ТриггерBT"
  BEFORE TRUNCATE
  ON "Проверка"
  FOR EACH STATEMENT
  EXECUTE PROCEDURE "ТрФункTRUNCATE"();
--
-- Выполняем очистку таблицы Проверка
-- Ожидается отказ доступа для всех пользователей,
-- метка которых меньше метки таблицы ({2,3})
-- и успех, у кого метка пользователя больше метки таблицы
\connect - u_0_00
TRUNCATE "Проверка";
-- Просматриваем записи таблицы
\connect - postgres
-- Ожидается 10 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
\connect - u_0_01
TRUNCATE "Проверка";
-- Просматриваем записи таблицы
\connect - postgres
-- Ожидается 10 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
\connect - u_1_01
TRUNCATE "Проверка";
-- Просматриваем записи таблицы
\connect - postgres
-- Ожидается 10 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
\connect - u_0_10
TRUNCATE "Проверка";
-- Просматриваем записи таблицы
\connect - postgres
-- Ожидается 10 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
\connect - u_1_10
TRUNCATE "Проверка";
-- Просматриваем записи таблицы
\connect - postgres
-- Ожидается 10 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
\connect - u_1_11
TRUNCATE "Проверка";
-- Просматриваем записи таблицы
\connect - postgres
-- Ожидается 10 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
\connect - u_2_11
TRUNCATE "Проверка";
-- Просматриваем записи таблицы
\connect - postgres
-- Ожидается 0 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем удаление данных администратором
-- и пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
\connect - postgres
TRUNCATE "Проверка";
\connect - dbsa
TRUNCATE "Проверка";
--
\connect - postgres
-- Проверяем, что записи были успешно удалены
-- Ожидается 0 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Удаляем триггер и триггерную фунцию
DROP TRIGGER "ТриггерBT" ON "Проверка";
DROP FUNCTION "ТрФункTRUNCATE"();
--
-- =================================
-- Проверяем AFTER INSERT триггеры
-- =================================
--
\connect - postgres
--
-- Очищаем таблицу Проверка
--
DELETE FROM "Проверка";
--
-- Создаем триггер AFTER INSERT
--
CREATE FUNCTION "ТрФункINSERT"() RETURNS trigger LANGUAGE pltclu AS $$
  set message [join [list "Триггер:" $TG_name "пользователь:" $AC_USER_NAME "таблица:" $TG_table_schema"."$TG_table_name]]
  elog NOTICE $message
  return [array get NEW]
$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФункINSERT" () TO PUBLIC;
--
CREATE TRIGGER "ТриггерAI"
  AFTER INSERT
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункINSERT"();
--
-- Выполняем вставку данных в таблицу
-- Ожидается успех
\connect - u_0_00
INSERT INTO "Проверка" VALUES(1, 'данные ' || USER);
\connect - u_0_01
INSERT INTO "Проверка" VALUES(2, 'данные ' || USER);
\connect - u_1_01
INSERT INTO "Проверка" VALUES(3, 'данные ' || USER);
\connect - u_0_10
INSERT INTO "Проверка" VALUES(4, 'данные ' || USER);
\connect - u_1_10
INSERT INTO "Проверка" VALUES(5, 'данные ' || USER);
\connect - u_1_11
INSERT INTO "Проверка" VALUES(6, 'данные ' || USER);
\connect - u_2_11
INSERT INTO "Проверка" VALUES(7, 'данные ' || USER);
--
-- Проверяем вставку данных администратором
-- и пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
\connect - postgres
INSERT INTO "Проверка" VALUES(8, 'данные ' || USER);
\connect - dbsa
INSERT INTO "Проверка" VALUES(9, 'данные ' || USER);
--
\connect - postgres
-- Проверяем, что записи были успешно вставлены в таблицу Проверка
-- с теми метками, под которыми зашли пользователи
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Удаляем триггер и триггерную функцию
DROP TRIGGER "ТриггерAI" ON "Проверка";
DROP FUNCTION "ТрФункINSERT"();
-- Очищаем таблицу проверка
DELETE FROM "Проверка";
--
-- =================================
-- Проверяем AFTER DELETE триггеры
-- =================================
--
-- Наполняем таблицу Проверка данными
\i 'support/init-test-table.sql'
--
-- Создаем триггер AFTER DELETE
--
\connect - postgres
--
CREATE FUNCTION "ТрФункDELETE"() RETURNS trigger LANGUAGE pltclu AS $$
  set OLD(данные) [join [list $OLD(данные) ТрФункDELETE] -]
  set query [join [list "INSERT INTO \"Результаты\" VALUES(" $OLD(идентификатор) ", '" $AC_USER_NAME "', '" $OLD(данные) "');"]]
  spi_exec $query
  return [array get OLD]
$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФункDELETE" () TO PUBLIC;
--
CREATE TRIGGER "ТриггерAD"
  AFTER DELETE
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункDELETE"();
--
-- Выполняем удаление данных из таблицы
-- Ожидается успех
\connect - u_0_00
DELETE FROM "Проверка";
\connect - u_0_01
DELETE FROM "Проверка";
\connect - u_1_01
DELETE FROM "Проверка";
\connect - u_0_10
DELETE FROM "Проверка";
\connect - u_1_10
DELETE FROM "Проверка";
\connect - u_1_11
DELETE FROM "Проверка";
\connect - u_2_11
DELETE FROM "Проверка";
--
-- Проверяем удаление данных администратором
-- и пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
\connect - postgres
DELETE FROM "Проверка";
\connect - dbsa
DELETE FROM "Проверка";
--
\connect - postgres
-- Проверяем, что записи были успешно удалены
-- Ожидается 0 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Ожидается, что записи удаляются согласно уровню и привилегиям пользователя
SELECT maclabel, * FROM "Результаты" ORDER BY "идентификатор";
-- Удаляем триггер и триггерную функцию
DROP TRIGGER "ТриггерAD" ON "Проверка";
DROP FUNCTION "ТрФункDELETE"();
--
-- =================================
-- Проверяем AFTER TRUNCATE триггеры
-- =================================
--
-- Наполняем таблицу Проверка данными
\i 'support/init-test-table.sql'
--
\connect - postgres
-- Очищаем таблицу результатов
DELETE FROM "Результаты";
--
-- Создаем триггер AFTER TRUNCATE
--
\connect - postgres
--
CREATE FUNCTION "ТрФункTRUNCATE"() RETURNS trigger LANGUAGE pltclu AS $$
  set message [join [list "Триггер:" $TG_name "пользователь:" $AC_USER_NAME "таблица:" $TG_table_schema"."$TG_table_name]]
  elog NOTICE $message
  return OK;
$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФункTRUNCATE" () TO PUBLIC;
--
CREATE TRIGGER "ТриггерAT"
  AFTER TRUNCATE
  ON "Проверка"
  FOR EACH STATEMENT
  EXECUTE PROCEDURE "ТрФункTRUNCATE"();
--
-- Выполняем очистку таблицы Проверка
-- Ожидается отказ доступа для всех пользователей,
-- метка которых меньше метки таблицы ({2,3})
-- и успех, у кого метка пользователя больше метки таблицы
\connect - u_0_00
TRUNCATE "Проверка";
-- Просматриваем записи таблицы
\connect - postgres
-- Ожидается 10 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
\connect - u_0_01
TRUNCATE "Проверка";
-- Просматриваем записи таблицы
\connect - postgres
-- Ожидается 10 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
\connect - u_1_01
TRUNCATE "Проверка";
-- Просматриваем записи таблицы
\connect - postgres
-- Ожидается 10 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
\connect - u_0_10
TRUNCATE "Проверка";
-- Просматриваем записи таблицы
\connect - postgres
-- Ожидается 10 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
\connect - u_1_10
TRUNCATE "Проверка";
-- Просматриваем записи таблицы
\connect - postgres
-- Ожидается 10 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
\connect - u_1_11
TRUNCATE "Проверка";
-- Просматриваем записи таблицы
\connect - postgres
-- Ожидается 10 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
\connect - u_2_11
TRUNCATE "Проверка";
-- Просматриваем записи таблицы
\connect - postgres
-- Ожидается 0 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем удаление данных администратором
-- и пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
\connect - postgres
TRUNCATE "Проверка";
\connect - dbsa
TRUNCATE "Проверка";
--
\connect - postgres
-- Проверяем, что записи были успешно удалены
-- Ожидается 0 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
-- Удаляем триггер и триггерную функцию
DROP TRIGGER "ТриггерAT" ON "Проверка";
DROP FUNCTION "ТрФункTRUNCATE"();
--
-- =====================================
-- Проверяем INSTEAD OF INSERT триггеры
-- =====================================
--
\connect - postgres
-- Создаем представление таблицы Проверка
CREATE VIEW "ПроверкаVIEW" AS SELECT * FROM "Проверка";
MAC LABEL ON VIEW "ПроверкаVIEW" IS '{2,3}';
MAC CCR ON VIEW "ПроверкаVIEW" IS OFF;
GRANT ALL ON "ПроверкаVIEW" TO PUBLIC;
--
-- Создаем триггер INSTEAD OF INSERT
CREATE FUNCTION "ТрФункINSERT"() RETURNS TRIGGER AS $$
  set NEW(данные) [join [list $NEW(данные) ТрФункINSERT] -]
  set query [join [list "INSERT INTO \"Проверка\" VALUES(" $NEW(идентификатор) ", '" $NEW(данные) "');"]]
  spi_exec $query
  return [array get NEW]
$$ LANGUAGE pltclu;
GRANT EXECUTE ON FUNCTION "ТрФункINSERT"() TO PUBLIC;
--
CREATE TRIGGER "ТриггерIOI"
  INSTEAD OF INSERT
  ON "ПроверкаVIEW"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункINSERT"();
--
-- Выполняем вставку данных в таблицу
-- Ожидается успех
\connect - u_0_00
INSERT INTO "ПроверкаVIEW" VALUES(1, 'данные ' || USER);
\connect - u_0_01
INSERT INTO "ПроверкаVIEW" VALUES(2, 'данные ' || USER);
\connect - u_1_01
INSERT INTO "ПроверкаVIEW" VALUES(3, 'данные ' || USER);
\connect - u_0_10
INSERT INTO "ПроверкаVIEW" VALUES(4, 'данные ' || USER);
\connect - u_1_10
INSERT INTO "ПроверкаVIEW" VALUES(5, 'данные ' || USER);
\connect - u_1_11
INSERT INTO "ПроверкаVIEW" VALUES(6, 'данные ' || USER);
\connect - u_2_11
INSERT INTO "ПроверкаVIEW" VALUES(7, 'данные ' || USER);
--
-- Проверяем вставку данных администратором
-- и пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
\connect - postgres
INSERT INTO "ПроверкаVIEW" VALUES(8, 'данные ' || USER);
\connect - dbsa
INSERT INTO "ПроверкаVIEW" VALUES(9, 'данные ' || USER);
--
\connect - postgres
-- Просматриваем содержимое таблицы Проверка
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
-- Просматриваем содержимое представления ПроверкаVIEW
SELECT * FROM "ПроверкаVIEW" ORDER BY "идентификатор";
-- Очищаем таблицу
DELETE FROM "Проверка";
-- Удаляем триггер и триггерную функцию
DROP TRIGGER "ТриггерIOI" ON "ПроверкаVIEW";
DROP FUNCTION "ТрФункINSERT"();
--
-- =====================================
-- Проверяем INSTEAD OF UPDATE триггеры
-- =====================================
--
\connect - postgres
--
-- Наполняем таблицу Проверка
\i 'support/init-test-table.sql'
--
\connect - postgres
--
-- Просматриваем содержимое таблицы Проверка и представления ПроверкаVIEW
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
SELECT * FROM "ПроверкаVIEW" ORDER BY "идентификатор";
--
-- Создаем триггер INSTEAD OF UPDATE
CREATE FUNCTION "ТрФункUPDATE"() RETURNS TRIGGER AS $$
  set NEW(данные) [join [list $NEW(данные) ТрФункUPDATE] -]
  set query [join [list "UPDATE \"Проверка\" SET \"данные\" = '" $NEW(данные) "' WHERE \"идентификатор\" = " $NEW(идентификатор) ";"]]
  spi_exec $query
  return OK
$$ LANGUAGE pltclu;
GRANT EXECUTE ON FUNCTION "ТрФункUPDATE"() TO PUBLIC;
--
CREATE TRIGGER "ТриггерIOU"
  INSTEAD OF UPDATE
  ON "ПроверкаVIEW"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункUPDATE"();
--
-- Выполняем обновление данных таблицы
-- Ожидается успех
\connect - u_0_00
UPDATE "ПроверкаVIEW" SET "данные" = "данные" || '+' || USER;
\connect - u_0_01
UPDATE "ПроверкаVIEW" SET "данные" = "данные" || '+' || USER;
\connect - u_1_01
UPDATE "ПроверкаVIEW" SET "данные" = "данные" || '+' || USER;
\connect - u_0_10
UPDATE "ПроверкаVIEW" SET "данные" = "данные" || '+' || USER;
\connect - u_1_10
UPDATE "ПроверкаVIEW" SET "данные" = "данные" || '+' || USER;
\connect - u_1_11
UPDATE "ПроверкаVIEW" SET "данные" = "данные" || '+' || USER;
\connect - u_2_11
UPDATE "ПроверкаVIEW" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем обновление данных администратором
-- и пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
\connect - postgres
UPDATE "ПроверкаVIEW" SET "данные" = "данные" || '+' || USER;
\connect - dbsa
UPDATE "ПроверкаVIEW" SET "данные" = "данные" || '+' || USER;
--
\connect - postgres
-- Просматриваем содержимое таблицы Проверка
-- Ожидается, что пользователи смогли обновить только свои записи
-- а администратор и пользователь с атрибутами игнорирования МПРД
-- смогли обновить все записи
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
-- Просматриваем содержимое представления ПроверкаVIEW
SELECT * FROM "ПроверкаVIEW" ORDER BY "идентификатор";
-- Очищаем таблицу Проверка
DELETE FROM "Проверка";
--
--
-- =====================================
-- Проверяем INSTEAD OF DELETE триггеры
-- =====================================
--
-- Наполняем таблицу Проверка
\i 'support/init-test-table.sql'
--
-- Создаем триггер INSTEAD OF DELETE
--
-- Очищаем таблицу Результаты
DELETE FROM "Результаты";
--
\connect - postgres
--
CREATE FUNCTION "ТрФункDELETE"() RETURNS trigger LANGUAGE pltclu AS $$
  set query [join [list "DELETE FROM \"Проверка\" WHERE \"идентификатор\" = " $OLD(идентификатор) ";"]]
  spi_exec $query
  set OLD(данные) [join [list $OLD(данные) ТрФункDELETE] -]
  set query [join [list "INSERT INTO \"Результаты\" VALUES(" $OLD(идентификатор) ", '" $AC_USER_NAME "', '" $OLD(данные) "');"]]
  spi_exec $query
  return OK
$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФункDELETE" () TO PUBLIC;
--
CREATE TRIGGER "ТриггерIOD"
  INSTEAD OF DELETE
  ON "ПроверкаVIEW"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФункDELETE"();
--
-- Выполняем удаление данных из таблицы
-- Ожидается успех
\connect - u_0_00
DELETE FROM "ПроверкаVIEW";
\connect - u_0_01
DELETE FROM "ПроверкаVIEW";
\connect - u_1_01
DELETE FROM "ПроверкаVIEW";
\connect - u_0_10
DELETE FROM "ПроверкаVIEW";
\connect - u_1_10
DELETE FROM "ПроверкаVIEW";
\connect - u_1_11
DELETE FROM "ПроверкаVIEW";
\connect - u_2_11
DELETE FROM "ПроверкаVIEW";
--
-- Проверяем удаление данных администратором
-- и пользователем с атрибутами игнорирования МПРД
-- Ожидается успех
\connect - postgres
DELETE FROM "ПроверкаVIEW";
\connect - dbsa
DELETE FROM "ПроверкаVIEW";
--
\connect - postgres
-- Проверяем, что записи были успешно удалены
-- Ожидается 0 записей
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Ожидается, что записи удаляются согласно уровню и привилегиям пользователя
SELECT maclabel, * FROM "Результаты" ORDER BY "идентификатор";
-- Удаляем триггер и триггерную функцию
DROP TRIGGER "ТриггерIOD" ON "ПроверкаVIEW";
DROP FUNCTION "ТрФункDELETE"();
--
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'
