-- RBT-TEST - ALTER TABLE
-- Проверка сохранения атрибутов безопасности при выполнении команды ALTER TABLE
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
--
-- Проверяем корректность заполнения таблицы
\i 'support/select-test-table.sql'
--
-- Добавляем колонку в таблицу
\connect - postgres
ALTER TABLE "Проверка" ADD COLUMN "Проверочная колонка" DOUBLE PRECISION;
--
-- "Проверяем, что и данные и все права на таблицу сохранились"
\i 'support/select-test-table.sql'
--
-- Переименовываем колонку в таблице
\connect - postgres
ALTER TABLE "Проверка" RENAME COLUMN "Проверочная колонка" TO "Ее новое имя";
--
-- Проверяем, что и данные и все права на таблицу сохранились
\i 'support/select-test-table.sql'
--
-- Удаляем колонку из таблицы
\connect - postgres
ALTER TABLE "Проверка" DROP COLUMN "Ее новое имя";
--
-- Проверяем, что и данные и все права на таблицу сохранились
\i 'support/select-test-table.sql'
--
-- Переименовываем таблицу
\connect - postgres
ALTER TABLE "Проверка" RENAME TO "Проверка2";
--
-- Переименовываем таблицу обратно
ALTER TABLE "Проверка2" RENAME TO "Проверка";
--
-- Проверяем, что и данные и все права на таблицу сохранились
\i 'support/select-test-table.sql'
--
-- Проверяем невозможность удаления меток у записей таблицы
--
-- Убедимся в том, что записи маркированы
\connect - postgres
SELECT maclabel, * FROM "Проверка";
-- Добавляем столбец oid к таблице 'Проверка'
-- Убеждаемся, что в таблице 'Проверка' отсутствует столбец OID
-- Ожидается одна запись
SELECT relname, relhasoids FROM pg_class WHERE relname = 'Проверка';
--
ALTER TABLE "Проверка" SET WITH OIDS;
-- Убеждаемся в том, что столбец oid добавился
-- Ожидается одна запись
SELECT relname, relhasoids FROM pg_class WHERE relname = 'Проверка';
-- Убеждаемся в том, что метка записей не изменилась
-- Ожидается 10 записей
SELECT maclabel, * FROM "Проверка";
-- Удаляем столбец oid из таблице 'Проверка'
-- Ожидается успех
ALTER TABLE "Проверка" SET WITHOUT OIDS;
-- Убеждаемся, что в таблице 'Проверка' отсутствует столбец OID
-- Ожидается одна запись
SELECT relname, relhasoids FROM pg_class WHERE relname = 'Проверка';
-- Убеждаемся в том, что метка записей не изменилась
-- Ожидается 10 записей
SELECT maclabel, * FROM "Проверка";
--
\connect - postgres
-- Просматриваем записи таблицы Проверка
SELECT maclabel, * FROM "Проверка";
-- Проверяем, что добавление столбцов с значением по умолчанию
-- к существующей таблице с метками не меняет метку записей
ALTER TABLE "Проверка" ADD COLUMN "Новый столбец" INTEGER DEFAULT 1;
-- Просматриваем записи таблицы Проверка
-- Ожидается, что метки записей не изменились
SELECT maclabel, * FROM "Проверка";
-- Проверяем, что добавление столбцов
-- к существующей таблице с метками не меняет метку записей
ALTER TABLE "Проверка" ADD COLUMN "Новый столбец 2" INTEGER;
-- Просматриваем записи таблицы Проверка
-- Ожидается, что метки записей не изменились
SELECT maclabel, * FROM "Проверка";
--
\connect - postgres
-- Модифицируем таблицу 'Проверка' -- меняем владельца на 'u_0_00'
ALTER TABLE "Проверка" OWNER TO u_0_00;
-- Проверяем возможность удаления меток у записей таблицы, если пользователь не является администратором
\connect - u_0_00
-- Ожидается ошибка доступа
ALTER TABLE "Проверка" DROP COLUMN "maclabel";
-- Администратор имеет право удалять столбец maclabel
-- Ожидается успех
\connect - postgres
ALTER TABLE "Проверка" DROP COLUMN "maclabel";
-- Убеждаемся, что в таблице 'Проверка' отсутствует столбец maclabel
-- Ожидается одна запись
SELECT relname, relhasmacs FROM pg_class WHERE relname = 'Проверка';
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'