-- RBT-TEST - Проверка параметров аудита
-- Проверка работы параметров аудита
--
\i 'support/init.sql'
\i 'support/init-config-tools.sql'
\i 'support/init-settings-info-tools.sql'
--
-- ==========================================
-- I. Проверка параметра режима работы аудита
-- ==========================================
-- Аудит может работать в следующих режимах:
-- 1. 'internal'
-- 2. 'external'
-- 3. 'internal,external'
-- 4. 'external,internal'
-- 5. 'none'
-- При этом режим не может быть изменён с помощью команды SET
--
-- Подключаемся как пользователь u_0_00
\connect - u_0_00
-- Просматриваем текущий режим работы аудита
-- Ожидается internal
SHOW ac_audit_mode;
-- Пытаемся изменить параметр
-- Ожидается ошибка
SET ac_audit_mode TO 'external';
--
-- Подключаемся как пользователь postgres
\connect - postgres
-- Просматриваем режим работы аудита
-- Ожидается internal
SHOW ac_audit_mode;
-- Пытаемся изменить параметр
-- Ожидается ошибка
SET ac_audit_mode TO 'external';
--
-- ===============================================
-- II. Проверка настроек аудита в режиме internal
-- ===============================================
--
-- Проверка работы параметра ac_default_audit -- маска аудита по умолчанию
-- (не может быть изменён администратором)
--
-- Заходим как пользователь u_1_11
\connect - u_1_11
-- Просматриваем текущий режим работы аудита
-- Ожидается internal
SHOW ac_audit_mode;
-- Просматриваем значение параметра
-- Ожидается {SsRDxCTEMce:SsRVrawdDxtXUCTEMce} -- аудит по умолчанию
SHOW ac_default_audit;
-- Пробуем обновить параметр ac_default_audit
-- от имени пользователя u_1_11
-- Ожидается ошибка
SET ac_default_audit TO '{cea:cea}';
-- Пробуем обновить параметр ac_default_audit
-- от имени администратора postgres
-- Ожидается ошибка
\connect - postgres
-- Ожидается ошибка
SET ac_default_audit TO '{cea:cea}';
-- Просматриваем значение аудита по умолчанию
-- Ожидается {SsRDxCTEMce:SsRVrawdDxtXUCTEMce}
SHOW ac_default_audit;
--
-- Проверка работы параметра ac_session_audit -- маска аудита сессии
-- (может быть изменён администратором в рамках сессии)
--
-- Заходим как пользователь u_1_11
\connect - u_1_11
-- Просматриваем значение параметра
-- Ожидается {SsRDxCTEMce:SsRVrawdDxtXUCTEMce}
SHOW ac_session_audit;
-- Обновляем значение аудита сессии
-- Ожидается ошибка доступа
SET ac_session_audit TO '{a:a}';
-- Подключаемся как postgres и меняем маску аудита
\connect - postgres
-- Ожидается ошибка
SET ac_session_audit TO '{0xFFFFFFFF:0xFFFFFFFF}';
-- Ожидается успех
SET ac_session_audit TO '{a:a}';
-- Просматриваем значение параметра маски сессии
-- Ожидается новое значение маски аудита, являющееся суперпозицией маски аудита по умолчанию {ce:ce} и значения,
-- указанного в конструкции SET: {ace:ace}
SHOW ac_session_audit;
--
-- Проверка установки маски аудита для комбинаций: база данных - пользователь, база данных - все пользователи,
-- все базы данных-пользователь, все базы данных - все пользователи.
-- При этом маска аудита, устанавливаемая с помощью команды ALTER ROLE ... SET
-- формируется как суперпозиция маски аудита {ce:ce} и маски аудита, указанной в этой команде.
-- Приоритет назначения настроек (GUC):
-- 1. База данных - пользователь
-- 2. Все базы данных - пользователь
-- 3. База данных - пользователь
-- 4. Все базы данных - все пользователь
-- При сбросе(модификации) настроек (с помощью ALTER ROLE ... RESET) обновление аудита сессии происходит по следующему алгоритму:
-- Если происходит сброс (модификация) примененной настройки, ищется менее приоритетная настройка аудита, если отсутствует, применяется аудит по умолчанию.
-- Если проихсодит сброс (модификация) настройки другого уровня, отличного от примененного, изменений на текущий аудит не последует.
-- Помимо GUC-переменной ac_session_audit для получения аудита сессии и текущего настройки аудита используются конструкции
-- SELECT session_audit и SELECT current_audit
--
\connect - u_1_11
-- Устанавливаем маску аудита сессии для пользователя-базы данных
-- Ожидается ошибка доступа, т.к. необходимы права администратора
ALTER ROLE u_1_11 IN DATABASE mtest SET ac_session_audit TO '{a:a}';
--
\connect - postgres
-- Устанавливаем маску аудита сессии по умолчанию
-- Ожидается успех
ALTER ROLE ALL SET ac_session_audit TO '{a:a}';
-- Просматриваем значение маски аудита по умолчанию
-- Ожидается {ace:ace}
SHOW ac_default_audit;
-- Устанавливаем маску аудита сессии для базы данных
-- Ожидается успех
ALTER ROLE ALL IN DATABASE mtest SET ac_session_audit TO '{r:r}';
-- Устанавливаем маску аудита сессии для пользователя
-- Ожидается успех
ALTER ROLE u_1_11 SET ac_session_audit TO '{w:w}';
-- Устанавливаем маску аудита сессии для связки пользователь-база данных
-- Ожидается успех
ALTER ROLE u_1_11 IN DATABASE mtest SET ac_session_audit TO '{d:d}';
-- Настройки пользователей хранятся в таблице pg_db_role_setting
-- Убедимся, что настройки были сохранены в таблице
SELECT id AS "Приоритет применения", dbname AS "Имя базы данных", username AS "Имя пользователя", options AS "Опции"  FROM settings_info() ORDER BY "Приоритет применения";
--
-- Заходим пользователем u_1_11
-- и проверяем маску аудита сессии
\connect - u_1_11
-- Ожидается {dce:dce}
SHOW ac_session_audit;
-- Ожидается {dce:dce}
SELECT session_audit;
-- Ожидается {dce:dce}
SELECT current_audit;
--
-- Заходим пользователем postgres
-- Удаляем маску аудита для пользователя u_1_11 и базы данных mtest
\connect - postgres
-- Ожидается успех
ALTER ROLE u_1_11 IN DATABASE mtest RESET ac_session_audit;
--
-- Заходим пользователем u_1_11
-- и проверяем маску аудита сессии
\connect - u_1_11
-- Ожидается {wce:wce}
SHOW ac_session_audit;
-- Ожидается {wce:wce}
SELECT session_audit;
-- Ожидается {wce:wce}
SELECT current_audit;
--
-- Заходим пользователем postgres
-- Удаляем маску аудита для пользователя u_1_11
\connect - postgres
-- Ожидается успех
ALTER ROLE u_1_11 RESET ac_session_audit;
--
-- Заходим пользователем u_1_11
-- и проверяем маску аудита сессии
\connect - u_1_11
-- Ожидается {rce:rce}
SHOW ac_session_audit;
-- Ожидается {rce:rce}
SELECT session_audit;
-- Ожидается {rce:rce}
SELECT current_audit;
--
-- Заходим пользователем postgres
-- Удаляем маску аудита для базы данных mtest
\connect - postgres
-- Ожидается успех
ALTER ROLE ALL IN DATABASE mtest RESET ac_session_audit;
--
-- Заходим пользователем u_1_11
-- и проверяем маску аудита сессии
\connect - u_1_11
-- Ожидается {ace:ace}
SHOW ac_session_audit;
-- Ожидается {ace:ace}
SELECT session_audit;
-- Ожидается {ace:ace}
SELECT current_audit;
-- Заходим пользователем postgres
-- Удаляем маску аудита по умолчанию
\connect - postgres
-- Ожидается успех
ALTER ROLE ALL RESET ac_session_audit;
--
-- Заходим пользователем u_1_11
-- и проверяем маску аудита сессии
\connect - u_1_11
-- Ожидается {ce:ce}
SHOW ac_session_audit;
-- Ожидается {ce:ce}
SELECT session_audit;
-- Ожидается {ce:ce}
SELECT current_audit;
--
-- Проверка различных ситуаций смены маски аудита для текущего/нетекущего пользователя.
-- Маска аудита будет изменена для текущего пользователя согласно приоритету настроек аудита:
-- т.е. текущий уровень аудита превосходит уровень аудита, устанавливаемый в запросе,
-- то смены маски аудита не будет.
--
-- Подключаемся как пользователь postgres
\connect - postgres
-- Убеждаемся, что маска сессии - {SsRawdCTEMce:ce}
SELECT session_audit;
-- Изменяем маску аудита по умолчанию
-- Ожидается успех
ALTER ROLE ALL SET ac_session_audit = '{a:a}';
-- Просматриваем маску аудита по умолчанию
-- Ожидается {SsRawdCTEMce:ce}
SHOW ac_default_audit;
-- Получаем маску аудита сессии
-- Ожидается {ace:ace}
SELECT session_audit;
-- Изменяем маску аудита для текущего пользователя
-- Ожидается успех
ALTER ROLE postgres SET ac_session_audit = '{r:r}';
-- Получаем маску аудита сессии
-- Ожидается {rce:rce}
SELECT session_audit;
-- Изменяем маску аудита для текущей базы данных и пользователя postgres
-- Ожидается успех
ALTER ROLE postgres IN DATABASE mtest SET ac_session_audit = '{w:w}';
-- Получаем маску аудита сессии
-- Ожидается {wce:wce}
SELECT session_audit;
-- Изменяем маску аудита для другой базы данных и текущего пользователя
-- Ожидается успех
ALTER ROLE postgres IN DATABASE postgres SET ac_session_audit = '{d:d}';
-- Получаем маску аудита сессии
-- Ожидается {wce:wce}, т.к. настройки были установлены для другой базы данных
SELECT session_audit;
-- Изменяем маску аудита для всех баз данных и всех пользователей
-- Ожидается успех
ALTER ROLE ALL SET ac_session_audit = '{D:D}';
-- Просматриваем маску аудита по умолчанию
-- Ожидается {Dce:Dce}
SHOW ac_default_audit;
-- Получаем маску аудита сессии
-- Ожидается {wce:wce}, т.к. примененные настройки выше по приоритету установленных командой
SELECT session_audit;
-- Изменяем маску аудита для пользователя u_1_11 и базы данных postgres
-- Ожидается успех
ALTER ROLE u_1_11 IN DATABASE postgres SET ac_session_audit = '{x:x}';
-- Получаем маску аудита сессии
-- Ожидается {wce:wce}, т.к. устанавливается для другого пользователя
SELECT session_audit;
-- Изменяем маску аудита для пользователя postgres и базы данных mtest
-- Ожидается успех
ALTER ROLE postgres IN DATABASE mtest SET ac_session_audit = '{V:V}';
-- Получаем маску аудита сессии
-- Ожидается {Vce:Vce}, т.к. устанавливаются для текущего пользователя (postgres),
-- подключенного к текущей базе данных (mtest)
SELECT session_audit;
--
-- Проверка различных ситуаций сброса маски аудита для текущего/нетекущего пользователя.
-- Маска аудита будет сброшена до состояния, предшествующая выполнению команды ALTER ROLE ... RESET
-- т.е. сброс маски аудита для связки текущая База данных - текущий пользователь приведёт к попытке установке маски аудита на текущего пользователя,
-- если нет записи для него - попытка установки маски для текущей базы данных, далее - к значению по умолчанию.
-- Аналогично предыдущему, сброс маски аудита не для текущих баз данных и пользователей не приведут к изменению маски аудита у текущего пользователя.
--
-- Просматриваем маску аудита сессии
-- Ожидается {Vce:Vce}
SELECT session_audit;
-- Просматриваем таблицу pg_db_role_setting
-- Ожидается 5 записей
SELECT id AS "Приоритет применения", dbname AS "Имя базы данных", username AS "Имя пользователя", options AS "Опции"  FROM settings_info() ORDER BY "Приоритет применения";
-- Сбрасываем маски аудита для текущего пользователя и текущей базы данных
-- Ожидается успех
ALTER ROLE postgres IN DATABASE mtest RESET ac_session_audit;
-- Просматриваем маску аудита
-- Ожидается {rce:rce} -- следующая настройка по приоритету (роль)
SELECT session_audit;
--
-- Просматриваем таблицу pg_db_role_setting
-- Ожидается 4 записи
SELECT id AS "Приоритет применения", dbname AS "Имя базы данных", username AS "Имя пользователя", options AS "Опции"  FROM settings_info() ORDER BY "Приоритет применения";
-- Сбрасываем маски аудита для пользователя postgres и всех баз данных
-- Ожидается успех
ALTER ROLE postgres RESET ac_session_audit;
-- Просматриваем маску аудита
-- Ожидается {Dce:Dce} -- следующая настройка по приоритету (все базы данных - все роли)
SELECT session_audit;
--
-- Просматриваем таблицу pg_db_role_setting
-- Ожидается 3 записи
SELECT id AS "Приоритет применения", dbname AS "Имя базы данных", username AS "Имя пользователя", options AS "Опции"  FROM settings_info() ORDER BY "Приоритет применения";
-- Сбрасываем маску для пользователя postgres в базе данных postgres
ALTER ROLE postgres IN DATABASE postgres RESET ac_session_audit;
-- Просматриваем маску аудита
-- Ожидается {Dce:Dce} -- изменение настройки не произойдет,
-- т.к. сброс производится для пользователя postgres и базы данных postgres,
-- а в текущий момент пользователь postgres подключен к базе mtest
SELECT session_audit;
--
-- Просматриваем таблицу pg_db_role_setting
-- Ожидается 2 записи
SELECT id AS "Приоритет применения", dbname AS "Имя базы данных", username AS "Имя пользователя", options AS "Опции"  FROM settings_info() ORDER BY "Приоритет применения";
-- Сбрасываем маску для пользователя u_1_11 в базе данных postgres
-- Ожидается успех
ALTER ROLE u_1_11 IN DATABASE postgres RESET ac_session_audit;
-- Просматриваем маску аудита
-- Ожидается {Dce:Dce} -- изменение настройки не произойдет,
-- т.к. сброс производится для пользователя u_1_11 и базы данных postgres,
-- а в текущий момент пользователь postgres подключен к базе mtest
SELECT session_audit;
--
-- Убеждаемся, что выполнен сброс параметров аудита
-- Ожидается 1 запись
SELECT id AS "Приоритет применения", dbname AS "Имя базы данных", username AS "Имя пользователя", options AS "Опции"  FROM settings_info() ORDER BY "Приоритет применения";
-- Сбрасываем маску всех пользователей во всех базах данных
-- Ожидается успех
ALTER ROLE ALL RESET ac_session_audit;
-- Просматриваем маску аудита
-- Ожидается {ce:ce} -- маска аудита по умолчанию
SELECT session_audit;
--
-- Просматриваем таблицу pg_db_role_setting
-- Ожидается 0 записей
SELECT id AS "Приоритет применения", dbname AS "Имя базы данных", username AS "Имя пользователя", options AS "Опции"  FROM settings_info() ORDER BY "Приоритет применения";
--
-- ================================================
-- III. Проверка настроек аудита в режиме external
-- ================================================
--
-- В режиме работы 'external' настройки аудита берутся из файла
-- (по умолчанию из pg_audit.conf), синтаксис которого следующий:
--
-- success events mask = value failure events mask = value user = имя_пользователя database = имя_базы_данных
-- success events mask = value failure events mask = value user = имя_пользователя
-- success events mask = value failure events mask = value
--
-- Приоритет установки маски такой же, как и в случае с режимом internal, а именно:
-- База данных - пользователь > Пользователь > База данных > Все базы данных - все пользователи.
-- В данном режиме работы изменить аудит сессии администратор не может. Установку глобальных настроек аудита
-- (в pg_db_role_setting) с помощью команды ALTER ROLE ... SET(RESET) также не повлиют на текущий аудит сессии.
--
-- Подключаемся как пользователь postgres
\connect - postgres
-- Меняем режим работы аудита
SELECT ac_change_param_text('ac_audit_mode', 'external');
-- Перезаходим как пользователь postgres
\connect - postgres
-- Просматриваем текущий режим работы аудита
-- Ожидается external
SHOW ac_audit_mode;
-- Просматриваем все настройки из pg_audit
-- Ожидается 6 строк
\! cat /var/lib/postgresql/9.6/setest/pg_audit.conf | grep -v '^#' | grep -v '^$'
--
--
-- Просматриваем аудит по умолчанию
-- Ожидается {ce:ce}
SHOW ac_default_audit;
-- Просматриваем маску аудита сессии
-- Ожидается {rawDMce:rawDMce}
SELECT session_audit;
--
-- Заходим как пользователь u_0_00
\connect - u_0_00
-- Просматриваем маску аудита сессии
-- Ожидается {rawDce:rawDce}
SELECT session_audit;
--
-- Заходим как пользователь u_1_11
\connect - u_1_11
-- Просматриваем маску аудита сессии
-- Ожидается {race:race}
SELECT session_audit;
--
-- Заходим как пользователь u_2_11
\connect - u_2_11
-- Просматриваем маску аудита
-- Ожидается {O:O} -- аудит не установлен,
-- и любые действия пользователя логироваться не будут
SELECT session_audit;
--
--
\connect - postgres
-- Пытаемся изменить маску аудита сесии
-- Ожидаестся предупреждение
SET ac_session_audit TO '{cead:cead}';
-- Убеждаемся, что маска сессии не изменилась
-- Ожидается {rawDMce:rawDMce}
SELECT session_audit;
-- Убеждаемся, что маска аудита по умолчанию не изменилась
-- Ожидается {ce:ce}
SHOW ac_default_audit;
--
-- Выполняем попытку назначить метку аудита для пользователя postgres
-- и базы данных mtest
-- Ожидается предупреждение
ALTER ROLE postgres IN DATABASE mtest SET ac_session_audit TO '{DM:DM}';
-- Просматриваем маску аудита сессии
-- Ожидается {rawDMce:rawDMce}
SELECT session_audit;
-- Просматриваем таблицу pg_db_role_setting
-- Ожидается 1 запись
SELECT id AS "Приоритет применения", dbname AS "Имя базы данных", username AS "Имя пользователя", options AS "Опции"  FROM settings_info() ORDER BY "Приоритет применения";
-- Удаляем настройки из pg_db_role_setting
-- Ожидается успех
ALTER ROLE postgres IN DATABASE mtest RESET ac_session_audit;
-- Просматриваем таблицу pg_db_role_setting
-- Ожидается 0 записей
SELECT id AS "Приоритет применения", dbname AS "Имя базы данных", username AS "Имя пользователя", options AS "Опции"  FROM settings_info() ORDER BY "Приоритет применения";
--
-- Убеждаемся в том, что удаление настроек не повлияло на настройки аудита
-- Просматриваем маску аудита сессии
-- Ожидается {rawDMce:rawDMce}
SELECT session_audit;
-- Просматриваем маску аудита по умолчанию
-- Ожидается {ce:ce}
SHOW ac_default_audit;
-- Устанавливаем режим работы аудита в исходное состояние
SELECT ac_change_param_text('ac_audit_mode', 'internal');
--
-- ========================================================
-- IV. Проверка настроек аудита в режиме internal,external
-- ========================================================
--
-- Модуль аудита в режиме работы 'internal,external' загружает маски
-- пользователям сначала из pg_db_role_setting, потом из файла pg_audit.conf,
-- если в файле содержатся настройки аудита по приоритету выше, чем в pg_db_role_setting
--
-- Устанавливаем маски аудита для пользователей
\connect - postgres
-- Устанавливаем маску аудита сессии по умолчанию
-- Ожидается успех
ALTER ROLE ALL SET ac_session_audit TO '{a:a}';
-- Устанавливаем маску аудита сессии для базы данных
-- Ожидается успех
ALTER ROLE ALL IN DATABASE mtest SET ac_session_audit TO '{r:r}';
-- Устанавливаем маску аудита сессии для пользователя
-- Ожидается успех
ALTER ROLE u_1_11 SET ac_session_audit TO '{w:w}';
-- Устанавливаем маску аудита сессии для связки пользователь-база данных
-- Ожидается успех
ALTER ROLE u_1_11 IN DATABASE mtest SET ac_session_audit TO '{d:d}';
-- Настройки пользователей хранятся в таблице pg_db_role_setting
-- Убедимся, что настройки были сохранены в таблице
SELECT id AS "Приоритет применения", dbname AS "Имя базы данных", username AS "Имя пользователя", options AS "Опции"  FROM settings_info() ORDER BY "Приоритет применения";
-- Просматриваем параметры аудита из файла pg_audit.conf
-- Ожидается 6 строк
\! cat /var/lib/postgresql/9.6/setest/pg_audit.conf | grep -v '^#' | grep -v '^$'
--
--
-- Изменяем режим работы аудита
-- Ожидается успех
SELECT ac_change_param_text('ac_audit_mode', 'internal,external');
--
-- Перезаходим как пользователь postgres
\connect - postgres
-- Просматриваем режим работы аудита
-- Ожидается internal,external
SHOW ac_audit_mode;
-- Проверяем текущую маску аудита
-- Ожидается {rce:rce}
-- т.к. в pg_db_role_setting применяется аудит для базы данных
-- к пользователю postgres и базе данных mtest,
-- а в pg_audit.conf для пользователя postgres применяется аудит
-- по умолчанию.
SELECT session_audit;
-- Проверяем маску аудита по умолчанию
-- Ожидается {ace:ace}
SHOW ac_default_audit;
--
-- Заходим как пользователь u_1_11
\connect - u_1_11
-- Проверяем текущую маску аудита
-- Ожидается {dce:dce}
SELECT session_audit;
-- Проверяем маску аудита по умолчанию
-- Ожидается {ace:ace}
SHOW ac_default_audit;
--
-- Заходим как пользователь u_0_00
\connect - u_0_00
-- Просматриваем текущую маску аудита
-- т.к. в pg_db_role_setting будет применена
-- настройка аудита по умолчанию для u_0_00,
-- а в audit.conf есть настройка для этого пользователя,
-- то и маска аудита сессии и аудит по умолчанию будет установлен
-- из файла
-- Ожидается {rawDce:rawDce}
SELECT session_audit;
-- Просматриваем маску аудита по умолчанию
-- Ожидается {ce:ce}
SHOW ac_default_audit;
--
-- Подключаемся как пользователь postgres
\connect - postgres
-- Просматриваем маску аудита сессии
-- Ожидается {rce:rce}
SELECT session_audit;
-- Обновляем настройки аудита для пользователя postgres
-- Ожидается успех
ALTER ROLE ALL IN DATABASE mtest SET ac_session_audit = '{ra:ra}';
-- Просматриваем маску аудита сессии
-- Ожидается {race:race}
SELECT session_audit;
-- Просматриваем маску аудита по умолчанию
-- Ожидается {ace:ace}
SHOW ac_default_audit;
-- Сбрасываем маску аудита для всех пользователей и базы данных mtest
-- Ожидается успех
ALTER ROLE ALL IN DATABASE mtest RESET ac_session_audit;
-- Просматриваем маску аудита сессии
-- Ожидается {ace:ace}
SELECT session_audit;
-- Переподключаемся как пользователь postgres
\connect - postgres
-- Просматриваем маску аудита сессии
-- Ожидается {cearwDM:cearwDM}
SELECT session_audit;
-- Просматриваем маску аудита по умолчанию
-- Ожидается {ce:ce}
SHOW ac_default_audit;
--
-- Переключаем режим работы аудита в external,internal
SELECT ac_change_param_text('ac_audit_mode', 'external,internal');
--
-- =======================================================
-- V. Проверка настроек аудита в режиме external,internal
-- =======================================================
--
-- Модуль аудита в режиме работы 'external,internal' загружает маски
-- пользователям сначала из файла pg_audit.conf, потом из pg_db_role_setting,
-- если в pg_db_role_setting содержатся настройки аудита по приоритету выше, чем в файле
--
\connect - postgres
-- Просматриваем настройки аудита из pg_db_role_setting
SELECT id AS "Приоритет применения", dbname AS "Имя базы данных", username AS "Имя пользователя", options AS "Опции"  FROM settings_info() ORDER BY "Приоритет применения";
-- Просматриваем параметры аудита из файла pg_audit.conf
-- Ожидается 6 строк
\! cat /var/lib/postgresql/9.6/setest/pg_audit.conf | grep -v '^#' | grep -v '^$'
--
--
-- Просматриваем режим работы аудита
-- Ожидается external,internal
SHOW ac_audit_mode;
-- Просматриваем маску аудита пользователя
-- Ожидается {rawDMce:rawDMce}
SELECT session_audit;
-- Просматриваем маску аудита по умолчанию
-- Ожидается {ce:ce}
SHOW ac_default_audit;
--
-- Устанавливаем аудит для пользователя u_1_11 и базы данных mtest
-- Ожидается успех
ALTER ROLE u_1_11 IN DATABASE mtest SET ac_session_audit TO '{raw:raw}';
-- Просматриваем настройки аудита из pg_db_role_setting
SELECT id AS "Приоритет применения", dbname AS "Имя базы данных", username AS "Имя пользователя", options AS "Опции"  FROM settings_info() ORDER BY "Приоритет применения";
--
-- Подключаемся как пользователь u_1_11
\connect - u_1_11
-- Просматриваем маску аудита сессии
-- Ожидается {race:race}
-- т.к. в pg_audit.conf содержится запись для пользователя u_1_11
-- и базы данных mtest
SELECT session_audit;
-- Просматриваем маску аудита по умолчанию
-- Ожидается {ce:ce}
SHOW ac_default_audit;
--
-- Подключаемся как пользователь postgres
\connect - postgres
-- Назначаем маску аудита для пользователя postgres и базы данных mtest
ALTER ROLE postgres IN DATABASE mtest SET ac_session_audit TO '{cet:cet}';
-- Просматриваем маску аудита
-- Ожидается {tce:tce}
SELECT session_audit;
-- Просматриваем маску аудита по умолчанию
-- Ожидается {ce:ce}
SHOW ac_default_audit;
--
-- Изменяем режим работы аудита на 'none'
SELECT ac_change_param_text('ac_audit_mode', 'none');
--
-- ===========================================
-- VI. Проверка настроек аудита в режиме none
-- ===========================================
--
-- Режим 'none' отключает регистрацию событий аудита в лог-файле.
-- При этом админстратору разрешено производить действия по назначению аудита
-- с помощью команд SET ..., ALTER ROLE SET ..., но изменения маски аудита проводиться не будет
--
-- Переподключаемся как пользователь postgres
\connect - postgres
-- Просматриваем режим работы аудита
-- Ожидается none
SHOW ac_audit_mode;
-- Просматриваем маску аудита сессии
-- Ожидается '{O:O}'
SELECT session_audit;
-- Просматриваем маску аудита по умолчанию
-- Ожидается {ce:ce}
SHOW ac_default_audit;
-- Назначаем полную маску аудита сессии с помощью команды SET
-- Ожидается предупреждение
SET ac_session_audit TO '{cea:cea}';
-- Просматриваем маску аудита сессии
SELECT session_audit;
-- Просматриваем значение параметра session_audit
-- Ожидается '{O:O}'
SHOW ac_session_audit;
-- Назначаем маску аудита для пользователя postgres и базы данных mtest
-- Ожидается успех
ALTER ROLE postgres IN DATABASE mtest SET ac_session_audit TO '{cea:cea}';
-- Просматриваем маску аудита сессии
-- Ожидается '{O:O}'
SELECT session_audit;
-- Просматриваем таблицу pg_db_role_setting
-- Ожидается 4 записи
SELECT id AS "Приоритет применения", dbname AS "Имя базы данных", username AS "Имя пользователя", options AS "Опции"  FROM settings_info() ORDER BY "Приоритет применения";
--
-- Возвращаем режим аудита в исходное состояние
SELECT ac_change_param_text('ac_audit_mode', 'internal');
--
-- ========================================
-- VII. Проверка встроенных функций аудита
-- ========================================
--
-- Были добавлены следующие функции:
-- session_audit() -- показать аудит сессии
-- current_audit() -- показать текущий аудит
-- auditmask_to_text(text) -- преобразовать маску аудита из шестнадцатеричного числа в текстовое
-- text_to_auditmask(text) -- преобразовать буквенное обозначение в шестнадцатеричное
--
-- Переподключаемся как пользователь postgres
\connect - postgres
-- Получаем текущий аудит
-- Ожидается {ace:ace}
SELECT current_audit;
-- Получаем маску аудита сессии
-- Ожидается {ace:ace}
SELECT session_audit;
--
-- Проверка функций конвертации
--
-- Конвертируем {ce:ce} в шестнадцатеричное представление
-- Ожидается {0xC0000000:0xC0000000}
SELECT text_to_auditmask('{ce:ce}');
-- Конвертируем {O:O} в шестнадцатеричное представление
-- Ожидается {0x0:0x0}
SELECT text_to_auditmask('{O:O}');
-- Конвертируем {*:*} в шестнадцатеричное представление
-- Ожидается {0xC01F3DFF:0xC01F3DFF}
SELECT text_to_auditmask('{*:*}');
-- Конвертируем {cea:*} в шестнадцатеричное представление
-- Ожидается {0xC0000020:0xC01F3DFF}
SELECT text_to_auditmask('{cea:*}');
-- Конвертируем {O:O} в шестнадцатеричное представление
--
-- Конвертируем 0xC01F3DFF:0xC01F3DFF в текстовое представление
-- Ожидается {SsRVrawdDxtXUCTEMmce:SsRVrawdDxtXUCTEMmce}
SELECT auditmask_to_text('{0xC01F3DFF:0xC01F3DFF}');
-- Конвертируем {0xC10F3FFF:0xC10F3FFF} в текстовое представление
-- Ожидается ошибка
SELECT auditmask_to_text('{0xC10F3FFF:0xC10F3FFF}');
-- Ковертируем {0xC0000000:0xC0000000} в текстовое представление
-- Ожидается {ce:ce}
SELECT auditmask_to_text('{0xC0000000:0xC0000000}');
-- Ковертируем {0xB0000000:0xB0000000} в текстовое представление
-- Ожидается {O:O}
SELECT auditmask_to_text('{0x00000000:0x00000000}');
--
\connect - postgres
-- Очищаем таблицу pg_db_role_setting
\connect - postgres
ALTER ROLE ALL RESET ac_session_audit;
ALTER ROLE ALL IN DATABASE mtest RESET ac_session_audit;
ALTER ROLE u_1_11 RESET ac_session_audit;
ALTER ROLE u_1_11 IN DATABASE mtest RESET ac_session_audit;
ALTER ROLE postgres IN DATABASE mtest RESET ac_session_audit;
-- Восстанавливаем параметры аудита
ALTER ROLE postgres SET ac_session_audit TO '{SsRawdCTEMce:ce}';
ALTER ROLE ALL SET ac_session_audit TO '{SsRDxCTEMce:SsRVrawdDxtXUCTEMce}';
--
\i 'support/done.sql'