-- RBT-TEST - Проверка pg_dump/pg_restore
-- Проверка резервирования/восстановления с помощью pg_dump,psql,pg_restore
--

\i 'support/init.sql'
\i 'support/init-mac.sql'
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
--
-- Подключаемся как пользователь с меткой {1,2}
\connect - u_1_10
--
-- Создаем тестовую таблицу
CREATE TABLE "Проверка BLOB" (loid oid, fd integer);
GRANT ALL ON "Проверка BLOB" TO PUBLIC;
--
-- Подключаемся как администратор кластера
\connect - postgres
--
-- Создаем таблицы для проверки вывода опций
CREATE TABLE "Проверка_RelOptions" (id integer) WITH (MACS=TRUE, fillfactor=50);
CREATE TABLE "Проверка_RelOptions2" (id integer) WITH (fillfactor=50);
--
-- Создаем индекс на таблицу Проверка с метками
CREATE INDEX bt_index_with_macs ON "Проверка" USING btree("идентификатор") WITH (MACS=TRUE);
-- Создаем индекс на таблицу Проверка без меток
CREATE INDEX bt_index_without_macs ON "Проверка" USING btree("идентификатор") WITH (MACS=FALSE);
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Отключаем ДПРД к таблице pg_largeobject
GRANT ALL ON pg_largeobject TO PUBLIC;
--
-- Вставляем запись в таблицу pg_largeobject с меткой {0,0} для объекта 200000
INSERT INTO pg_largeobject(loid, pageno, data) VALUES(200000, 0, 'test');
SELECT maclabel, * FROM pg_largeobject;
--
-- Подключаемся как пользователь с меткой {1,2}
\connect - u_1_10
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Создаем объект с идентификатором 200000 из таблицы pg_largeobject
-- ожидается ошибка доступа
INSERT INTO "Проверка BLOB" (loid) SELECT lo_create(200000);
--
-- Создаем объект с идентификаторм 200001
-- Ожидается успех
INSERT INTO "Проверка BLOB" (loid) SELECT lo_create(200001);
SELECT * FROM "Проверка BLOB";
--
-- Делаем доступным всем
GRANT ALL ON LARGE OBJECT 200001 TO PUBLIC;
--
-- Начинаем транзакцию
BEGIN;
--
-- Открываем объект на чтение и запись
--   INV_READ  = 0x40000
--   INV_WRITE = 0x20000
UPDATE "Проверка BLOB" SET fd = lo_open(loid, CAST(x'20000' | x'40000' AS integer));
--
-- Проверка записи, ожидается успех
SELECT lowrite(fd, '
Whose woods these are I think I know,
His house is in the village though.
He will not see me stopping here,
To watch his woods fill up with snow.

My little horse must think it queer,
To stop without a farmhouse near,
Between the woods and frozen lake,
The darkest evening of the year.

He gives his harness bells a shake,
To ask if there is some mistake.
The only other sound''s the sweep,
Of easy wind and downy flake.

The woods are lovely, dark and deep,
But I have promises to keep,
And miles to go before I sleep,
And miles to go before I sleep.

         -- Robert Frost
') FROM "Проверка BLOB";
--
-- Закрываем объект
-- Ожидается 0 - отсутствие ошибок, отрицательное значение - ошибка
SELECT lo_close(fd) FROM "Проверка BLOB";
--
-- Завершаем транзакцию
END;
--
-- Проверяем прямой доступ к данным объектов
-- Ожидается, что объект приобрел метку пользователя {1,2}
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Проверяем атрибуты объекта
-- Ожидается, что объект приобрел метку пользователя {1,2}
SELECT maclabel, oid FROM pg_largeobject_metadata;
--
--
-- Проверяем возможность работы с объектом пользователями с другими метками
-- Подключаемся как пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
-- Создаем тестовую таблицу
CREATE TABLE "Проверка BLOB0" (loid oid, fd integer);
INSERT INTO "Проверка BLOB0" values(200001, 0);
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Проверяем атрибуты объекта
-- Ожидается, что пользователь с меньшей меткой, не может увидеть объект 200001
SELECT maclabel, oid FROM pg_largeobject_metadata;
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Пытаемся открыть объект на чтение
-- Ожидается ошибка, объект для пользователя недоступен
BEGIN;
UPDATE "Проверка BLOB0" SET fd=lo_open(loid, CAST(x'40000' AS integer));
END;
--
-- Пытаемся открыть объект на запись
-- Ожидается ошибка, объект для пользователя опять же недоступен
BEGIN;
UPDATE "Проверка BLOB0" SET fd=lo_open(loid, CAST(x'20000' AS integer));
END;
--
-- Пытаемся удалить объект
-- Ожидается ошибка, объект для пользователя недоступен
SELECT lo_unlink(loid) from "Проверка BLOB0";
--
-- Подключаемся как пользователь u_2_11 с меткой {2,3}
\connect - u_2_11
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Проверяем атрибуты объекта
-- Ожидается, что объект виден для пользователя с большей меткой
SELECT maclabel, oid FROM pg_largeobject_metadata;
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Пытаемся добавить страницу к объекту
-- ожидается ошибка доступа
INSERT INTO pg_largeobject(loid, pageno, data) VALUES(200001, 1, convert_to(USER || ' data', 'UTF8'));
--
-- Создаем тестовую таблицу
CREATE TABLE "Проверка BLOB2" (loid oid, fd integer);
INSERT INTO "Проверка BLOB2" values(200001, 0);
--
-- Пытаемся открыть объект на чтение
-- Ожидается успех
BEGIN;
UPDATE "Проверка BLOB2" SET fd=lo_open(loid, CAST(x'40000' AS integer));
--
-- Устанавливаем позицию начала чтения
SELECT lo_lseek(fd, 422, 0) FROM "Проверка BLOB2";
--
-- Читаем 35 символов
SELECT loread(fd, 35) FROM "Проверка BLOB2";
--
-- Меняем позицию начала чтения
SELECT lo_lseek(fd, -19, 1) FROM "Проверка BLOB2";
--
-- Текущий размер
SELECT lo_lseek(fd, 0, 2) FROM "Проверка BLOB2";
SELECT lo_tell(fd) FROM "Проверка BLOB2";
--
-- Закрываем объект и завершаем транзакцию
SELECT lo_close(fd) FROM "Проверка BLOB2";
END;
--
-- Пытаемся открыть объект на запись
-- Ожидается ошибка доступа
BEGIN;
UPDATE "Проверка BLOB2" SET fd=lo_open(loid, CAST(x'20000' AS integer));
END;
--
-- Пытаемся удалить объект
-- Ожидается ошибка доступа
SELECT lo_unlink(loid) from "Проверка BLOB2";
--
--
-- Выполняем резервирование с помощью утилиты pg_dump в текстовом режиме
\! /usr/lib/postgresql/9.6/bin/pg_dump -U postgres mtest > /tmp_test/dump
--
-- Просматриваем результат резервирования
\! cat /tmp_test/dump
--
-- Пересоздаем базу данных
\connect template1 postgres
DROP DATABASE mtest;
CREATE DATABASE mtest;
--
-- Выполняем восстановление с помощью утилиты psql
\! /usr/lib/postgresql/9.6/bin/psql -U postgres -d mtest -f /tmp_test/dump
--
\connect mtest
--
-- Отключаем ДПРД к таблице pg_largeobject
GRANT ALL ON pg_largeobject TO PUBLIC;
--
-- Проверяем результат восстановления
\i 'support/select-test-table.sql'
--
-- Проверяем прямой доступ к данным объектов
-- Ожидается, что объект имеет метку {1,2}
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Проверяем атрибуты объекта
-- Ожидается, что объект имеет метку {1,2}
SELECT maclabel, oid FROM pg_largeobject_metadata;
--
--
-- Проверяем возможность работы с объектом пользователями с другими метками
-- Подключаемся как пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
INSERT INTO "Проверка BLOB0" values(200001, 0);
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Проверяем атрибуты объекта
-- Ожидается, что пользователь с меньшей меткой, не может увидеть объект 200001
SELECT maclabel, oid FROM pg_largeobject_metadata;
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Пытаемся открыть объект на чтение
-- Ожидается ошибка, объект для пользователя недоступен
BEGIN;
UPDATE "Проверка BLOB0" SET fd=lo_open(loid, CAST(x'40000' AS integer));
END;
--
-- Пытаемся открыть объект на запись
-- Ожидается ошибка, объект для пользователя опять же недоступен
BEGIN;
UPDATE "Проверка BLOB0" SET fd=lo_open(loid, CAST(x'20000' AS integer));
END;
--
-- Пытаемся удалить объект
-- Ожидается ошибка, объект для пользователя недоступен
SELECT lo_unlink(loid) from "Проверка BLOB0";
--
-- Подключаемся как пользователь u_2_11 с меткой {2,3}
\connect - u_2_11
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Проверяем атрибуты объекта
-- Ожидается, что объект виден для пользователя с большей меткой
SELECT maclabel, oid FROM pg_largeobject_metadata;
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Пытаемся добавить страницу к объекту
-- ожидается ошибка доступа
INSERT INTO pg_largeobject(loid, pageno, data) VALUES(200001, 1, convert_to(USER || ' data', 'UTF8'));
--
INSERT INTO "Проверка BLOB2" values(200001, 0);
--
-- Пытаемся открыть объект на чтение
-- Ожидается успех
BEGIN;
UPDATE "Проверка BLOB2" SET fd=lo_open(loid, CAST(x'40000' AS integer));
--
-- Устанавливаем позицию начала чтения
SELECT lo_lseek(fd, 422, 0) FROM "Проверка BLOB2";
--
-- Читаем 35 символов
SELECT loread(fd, 35) FROM "Проверка BLOB2";
--
-- Меняем позицию начала чтения
SELECT lo_lseek(fd, -19, 1) FROM "Проверка BLOB2";
--
-- Текущий размер
SELECT lo_lseek(fd, 0, 2) FROM "Проверка BLOB2";
SELECT lo_tell(fd) FROM "Проверка BLOB2";
--
-- Закрываем объект и завершаем транзакцию
SELECT lo_close(fd) FROM "Проверка BLOB2";
END;
--
-- Пытаемся открыть объект на запись
-- Ожидается ошибка доступа
BEGIN;
UPDATE "Проверка BLOB2" SET fd=lo_open(loid, CAST(x'20000' AS integer));
END;
--
-- Пытаемся удалить объект
-- Ожидается ошибка доступа
SELECT lo_unlink(loid) from "Проверка BLOB2";
--
-- Удаляем временные файлы и директории
\! /bin/rm -f /tmp_test/dump
\! /bin/rm -f /tmp_test/dump.tar
\! /bin/rm -fr /tmp_test/dump_dir
-- Создаем директорию для pg_dump в режиме вывода dir
\! mkdir /tmp_test/dump_dir
\! pdpl-file 3:0:3:ccnr,ccnri /tmp_test/dump_dir
\! chmod 777 /tmp_test/dump_dir
--
-- Выполняем резервирование с помощью утилиты pg_dump в бинарном режиме
\! /usr/lib/postgresql/9.6/bin/pg_dump -U postgres mtest -F c -f /tmp_test/dump
--
-- Выполняем резервирование с помощью утилиты pg_dump в режиме вывода tar
\! /usr/lib/postgresql/9.6/bin/pg_dump -U postgres -d mtest -F t -f /tmp_test/dump.tar
--
-- Выполняем резервирование с помощью утилиты pg_dump в режиме вывода dir
\! /usr/lib/postgresql/9.6/bin/pg_dump -U postgres -d mtest -F d -f /tmp_test/dump_dir
--
-- Просматриваем результат резервирования
-- Ожидается метка на файлах {2,3}
\! lsm -Mn /tmp_test/dump
\! lsm -Mn /tmp_test/dump.tar
\! lsm -Mn /tmp_test/dump_dir | tail -n +2 | awk '{split($6, a, "."); print $1, $2, $3, $4, $5, a[2], a[3]}'
--
-- Пересоздаем базу данных
\connect template1 postgres
DROP DATABASE mtest;
CREATE DATABASE mtest;
--
-- Выполняем восстановление с помощью утилиты pg_restore
\! /usr/lib/postgresql/9.6/bin/pg_restore -U postgres -d mtest /tmp_test/dump
--
-- Отключаем ДПРД к таблице pg_largeobject
\connect mtest
--
GRANT ALL ON pg_largeobject TO PUBLIC;
--
-- Проверяем результат восстановления
\i 'support/select-test-table.sql'
--
-- Проверяем атрибуты объекта
SELECT maclabel, oid FROM pg_largeobject_metadata;
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Удаляем временный файл
\! /bin/rm -f /tmp_test/dump
--
-- Проверка восстановления из дампа tar
-- Пересоздаем базу данных
\connect template1 postgres
DROP DATABASE mtest;
CREATE DATABASE mtest;
--
-- Выполняем восстановление с помощью утилиты pg_restore
\! /usr/lib/postgresql/9.6/bin/pg_restore -U postgres -d mtest /tmp_test/dump.tar
--
-- Отключаем ДПРД к таблице pg_largeobject
\connect mtest
--
GRANT ALL ON pg_largeobject TO PUBLIC;
--
-- Проверяем результат восстановления
\i 'support/select-test-table.sql'
--
-- Проверяем атрибуты объекта
SELECT maclabel, oid FROM pg_largeobject_metadata;
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Удаляем временный файл
\! /bin/rm -f /tmp_test/dump.tar
--
-- Проверка восстановления из дампа dir
-- Пересоздаем базу данных
\connect template1 postgres
DROP DATABASE mtest;
CREATE DATABASE mtest;
--
-- Выполняем восстановление с помощью утилиты pg_restore
\! /usr/lib/postgresql/9.6/bin/pg_restore -U postgres -d mtest /tmp_test/dump_dir
--
-- Отключаем ДПРД к таблице pg_largeobject
\connect mtest
--
GRANT ALL ON pg_largeobject TO PUBLIC;
--
-- Проверяем результат восстановления
\i 'support/select-test-table.sql'
--
-- Проверяем атрибуты объекта
SELECT maclabel, oid FROM pg_largeobject_metadata;
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Удаляем директорию дампа
\! /bin/rm -fr /tmp_test/dump_dir
--
-- Пересоздаем базу данных
\connect template1 postgres
DROP DATABASE mtest;
CREATE DATABASE mtest;
--
\connect mtest
\i 'support/init-mac.sql'
--
-- Создаем объекты БД
\connect - postgres
\i 'support/create-objects.sql'
--
-- Изменяем мандатную метку объектов
\connect - postgres
-- Меняем владельца представления на postgres
ALTER MATERIALIZED VIEW matview_u_1_01_2 OWNER TO postgres;
--
MAC LABEL ON CLUSTER IS '{3,1F}';
MAC LABEL ON DATABASE mtest IS '{2,1F}';
MAC LABEL ON ROLE group1 IS '{1,0}';
MAC LABEL ON SCHEMA public IS '{2,1F}';
MAC LABEL ON TABLE table_u_1_01 IS '{2,1}';
MAC LABEL ON TABLE table_u_1_01_С IS '{2,2}';
MAC LABEL ON VIEW view_u_1_01 IS '{2,3}';
MAC LABEL ON MATERIALIZED VIEW matview_u_1_01_2 IS '{0,0}';
MAC LABEL ON SEQUENCE seq_u_1_01 IS '{2,4}';
MAC LABEL ON TYPE type_u_1_01_c IS '{2,5}';
MAC LABEL ON TYPE type_u_1_01_e IS '{2,6}';
MAC LABEL ON TYPE type_u_1_01_r IS '{2,7}';
MAC LABEL ON DOMAIN type_u_1_01_d IS '{2,1}';
MAC LABEL ON FUNCTION func_u_1_01() IS '{2,9}';
MAC LABEL ON TRIGGER trg_u_1_01 ON table_u_1_01 IS '{2,A}';
MAC LABEL ON AGGREGATE agg_u_1_01(float8) IS '{2,B}';
MAC LABEL ON CAST (maclabel AS varchar) IS '{2,C}';
MAC LABEL ON COLLATION rus_u_1_01 IS '{2,D}';
MAC LABEL ON CONVERSION conv_u_1_01 IS '{2,E}';
MAC LABEL ON SERVER srv_u_1_01 IS '{2,F}';
MAC LABEL ON FOREIGN TABLE ftbl_u_1_01 IS '{2,10}';
MAC LABEL ON LARGE OBJECT 200000 IS '{2,11}';
MAC LABEL ON OPERATOR FAMILY opf1 USING btree IS '{2,12}';
MAC LABEL ON OPERATOR CLASS opc1 USING btree  IS '{2,13}';
MAC LABEL ON OPERATOR ##(path,path) IS '{2,14}';
MAC LABEL ON TEXT SEARCH PARSER tsp1 IS '{2,15}';
MAC LABEL ON TEXT SEARCH TEMPLATE tst1 IS '{2,16}';
MAC LABEL ON TEXT SEARCH DICTIONARY tsd_u_1_01 IS '{2,17}';
MAC LABEL ON TEXT SEARCH CONFIGURATION tsc_u_1_01 IS '{2,18}';
MAC LABEL ON TABLESPACE dbspace IS '{2,1C}';
MAC LABEL ON EXTENSION adminpack IS '{2,1D}';
MAC LABEL ON FOREIGN DATA WRAPPER dummy IS '{2,1E}';
MAC LABEL ON LANGUAGE plpgsql IS '{2,1F}';
MAC LABEL ON EVENT TRIGGER ev_trigger IS '{2,1F}';
--
-- Проверяем объекты БД
\i 'support/check-objects.sql'
--
--
-- Выполняем резервирование с помощью утилиты pg_dumpall в текстовом режиме
\! /usr/lib/postgresql/9.6/bin/pg_dumpall -U postgres > /tmp_test/db.out
--
-- Удаляем занятые объекты
\i 'support/delete-objects.sql'
--
-- Удаляем глобальные объекты
\i 'support/done.sql'
--
-- Выполняем восстановление с помощью утилиты psql
\i '/tmp_test/db.out'
--
-- Удаляем временный файл
\! /bin/rm -f /tmp_test/db.out
--
-- Проверяем объекты БД
\connect mtest
\i 'support/check-objects.sql'
--
\i 'support/delete-objects.sql'
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'