-- RBT-TEST - Проверка механизмов очистки памяти
-- Проверка механизмов очистки памяти:
--   1. Очистка памяти на странице (ac_wipe_pages)
--   2. Очистка памяти при усечении файла (ac_wipe_files)
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
\i 'support/init-binary-tools.sql'
\i 'support/init-config-tools.sql'
--
-- Подключаемся как администратор postgres
-- И иницилизируем расширение pageinspect
\connect - postgres
--
-- Создаем тестувую таблицу
CREATE TABLE "Проверка"("Идентификатор" INTEGER, "Данные" TEXT);
GRANT ALL ON TABLE "Проверка" TO PUBLIC;
--
-- ========================================
-- I. Проверка механизма очистки страниц
-- ========================================
--
-- 1. Проверка без включенного режима очистки страниц
--
-- Убеждаемся, что режим очистки страниц не активен
-- Ожидается off
SHOW ac_wipe_pages;
--
-- Выполняем вставку данных, которые можно будет увидеть
-- утилитами командной строки
INSERT INTO "Проверка" VALUES(1, 'test 1');
INSERT INTO "Проверка" VALUES(2, 'test 2');
INSERT INTO "Проверка" VALUES(3, 'test 3');
--
-- Просматриваем содержимое таблицы test
-- Ожидается 3 записи
SELECT * FROM "Проверка" ORDER BY "Идентификатор";
--
-- Сбрасываем данные на диск
CHECKPOINT;
--
-- Просматриваем содержимое таблицы на жестком диске
-- Ожидается, что в дампе будет содержаться 3 записи 'test'
SELECT get_db_file_dump('Проверка', 'test');
--
-- Убеждаемся, что записи найдены в файле отношения на диске
-- Ожидается, что все записи найдены -- t
SELECT check_pattern_in_db_file('Проверка', 'test 1');
SELECT check_pattern_in_db_file('Проверка', 'test 2');
SELECT check_pattern_in_db_file('Проверка', 'test 3');
--
-- Обновляем таблицу, изменив столбец Данные
UPDATE "Проверка" SET "Данные" = 'data ' || "Идентификатор";
--
-- Сбрасываем данные на диск
CHECKPOINT;
--
-- Просматриваем записи таблицы Проверка
-- Ожидается 3 записи
SELECT * FROM "Проверка";
--
-- Просматриваем содержимое таблицы на жестком диске
-- Ожидается, что в дампе будет содержаться 3 записи 'test' (старые),
-- а также новые записи 'data' -- 3 записи
SELECT get_db_file_dump('Проверка', 'test|data');
--
-- Выполяем VACUUM
VACUUM "Проверка";
--
-- Просматриваем содержимое таблицы на жестком диске
-- Ожидается, что в дампе будет содержаться 3 записи 'test' (старые),
-- а также новые записи 'data' -- 3 записи
SELECT get_db_file_dump('Проверка', 'test|data');
--
-- Убеждаемся, что старые записи 'test ..' по-прежнему присутствуют в файле
-- отношения на диске
-- Ожидается, что все записи найдены -- t
SELECT check_pattern_in_db_file('Проверка', 'test 1');
SELECT check_pattern_in_db_file('Проверка', 'test 2');
SELECT check_pattern_in_db_file('Проверка', 'test 3');
--
-- Выполняем VACUUM FULL
VACUUM FULL;
--
-- Просматриваем содержимое таблицы на жестком диске
-- Ожидается, что в дампе только 3 новые записи 'data ...'
SELECT get_db_file_dump('Проверка', 'test|data');
--
-- Убеждаемся, что старне записи 'test ..' теперь отсутствуют
-- в файле отношения на диске
-- Ожидается, что все записи не найдены -- f
SELECT check_pattern_in_db_file('Проверка', 'test 1');
SELECT check_pattern_in_db_file('Проверка', 'test 2');
SELECT check_pattern_in_db_file('Проверка', 'test 3');
--
-- Очищаем таблицу "Проверка"
DELETE FROM "Проверка";
VACUUM FULL;
--
-- 2. Проверка с включенным режимом очистки страниц
--
-- Включаем параметр ac_wiping_pages
--
\i 'support/enable-wiping-pages.sql'
--
-- Выполняем вставку данных, которые можно будет увидеть
-- утилитами командной строки
INSERT INTO "Проверка" VALUES(1, 'test 1');
INSERT INTO "Проверка" VALUES(2, 'test 2');
INSERT INTO "Проверка" VALUES(3, 'test 3');
--
-- Просматриваем содержимое таблицы test
-- Ожидается 3 записи
SELECT * FROM "Проверка" ORDER BY "Идентификатор";
--
-- Сбрасываем данные на диск
CHECKPOINT;
--
-- Просматриваем содержимое таблицы на жестком диске
-- Ожидается, что в дампе будет содержаться 3 записи 'test'
SELECT get_db_file_dump('Проверка', 'test');
--
-- Убеждаемся, что записи найдены в файле отношения на диске
-- Ожидается, что все записи найдены -- t
SELECT check_pattern_in_db_file('Проверка', 'test 1');
SELECT check_pattern_in_db_file('Проверка', 'test 2');
SELECT check_pattern_in_db_file('Проверка', 'test 3');
--
-- Обновляем таблицу, изменив столбец Данные
UPDATE "Проверка" SET "Данные" = 'data ' || "Идентификатор";
--
-- Просматриваем записи таблицы Проверка
-- Ожидается 3 записи
SELECT * FROM "Проверка";
--
-- Просматриваем содержимое таблицы на жестком диске
-- Ожидается, что в дампе будет содержаться 3 записи 'test' (старые),
-- а также новые записи 'data' -- 3 записи
SELECT get_db_file_dump('Проверка', 'test|data');
--
-- Выполяем VACUUM
VACUUM "Проверка";
--
-- Просматриваем содержимое таблицы на жестком диске
-- Ожидается, что в дампе будет только записи 'data ...' (новые)
SELECT get_db_file_dump('Проверка', 'test|data');
--
-- Убеждаемся, что старые записи 'test ..' отсутствуют в файле
-- отношения на диске
-- Ожидается, что все записи отсутствуют -- f
SELECT check_pattern_in_db_file('Проверка', 'test 1');
SELECT check_pattern_in_db_file('Проверка', 'test 2');
SELECT check_pattern_in_db_file('Проверка', 'test 3');
--
-- Выключаем параметр ac_memory_wiping
\i 'support/disable-wiping-pages.sql'
--
-- Очищаем содержимое таблицы Проверка
DROP TABLE "Проверка";
VACUUM FULL;
--
-- ========================================
-- II. Проверка механизма очистки файлов
-- ========================================
--
-- Подготовка к тестам
--
-- Создаем образ файла ОС для определения удаленных блоков
\! 'support/create-image'
--
-- Создаем табличное пространство для теста
CREATE TABLESPACE wipespc LOCATION '/var/lib/postgresql/9.6/wipespc';
--
-- Создаем таблицу Проверка в табличном пространстве wipespc
CREATE TABLE "Проверка"("Идентификатор" INTEGER, "Данные" TEXT) TABLESPACE wipespc;
--
-- 1. Проверка без включенного режима очистки файлов
--
-- Просматриваем значение параметра ac_wipe_files
-- Ожидается off
SHOW ac_wipe_files;
--
-- Наполняем таблицу Проверка
INSERT INTO "Проверка" SELECT i, 'test ' || i
FROM generate_series(1, 10000) g(i);
--
-- Принудительно сбрасываем данные на диск
CHECKPOINT;
--
-- Просматриваем содержимое файла-образа
-- Выводим первые 10 строк, содержащие данные со строкой 'test'
SELECT get_image_dump('(?P<target>test\s[0-9]{1,4})', 10);
--
-- Просматриваем содержимое таблицы Проверка (выводим первые 10 записей)
SELECT * FROM "Проверка" ORDER BY "Идентификатор" LIMIT 10;
--
-- Ищем вставленную строки 'test ..' в созданном файле-образе
-- Ожидается t -- искомые строки найдены
SELECT check_pattern_in_image('test 1');
SELECT check_pattern_in_image('test 2');
SELECT check_pattern_in_image('test 3');
--
-- Удаляем последние 5000 записей
DELETE FROM "Проверка" WHERE "Идентификатор" >= 5000;
--
-- Выполняем полную очистку
VACUUM FULL;
--
-- Удостоверяемся, что в таблице отсутствует записи
-- с идентификатором >= 5000
-- Ожидается 0 записей
SELECT * FROM "Проверка" WHERE "Идентификатор" >= 5000 ORDER BY "Идентификатор" LIMIT 10;
--
-- Ищем 10 удаленных записей ('test 500*') в файле-образе
-- Ожидается, что они будут найдены
SELECT get_image_dump('(?P<target>test\s500[0-9])', 10);
--
-- Ищем удаленные записи
-- Ожидается t -- запись найдена (т.е. данные физически расположены на диске)
SELECT check_pattern_in_image('test 5000');
SELECT check_pattern_in_image('test 5001');
SELECT check_pattern_in_image('test 5002');
--
-- Выполняем полную очистку таблицы
TRUNCATE "Проверка";
--
-- Просматриваем содержимое таблицы
-- Ожидается 0 записей
SELECT * FROM "Проверка" ORDER BY "Идентификатор" LIMIT 10;
--
-- Ищем 10 удаленных записей ('test 499*') в файле-образе
-- Ожидается, что они будут найдены
SELECT get_image_dump('(?P<target>test\s499[0-9])', 10);
--
-- Ищем удаленные записи
-- Ожидается t -- запись найдена (т.е. данные физически расположены на диске)
SELECT check_pattern_in_image('test 4997');
SELECT check_pattern_in_image('test 4998');
SELECT check_pattern_in_image('test 4999');
--
-- Выполняем полную очистку
VACUUM FULL;
--
-- Очистка ранее созданных объектов
--
-- Удаляем таблицу Проверка
DROP TABLE "Проверка";
--
-- Удаляем табличное пространство
DROP TABLESPACE wipespc;
--
-- Удаляем образ файла
\! 'support/remove-image'
--
-- Подготовка к тесту
--
-- Создаем образ файла ОС для определения удаленных блоков
\! 'support/create-image'
--
-- Создаем табличное пространство для теста
CREATE TABLESPACE wipespc LOCATION '/var/lib/postgresql/9.6/wipespc';
--
-- Создаем таблицу Проверка в табличном пространстве wipespc
CREATE TABLE "Проверка"("Идентификатор" INTEGER, "Данные" TEXT) TABLESPACE wipespc;
--
-- 2. Проверка с включенным режимом очистки файлов
--
\i 'support/enable-wiping-files.sql'
--
-- Наполняем таблицу Проверка
INSERT INTO "Проверка" SELECT i, 'test ' || i
FROM generate_series(1, 10000) g(i);
--
-- Принудительно сбрасываем данные на диск
CHECKPOINT;
--
-- Просматриваем содержимое таблицы Проверка (выводим первые 10 записей)
SELECT * FROM "Проверка" ORDER BY "Идентификатор" LIMIT 10;
--
-- Просматриваем содержимое файла-образа
-- Выводим первые 10 строк, содержащие данные со строкой 'test'
SELECT get_image_dump('(?P<target>test\s[0-9]{1,4})', 10);
--
-- Ищем вставленную строки 'test ..' в созданном файле-образе
-- Ожидается t -- искомые строки найдены
SELECT check_pattern_in_image('test 1');
SELECT check_pattern_in_image('test 2');
SELECT check_pattern_in_image('test 3');
--
-- Удаляем последние 5000 записей
DELETE FROM "Проверка" WHERE "Идентификатор" >= 5000;
--
-- Выполняем полную очистку
VACUUM FULL;
--
-- Удостоверяемся, что в таблице отсутствует записи
-- с идентификатором >= 5000
-- Ожидается 0 записей
SELECT * FROM "Проверка" WHERE "Идентификатор" >= 5000 ORDER BY "Идентификатор" LIMIT 10;
--
-- Ищем 3 удаленные записи в файле-образе
-- Ожидается, что они найдены не будут
SELECT get_image_dump('(?P<target>test\s500[0-9])', 10);
--
-- Ищем удаленные записи
-- Ожидается f -- запись не найдена (т.е. данные были затерты на диске)
SELECT check_pattern_in_image('test 5000');
SELECT check_pattern_in_image('test 5001');
SELECT check_pattern_in_image('test 5002');
--
-- Выполняем полную очистку таблицы
TRUNCATE "Проверка";
--
-- Просматриваем содержимое таблицы
-- Ожидается 0 записей
SELECT * FROM "Проверка" ORDER BY "Идентификатор" LIMIT 10;
--
-- Ищем 3 удаленные записи в файле-образе
-- Ожидается, что они найдены не будут
SELECT get_image_dump('(?P<target>test\s499[0-9])', 10);
--
-- Ищем удаленные записи
-- Ожидается f -- запись не найдена
SELECT check_pattern_in_image('test 4997');
SELECT check_pattern_in_image('test 4998');
SELECT check_pattern_in_image('test 4999');
--
-- Выполняем полную очистку
VACUUM FULL;
--
-- Выключаем параметр очистки файлов
\i 'support/disable-wiping-files.sql'
--
-- Очистка ранее созданных объектов
--
-- Удаляем таблицу Проверка
DROP TABLE "Проверка";
--
-- Удаляем табличное пространство
DROP TABLESPACE wipespc;
--
-- Удаляем образ файла
\! 'support/remove-image'
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'