-- RBT-TEST - Дополнительный контроль ролей
-- Проверка дополнительных атрибутов роли
--
\i 'support/init.sql'
--
-- Подключаемся как суперпользователь postgres
\connect - postgres

-- Переименуем пользоватея, для этого подготовим пользователя в ОС
\! useradd patient
\! pdpl-user -l 0:3 patient

-- Переименование пользователя
ALTER ROLE u_0_00 RENAME TO patient;

-- Проверка корректности сессии, ожидается успех
\connect - patient
SELECT CURRENT_USER;

-- =============================================================
-- 1. Проверка механизма временного пароля
-- =============================================================
-- Добавим роли срок действия пароля
\connect - postgres

ALTER ROLE patient WITH PASSWORD '12345678';

DO
$$
    BEGIN
        EXECUTE format('ALTER ROLE patient VALID UNTIL %L', current_timestamp + interval '120 secs');
    END
$$ LANGUAGE 'plpgsql';

-- Ожидается успех, т.к. срок действия пароля не истек
\! expect support/expect-patient.exp

\! sleep 150

-- Ожидается ошибка, т.к. срок действия пароля истек
\! expect support/expect-patient.exp

-- =============================================================
-- 2. Проверка механизма ограничения количества сессий
-- =============================================================
\connect - postgres

-- Подготовка пользователя
ALTER ROLE patient RENAME TO u_0_00;
ALTER ROLE u_0_00 CONNECTION LIMIT 1;

-- Ожидается успех при создании подключения
\! psql -h localhost -p 6000 -d mtest -U u_0_00 -c 'SELECT pg_sleep(50);' > /dev/null 2>&1 &
-- Ожидается ошибка, т.к. одно соединение для данного пользователя уже есть (в фоне)
\! psql -h localhost -p 6000 -d mtest -U u_0_00 -c 'SELECT CURRENT_USER;'

\! sleep 70

-- =============================================================
-- 3. Проверка механизма блокирования роли
-- =============================================================
ALTER ROLE u_0_00 CONNECTION LIMIT 5;

-- Заранее создаем подключение
\! psql -h localhost -p 6000 -d mtest -U u_0_00 -c 'SELECT pg_sleep(150);' >> /tmp_test/file.out 2>&1 &
-- Блокируем вход роли
ALTER ROLE u_0_00 NOLOGIN;
-- Ожидается ошибка для этого подключения и ждущего, созданного ранее.
\! psql -h localhost -p 6000 -d mtest -U u_0_00 -c 'SELECT CURRENT_USER;' >> /tmp_test/file.out 2>&1

\! sleep 5

-- Результат просматривается в файле для синхронизации вывода
\! cat /tmp_test/file.out
--
\connect template1 postgres
\! userdel patient
\i 'support/done.sql'
