-- RBT-TEST - Управление ролями
-- Проверка ролевого разграничения доступа
--
\i 'support/init.sql'
\i 'support/init-config-tools.sql'
\i 'support/disable-trusted-owner.sql'
--
-- Подключаемся как суперпользователь postgres
\connect - postgres
-- Создаем тестовую роль
CREATE ROLE admins;
--
-- ======================================================
-- I. Проверяем работу параметра ac_enable_trusted_owner
-- ======================================================
-- Параметр ac_enable_trusted_owner = false не позволяет передавать
-- права неадминистраторам права на объекты другим пользователям
--
-- Просматриваем параметр ac_enable_trusted_owner
-- Ожидается off
SHOW ac_enable_trusted_owner;
--
-- Создаем тестовую таблицу
CREATE TABLE "Проверка"("Идентификатор" INTEGER, "Данные" TEXT);
-- Изменяем владельца таблицы на u_0_00
ALTER TABLE "Проверка" OWNER TO u_0_00;
--
-- Подключаемся как пользователь u_0_00
\connect - u_0_00
-- Пробуем передать права на таблицу Проверка
-- Ожидается ошибка
GRANT SELECT ON TABLE "Проверка" TO u2_0_00;
-- Удостоверяемся, что пользователь u2_0_00 не обладает правами на таблицу
\dp "Проверка";
--
-- Подключаемся как суперпользователь postgres
\connect - postgres
-- Изымаем права с таблицы Проверка у пользователя u2_0_00
REVOKE ALL ON "Проверка" FROM u2_0_00;
-- Удостоверяемся, что пользователь u2_0_00 не обладает правами на таблицу
\dp "Проверка";
--
-- Пробуем передать права на таблицу Проверка
-- Ожидается успех
GRANT SELECT ON TABLE "Проверка" TO u2_0_00;
-- Удостоверяемся, что права обновились права
\dp "Проверка";
--
\connect - postgres
-- Устанавливаем параметр ac_enable_trusted_owner в true
SELECT ac_change_param('ac_enable_trusted_owner', true);
-- Изымаем права с таблицы Проверка у пользователя u2_0_00
REVOKE ALL ON "Проверка" FROM u2_0_00;
-- Удостоверяемся, что пользователь u2_0_00 не обладает правами на таблицу
\dp "Проверка";
--
-- Подключаемся как пользователь u_0_00
\connect - u_0_00
-- Просматриваем значение параметра ac_enable_trusted_owner
-- Ожидается on
SHOW ac_enable_trusted_owner;
-- Пробуем передать права на таблицу Проверка
-- Ожидаеся успех
GRANT SELECT ON TABLE "Проверка" TO u2_0_00;
-- Убеждаемся, что права были установлены на таблицу
\dp "Проверка";
--
-- =======================================================
-- II. Проверяем работу параметра ac_enable_grant_options
-- =======================================================
-- Параметр ac_enable_grant_options = false не позволяет передавать
-- права на объект пользователю другим пользователям, не являясь при этом
-- влальцем объекта
--
-- Подключаемся как пользователь postgres
\connect - postgres
-- Просматриваем значение параметра ac_enable_grant_options
-- Ожидается off
SHOW ac_enable_grant_options;
-- Изымаем права на таблицу Проверка
REVOKE ALL ON "Проверка" FROM u_0_00;
REVOKE ALL ON "Проверка" FROM u2_0_00;
-- Устанавливаем владельца таблицы как postgres
ALTER TABLE "Проверка" OWNER TO postgres;
-- Удостоверяемся, что роли u_0_00 и u2_0_00 не обладают правами на таблицу Проверка
\dp "Проверка"
--
-- Даем права пользователю u_0_00 на таблицу Проверка
-- Ожидается успех
GRANT SELECT ON "Проверка" TO u_0_00;
-- Даем права пользователю u2_0_00 с возможностью передавать их другим пользователям
-- на таблицу Проверка
-- Ожидается предупреждение об игнорировании GRANT OPTION
-- и успешное отработка команды
GRANT SELECT ON "Проверка" TO u2_0_00 WITH GRANT OPTION;
-- Просматриваем установленные права на таблицу
-- Ожидается установленные права на таблицу для пользователей,
-- но без GRANT OPTION для пользователя u2_0_00
\dp "Проверка";
--
-- Подключаемся как u_0_00
\connect - u_0_00
-- Пробуем передать права пользователю u_0_01
-- Ожидается предупреждение о необходимости быть владельцем отношения,
-- с последующим отказом доступа
GRANT SELECT ON "Проверка" TO u_0_01;
--
-- Подключаемся как u2_0_00
\connect - u2_0_00
-- Пробуем передать права пользователю u_0_01
-- Ожидается предупреждение о необходимости быть владельцем отношения,
-- с последующим отказом доступа
GRANT SELECT ON "Проверка" TO u_0_01;
--
\connect - postgres
-- Удостоверяемся, что пользователь u_0_01 не имеет права на чтение таблицы
\dp "Проверка";
--
-- Изменяем параметр ac_enable_grant_options = true
\connect - postgres
SELECT ac_change_param('ac_enable_grant_options', true);
-- Изымаем права у пользователей на таблицу
REVOKE ALL ON "Проверка" FROM u_0_00;
REVOKE ALL ON "Проверка" FROM u2_0_00;
-- Удостоверяемся, что роли u_0_00 и u2_0_00 не обладают правами на таблицу Проверка
\dp "Проверка"
-- Переподключаемся
\connect - postgres
-- Просматриваем значение параметра ac_enable_grant_options
-- Ожидается on
SHOW ac_enable_grant_options;
-- Даем права пользователю u_0_00 на таблицу Проверка
-- Ожидается успех
GRANT SELECT ON "Проверка" TO u_0_00;
-- Даем права пользователю u2_0_00 с возможностью передавать их другим пользователям
-- на таблицу Проверка
-- Ожидается успех
GRANT SELECT ON "Проверка" TO u2_0_00 WITH GRANT OPTION;
-- Просматриваем установленные права на таблицу
-- Ожидается установленные права на таблицу для пользователей,
-- у пользователя u2_0_00 имеется привилегия GRANT OPTIONS
\dp "Проверка";
--
-- Подключаемся как u_0_00
\connect - u_0_00
-- Пробуем передать права пользователю u_0_01
-- Ожидается предупреждение о том, что права назначены не были
GRANT SELECT ON "Проверка" TO u_0_01;
--
-- Подключаемся как u2_0_00
\connect - u2_0_00
-- Пробуем передать права пользователю u_0_01
-- Ожидается успех, т.к. у пользователя u2_0_00 есть привилегия GRANT OPTIONS
GRANT SELECT ON "Проверка" TO u_0_01;
-- Удостоверимся, что пользователь u_0_01 имеет права на таблицу Проверка
\dp "Проверка";
--
\connect - u2_0_00
-- Изымаем права у пользователя u_0_01 с таблицы Проверка
REVOKE ALL ON "Проверка" FROM u_0_01;
-- Удостоверяемся, что пользователь u_0_01 не имеет прав на таблицу Проверка
\dp "Проверка";
--
-- Подключаемся как суперпользователь postgres
\connect - postgres
-- Устанавливаем ac_enable_grant_options в false
SELECT ac_change_param('ac_enable_grant_options', false);
--
-- Подключаемся как пользователь u2_0_00
\connect - u2_0_00
-- Просматриваем значение параметра ac_enable_grant_options
-- Ожидается off
SHOW ac_enable_grant_options;
-- Пробуем передать права пользователю u_0_01
-- Ожидается предупреждение о том, пользователь должен быть владельцем отношения
-- с последующим отказом доступа
GRANT SELECT ON "Проверка" TO u_0_01;
-- Убеждаемся в том, что пользователь u_0_01 не имеет прав на таблицу Проверка
\dp "Проверка";
-- Пробуем удалить GRANT OPTION у пользователя u2_0_00
-- Ожидается предупреждение о том, пользователь должен быть владельцем отношения
-- с последующим отказом доступа
REVOKE GRANT OPTION FOR SELECT ON "Проверка" FROM u2_0_00;
-- Убеждаемся в том, что пользователь u2_0_00 не обладает GRANT OPTION на SELECT
\dp "Проверка";
--
-- Подключаемся как суперпользователь postgres
\connect - postgres
-- Пробуем удалить GRANT OPTION у пользователя u2_0_00
-- Ожидается успех
REVOKE GRANT OPTION FOR SELECT ON "Проверка" FROM u2_0_00;
-- Убеждаемся в том, что пользователь u2_0_00 не обладает GRANT OPTION на SELECT
\dp "Проверка";
--
-- ========================================================
-- III. Проверяем работу параметра ac_enable_admin_options
-- ========================================================
-- Параметр ac_enable_admin_options = false не позволяет передавать
-- права на членство в роли другим ролям.
--
-- Подключаемся как суперпользователь postgres
\connect - postgres
-- Просматриваем значение параметра ac_enable_grant_options
-- Ожидается off
SHOW ac_enable_admin_options;
-- Добавляем пользователя u_0_00 в группу admins
-- без возможности управления членством пользователей
-- Ожидается успех
GRANT admins TO u_0_00;
-- Добавляем пользователя u_0_00 в группу admins
-- с возможностью управления членством пользователей
-- Ожидается предупреждение об игнорировании этой опции
GRANT admins TO u2_0_00 WITH ADMIN OPTION;
-- Просматриваем членов ролей admins
-- Ожидается 2 записи, false в столбце "Привилегия администрирования"
-- у пользователей u_0_00, u2_0_00
SELECT pg_get_userbyid(roleid) AS "Группа", pg_get_userbyid(member) AS "Члены роли", pg_get_userbyid(grantor) AS "Назначивший", admin_option AS "Привилегия администрирования" FROM pg_auth_members;
-- Проверяем возможность добавления пользователей в группу admins
\connect - u_0_00
-- Ожидается предупреждение о необходимости включить параметр
-- ac_enable_admin_options и с последующей ошибкой
-- об отсутствии привилегии admin
GRANT admins TO u_0_01;
--
\connect - u2_0_00
-- Ожидается предупреждение о необходимости включить параметр
-- ac_enable_admin_options с последующей ошибкой
-- об отсутствии привилегии admin
GRANT admins TO u_0_01;
--
-- Подключаемся как суперпользователь postgres
\connect - postgres
-- Включаем параметр ac_enable_admin_options
SELECT ac_change_param('ac_enable_admin_options', true);
-- Переподключаемся
\connect - postgres
-- Просматриваем значение параметра ac_enable_admin_options
-- Ожидается on
SHOW ac_enable_admin_options;
--
-- Добавляем пользователя u2_0_00 в группу admins
-- с возможностью управления членством пользователей
-- Ожидается успех
GRANT admins TO u2_0_00 WITH ADMIN OPTION;
-- Просматриваем членов ролей admins
-- Ожидается 2 записи, в столбце "Привилегия администрирования"
-- у пользователя u_0_00 значение false, у u2_0_00 - true
SELECT pg_get_userbyid(roleid) AS "Группа", pg_get_userbyid(member) AS "Члены роли", pg_get_userbyid(grantor) AS "Назначивший", admin_option AS "Привилегия администрирования" FROM pg_auth_members;
--
-- Подключаемся как пользователь u2_0_00
\connect - u2_0_00
-- Пробуем добавить пользователя u_0_01 в группу admins
-- Ожидается успех
GRANT admins TO u_0_01;
--
-- Просматриваем членов ролей admins
-- Ожидается 3 записи, в столбце "Привилегия администрирования"
-- у пользователей admin, u_0_00 значение false, у u2_0_00 - true
SELECT pg_get_userbyid(roleid) AS "Группа", pg_get_userbyid(member) AS "Члены роли", pg_get_userbyid(grantor) AS "Назначивший", admin_option AS "Привилегия администрирования" FROM pg_auth_members;
--
-- Исключаем пользователя u_0_01 из группы admins
-- Ожидается успех
REVOKE admins FROM u_0_01;
-- Просматриваем членов ролей admins
-- Ожидается 2 записи, в столбце "Привилегия администрирования"
-- у пользователя u_0_00 значение false, у u2_0_00 - true
SELECT pg_get_userbyid(roleid) AS "Группа", pg_get_userbyid(member) AS "Члены роли", pg_get_userbyid(grantor) AS "Назначивший", admin_option AS "Привилегия администрирования" FROM pg_auth_members;
--
-- Подключаемся как суперпользователь postgres
\connect - postgres
-- Выключаем значение параметра ac_enable_admin_options
SELECT ac_change_param('ac_enable_admin_options', false);
--
-- Подключаемся как пользователь u2_0_00
\connect - u2_0_00
-- Просматриваем значение параметра ac_enable_admin_options
-- Ожидается off
SHOW ac_enable_admin_options;
-- Пробуем добавить пользователя u_0_01 в группу admins
-- Ожидается предупреждение о необходимости включить параметр
-- ac_enable_admin_options с последующей ошибкой
-- об отсутствии привилегии admin
GRANT admins TO u_0_01;
-- Просматриваем членов ролей admins
-- Ожидается 2 записи, в столбце "Привилегия администрирования"
-- у пользователя u_0_00 значение false, у u2_0_00 - true
SELECT pg_get_userbyid(roleid) AS "Группа", pg_get_userbyid(member) AS "Члены роли", pg_get_userbyid(grantor) AS "Назначивший", admin_option AS "Привилегия администрирования" FROM pg_auth_members;
-- Подключаемся как postgres
\connect - postgres
-- Удаляем ADMIN OPTION у пользователя u2_0_00
REVOKE ADMIN OPTION FOR admins FROM u2_0_00;
-- Просматриваем членов ролей admins
-- Ожидается 2 записи, false в столбце "Привилегия администрирования"
-- у пользователей u_0_00, u2_0_00
SELECT pg_get_userbyid(roleid) AS "Группа", pg_get_userbyid(member) AS "Члены роли", pg_get_userbyid(grantor) AS "Назначивший", admin_option AS "Привилегия администрирования" FROM pg_auth_members;
--
--
\connect - postgres
-- Удаляем созданные объекты
DROP ROLE admins;
DROP TABLE "Проверка";
-- Устанавливаем ac_enable_trusted_owner в исходное состояние
\i 'support/enable-trusted-owner.sql'
-- Устанавливаем ac_enable_grant_options в исходное состояние
SELECT ac_change_param('ac_enable_grant_options', false);
-- Устанавливаем ac_enable_admin_options в исходное состояние
SELECT ac_change_param('ac_enable_admin_options', false);
--
\i 'support/done.sql'
