-- RBT-TEST - МПРД при использовании правил (RULE)
-- Проверка сохранения работы МПРД при использовании правил (RULE)
--
\i 'support/init.sql'
\i 'support/init-mac.sql'
\i 'support/create-test-table.sql'
--
-- Создаем еще таблицу c защищенными строками и меткой {1,2}
CREATE TABLE "Проверка2" ("идентификатор" INTEGER PRIMARY KEY, "данные" TEXT) WITH (MACS=TRUE);
GRANT ALL ON "Проверка2" TO PUBLIC;
MAC LABEL ON TABLE "Проверка2" IS '{1,2}';
--
-- Создаем правило для операции INSERT
CREATE RULE "ПроверкаINS" AS ON INSERT TO "Проверка"
        WHERE NEW."идентификатор" > 28 DO ALSO
    INSERT INTO "Проверка2" VALUES (NEW."идентификатор", NEW."данные");
--
-- Смена владельца таблицы "Проверка" на u_1_10
ALTER TABLE "Проверка" OWNER TO u_1_10;
--
-- Пробуем заполнить таблицу
-- Ожидается что возможны будут только операции, разрешенные МПРД для таблицы Проверка2
-- Для ряда пользователей таблица Проверка2 не видна
\i 'support/init-test-table.sql'
--
-- Проверяем таблицу Проверка
-- Ожидается 4 записи, так как остальные не смогли вставиться по причине применения МПРД к таблице Проверка2
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем таблицу Проверка2
-- Ожидается 2 записи, отобранных условиями правила, метки записей сохранили свое значение
SELECT maclabel,* FROM "Проверка2" ORDER BY "идентификатор";
--
-- Создаем еще таблицу c незащищенными строками и меткой {1,2}
\connect - postgres
CREATE TABLE "Проверка3" ("идентификатор" INTEGER PRIMARY KEY, "данные" TEXT);
GRANT ALL ON "Проверка3" TO PUBLIC;
MAC LABEL ON TABLE "Проверка3" IS '{1,2}';
--
-- Изменяем правило для операции INSERT
CREATE OR REPLACE RULE "ПроверкаINS" AS ON INSERT TO "Проверка"
        WHERE NEW."идентификатор" > 28 DO ALSO
    INSERT INTO "Проверка3" VALUES (NEW."идентификатор", NEW."данные");
--
-- Очищаем таблицу
DELETE FROM "Проверка";
--
-- Смена владельца таблицы "Проверка" на u_1_10
ALTER TABLE "Проверка" OWNER TO u_1_10;
--
-- Пробуем заполнить таблицу
-- Ожидается что возможны будут только операции, разрешенные МПРД для таблицы Проверка3
\i 'support/init-test-table.sql'
--
-- Проверяем таблицу Проверка
-- Ожидается 4 записи, так как остальные не смогли вставиться по причине применения МПРД к таблице Проверка 3
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем таблицу Проверка3
-- Ожидается 2 записи, отобранных условиями правила, но уже без меток
SELECT * FROM "Проверка3" ORDER BY "идентификатор";
--
-- Создаем еще таблицу c защищенными строками, максимальной меткой и отключенным MAC CCR
\connect - postgres
CREATE TABLE "Проверка4" ("идентификатор" INTEGER PRIMARY KEY, "данные" TEXT) WITH (MACS=TRUE);
GRANT ALL ON "Проверка4" TO PUBLIC;
MAC LABEL ON TABLE "Проверка4" IS '{2,3}';
MAC CCR ON TABLE "Проверка4" IS OFF;
--
-- Изменяем правило для операции INSERT
CREATE OR REPLACE RULE "ПроверкаINS" AS ON INSERT TO "Проверка"
        WHERE NEW."идентификатор" > 28 DO ALSO
    INSERT INTO "Проверка4" VALUES (NEW."идентификатор", NEW."данные");
--
-- Очищаем таблицу
\connect - postgres
DELETE FROM "Проверка";
--
-- Смена владельца таблицы "Проверка" на u_1_10
ALTER TABLE "Проверка" OWNER TO u_1_10;
--
-- Пробуем заполнить таблицу
-- Ожидается что возможны будут все операции
\i 'support/init-test-table.sql'
--
-- Проверяем таблицу Проверка
-- Ожидается 10 записей, так как нет ограничений на вставку в таблицу Проверка4
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем таблицу Проверка4
-- Ожидается 7 записей, отобранных условиями правила
SELECT * FROM "Проверка4" ORDER BY "идентификатор";
--
-- Удаляем правило
\connect - postgres
DROP RULE "ПроверкаINS" ON "Проверка";
--
-- Создаем правило на обновление
CREATE RULE "ПроверкаUPD" AS ON UPDATE TO "Проверка"
        WHERE NEW."идентификатор" > 28 DO ALSO
    UPDATE "Проверка2" SET "данные" = "данные" || ' ! ' || NEW."данные"
    WHERE "идентификатор" = NEW."идентификатор"+1;
--
-- Проверяем модификацию данных
-- Пользователи меняют данные
-- Ожидается, что только пользователь с меткой {1,2} и пользователи с атрибутами игнорирования МПРД смогут осуществить обновление
\connect - postgres
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_2_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Подключается суперюзер postgres
\connect - postgres
--
-- Проверяем таблицу Проверка
-- Ожидается, что только пользователь с меткой {1,2} и пользователи с атрибутами игнорирования МПРД осуществили обновление
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем таблицу Проверка2
-- Видно что при обновлении, данные попали в другие строки
SELECT maclabel,* FROM "Проверка2" ORDER BY "идентификатор";
--
-- Очищаем таблицу Проверка4
DELETE FROM "Проверка4";
--
-- Очищаем тестовую таблицу
DELETE FROM "Проверка";
--
-- Удаляем правило
DROP RULE "ПроверкаUPD" ON "Проверка";
--
-- Создаем правило на удаление
CREATE RULE "ПроверкаDEL" AS ON DELETE TO "Проверка" DO ALSO
    INSERT INTO "Проверка4" VALUES (OLD."идентификатор", OLD."данные");
--
-- Подключается суперюзер postgres
\connect - postgres
\i 'support/init-test-table.sql'
--
-- Проверяем содержимое таблиц
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
SELECT * FROM "Проверка4" ORDER BY "идентификатор";
--
-- Проверка возможности удаления пользователями с разным уровнем
-- Попытка удаления пользователем u_0_00 с меткой {0,0}
-- Ожидается успех
\connect - u_0_00
DELETE FROM "Проверка";
--
\connect - postgres
--
-- Проверяем таблицу Проверка
-- Ожидается 8 записей, без удаленных с меткой {0,0}
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем таблицу Проверка4
-- Ожидается 2 записи, вставленных при удалении
SELECT * FROM "Проверка4" ORDER BY "идентификатор";
--
-- Попытка удаления пользователем u_0_01 с меткой {0,1}
-- Ожидается успех
\connect - u_0_01
DELETE FROM "Проверка";
--
\connect - postgres
--
-- Проверяем таблицу Проверка
-- Ожидается 7 записей, без удаленных с меткой {0,1}
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем таблицу Проверка4
-- Ожидается 3 записи
SELECT * FROM "Проверка4" ORDER BY "идентификатор";
--
-- Попытка удаления пользователем u_1_01 с меткой {1,1}
-- Ожидается успех
\connect - u_1_01
DELETE FROM "Проверка";
--
\connect - postgres
--
-- Проверяем таблицу Проверка
-- Ожидается 6 записей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем таблицу Проверка4
-- Ожидается 5 записей!
-- Поскольку правило применяется до операции, в таблицу вставляются доступные для чтения записи, при этом удаляется только одна
SELECT * FROM "Проверка4" ORDER BY "идентификатор";
--
-- Попытка удаления пользователем u_1_10 с меткой {1,2}
-- Ожидается ошибка, обусловленная указанным явлением
\connect - u_1_10
DELETE FROM "Проверка";
--
-- Рассмотрим вариант решения указанной проблемы
--
-- Удаляем правило
\connect - postgres
DROP RULE "ПроверкаDEL" ON "Проверка";
--
-- Очищаем тестовую таблицу
DELETE FROM "Проверка";
--
-- Очищаем таблицу Проверка4
DELETE FROM "Проверка4";
--
-- Меняем правило на удаление, теперь с проверкой метки сессии
CREATE RULE "ПроверкаDEL" AS ON DELETE TO "Проверка"
    WHERE OLD.maclabel = SESSION_MACLABEL DO ALSO
    INSERT INTO "Проверка4" VALUES (OLD."идентификатор", OLD."данные");
--
\i 'support/init-test-table.sql'
--
-- Проверяем содержимое таблиц
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
SELECT * FROM "Проверка4" ORDER BY "идентификатор";
--
-- Проверка возможности удаления пользователями с разным уровнем
-- Попытка удаления пользователем u_0_00 с меткой {0,0}
-- Ожидается успех
\connect - u_0_00
DELETE FROM "Проверка";
--
\connect - postgres
--
-- Проверяем таблицу Проверка
-- Ожидается 8 записей, без удаленных с меткой {0,0}
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем таблицу Проверка4
-- Ожидается 2 записи, вставленных при удалении
SELECT * FROM "Проверка4" ORDER BY "идентификатор";
--
-- Попытка удаления пользователем u_0_01 с меткой {0,1}
-- Ожидается успех
\connect - u_0_01
DELETE FROM "Проверка";
--
\connect - postgres
--
-- Проверяем таблицу Проверка
-- Ожидается 7 записей, без удаленных с меткой {0,1}
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем таблицу Проверка4
-- Ожидается 3 записи
SELECT * FROM "Проверка4" ORDER BY "идентификатор";
--
-- Попытка удаления пользователем u_1_01 с меткой {1,1}
-- Ожидается успех
\connect - u_1_01
DELETE FROM "Проверка";
--
\connect - postgres
--
-- Проверяем таблицу Проверка
-- Ожидается 6 записей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем таблицу Проверка4
-- Ожидается 4 записи
SELECT * FROM "Проверка4" ORDER BY "идентификатор";
--
-- Попытка удаления пользователем u_0_10 с меткой {0,1}
-- Ожидается успех
\connect - u_0_10
DELETE FROM "Проверка";
--
\connect - postgres
--
-- Проверяем таблицу Проверка
-- Ожидается 5 записей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем таблицу Проверка4
-- Ожидается 5 записей
SELECT * FROM "Проверка4" ORDER BY "идентификатор";
--
-- Попытка удаления пользователем u_1_10 с меткой {1,2}
-- Ожидается успех
\connect - u_1_10
DELETE FROM "Проверка";
--
\connect - postgres
--
-- Проверяем таблицу Проверка
-- Ожидается 4 записи
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем таблицу Проверка4
-- Ожидается 6 записей
SELECT * FROM "Проверка4" ORDER BY "идентификатор";
--
-- Попытка удаления пользователем u_1_11 с меткой {1,3}
-- Ожидается успех
\connect - u_1_11
DELETE FROM "Проверка";
--
\connect - postgres
--
-- Проверяем таблицу Проверка
-- Ожидается 2 записи
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем таблицу Проверка4
-- Ожидается 8 записей
SELECT * FROM "Проверка4" ORDER BY "идентификатор";
--
-- Попытка удаления пользователем u_2_11 с меткой {2,3}
-- Ожидается успех
\connect - u_2_11
DELETE FROM "Проверка";
--
\connect - postgres
--
-- Проверяем таблицу Проверка
-- Ожидается 1 запись
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем таблицу Проверка4
-- Ожидается 9 записей
SELECT * FROM "Проверка4" ORDER BY "идентификатор";
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'