-- Проверка объектов для тестов
--
-- от имени суперпользователя postgres
\connect - postgres
--
-- Проверяем все объекты, относящиеся к базе данных mtest
SELECT  0 AS "Порядок", maclabel AS "MAC", 'База данных' AS "Тип", datname AS "Наименование"
FROM    pg_database WHERE datname = 'mtest'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Параметр' AS "Тип",
      array_to_string(setconfig, ',') AS "Наименование"
FROM    pg_db_role_setting WHERE setdatabase = (SELECT oid FROM pg_database WHERE datname = 'mtest')
UNION
SELECT  3 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_shdepend WHERE objid = (SELECT oid FROM pg_database WHERE datname = 'mtest')
UNION
SELECT  4 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_shdescription WHERE objoid = (SELECT oid FROM pg_database WHERE datname = 'mtest')
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к табличному пространству dbspace
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Табличное пространство' AS "Тип", spcname AS "Наименование"
FROM    pg_tablespace WHERE spcname = 'dbspace'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_shdepend WHERE objid = (SELECT oid FROM pg_tablespace WHERE spcname = 'dbspace')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_shdescription WHERE objoid = (SELECT oid FROM pg_tablespace WHERE spcname = 'dbspace')
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к языку plpgsql
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Язык' AS "Тип", lanname AS "Наименование"
FROM    pg_language WHERE lanname = 'plpgsql'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_shdepend WHERE objid = (SELECT oid FROM pg_language WHERE lanname = 'plpgsql')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_shdescription WHERE objoid = (SELECT oid FROM pg_language WHERE lanname = 'plpgsql')
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к роли group1
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Роль' AS "Тип", rolname AS "Наименование"
FROM    pg_authid WHERE rolname = 'group1'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Входит в роль' AS "Тип",
    (SELECT rolname FROM pg_authid WHERE oid = roleid) AS "Наименование"
FROM    pg_auth_members WHERE member = (SELECT oid FROM pg_authid WHERE rolname = 'group1')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Параметр' AS "Тип",
      array_to_string(setconfig, ',') AS "Наименование"
FROM    pg_db_role_setting WHERE setrole = (SELECT oid FROM pg_authid WHERE rolname = 'group1')
UNION
SELECT  3 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_shdepend WHERE objid = (SELECT oid FROM pg_authid WHERE rolname = 'group1')
UNION
SELECT  4 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_shdescription WHERE objoid =  (SELECT oid FROM pg_authid WHERE rolname = 'group1')
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к роли u_1_01
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Роль' AS "Тип", rolname AS "Наименование"
FROM    pg_authid WHERE rolname = 'u_1_01'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Входит в роль' AS "Тип",
    (SELECT rolname FROM pg_authid WHERE oid = roleid) AS "Наименование"
FROM    pg_auth_members WHERE member = (SELECT oid FROM pg_authid WHERE rolname = 'u_1_01')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Параметр' AS "Тип",
      array_to_string(setconfig, ',') AS "Наименование"
FROM    pg_db_role_setting WHERE setrole = (SELECT oid FROM pg_authid WHERE rolname = 'u_1_01')
UNION
SELECT  3 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_shdepend WHERE objid = (SELECT oid FROM pg_authid WHERE rolname = 'u_1_01')
UNION
SELECT  4 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_shdescription WHERE objoid =  (SELECT oid FROM pg_authid WHERE rolname = 'u_1_01')
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к схеме public
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Схема' AS "Тип", nspname AS "Наименование"
FROM    pg_namespace WHERE nspname = 'public'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = (SELECT oid FROM pg_namespace WHERE nspname = 'public')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = (SELECT oid FROM pg_namespace WHERE nspname = 'public')
UNION
SELECT  3 AS "Порядок", maclabel AS "MAC", 'Default ACL' AS "Тип",
      array_to_string(defaclacl, ',') AS "Наименование"
FROM    pg_default_acl WHERE defaclnamespace = (SELECT oid FROM pg_namespace WHERE nspname = 'public')
ORDER BY 1, 4;
--
--
-- Проверяем все объекты, относящиеся к схеме "Схема_1_01"
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Схема' AS "Тип", nspname AS "Наименование"
FROM    pg_namespace WHERE nspname = 'Схема_1_01'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = (SELECT oid FROM pg_namespace WHERE nspname = 'Схема_1_01')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = (SELECT oid FROM pg_namespace WHERE nspname = 'Схема_1_01')
UNION
SELECT  3 AS "Порядок", maclabel AS "MAC", 'Default ACL' AS "Тип",
      array_to_string(defaclacl, ',') AS "Наименование"
FROM    pg_default_acl WHERE defaclnamespace = (SELECT oid FROM pg_namespace WHERE nspname = 'Схема_1_01')
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к расширению adminpack
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Расширение' AS "Тип", extname AS "Наименование"
FROM    pg_extension WHERE extname = 'adminpack'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = (SELECT oid FROM pg_extension WHERE extname = 'adminpack')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = (SELECT oid FROM pg_extension WHERE extname = 'adminpack')
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к обертке внешних данных dummy
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Обертка внешних данных' AS "Тип", fdwname AS "Наименование"
FROM    pg_foreign_data_wrapper WHERE fdwname = 'dummy'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = (SELECT oid FROM pg_foreign_data_wrapper WHERE fdwname = 'dummy')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = (SELECT oid FROM pg_foreign_data_wrapper WHERE fdwname = 'dummy')
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к таблице table_u_1_01
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Таблица' AS "Тип", 0 AS "Столбец", relname AS "Наименование"
FROM    pg_class WHERE relname = 'table_u_1_01'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Столбец таблицы' AS "Тип", attnum AS "Столбец", attname AS "Наименование"
FROM    pg_attribute WHERE attrelid = 'table_u_1_01'::regclass::oid
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Default' AS "Тип", adnum AS "Столбец", '' AS "Наименование"
FROM    pg_attrdef WHERE adrelid = 'table_u_1_01'::regclass::oid
UNION
SELECT  3 AS "Порядок", maclabel AS "MAC", 'Индекс' AS "Тип", 0 AS "Столбец",
(SELECT relname from pg_class where oid=indexrelid) AS "Наименование"
FROM    pg_index WHERE indrelid = 'table_u_1_01'::regclass::oid
UNION
SELECT  4 AS "Порядок", maclabel AS "MAC", 'Таблица индекса' AS "Тип", 0 AS "Столбец", relname AS "Наименование"
FROM    pg_class WHERE oid in (SELECT indexrelid FROM pg_index WHERE indrelid = 'table_u_1_01'::regclass::oid)
UNION
SELECT  5 AS "Порядок", maclabel AS "MAC", 'Столбец индекса' AS "Тип", attnum AS "Столбец", attname AS "Наименование"
FROM    pg_attribute WHERE attrelid in (SELECT indexrelid FROM pg_index WHERE indrelid = 'table_u_1_01'::regclass::oid)
UNION
SELECT  6 AS "Порядок", maclabel AS "MAC", 'TOAST' AS "Тип", 0 AS "Столбец", 'pg_toast_?' AS "Наименование"
FROM    pg_class WHERE oid in (SELECT reltoastrelid FROM pg_class WHERE relname = 'table_u_1_01')
UNION
SELECT  7 AS "Порядок", maclabel AS "MAC", 'Столбец TOAST таблицы' AS "Тип", attnum AS "Столбец", attname AS "Наименование"
FROM    pg_attribute WHERE attrelid in (SELECT reltoastrelid FROM pg_class WHERE relname = 'table_u_1_01')
UNION
SELECT  8 AS "Порядок", maclabel AS "MAC", 'Тип таблицы' AS "Тип", 0 AS "Столбец", typname AS "Наименование"
FROM    pg_type WHERE oid = (select reltype from pg_class where relname = 'table_u_1_01')
UNION
SELECT  9 AS "Порядок", maclabel AS "MAC", 'Тип таблицы' AS "Тип", 0 AS "Столбец", typname AS "Наименование"
FROM    pg_type WHERE oid = (select reloftype from pg_class where relname = 'table_u_1_01')
UNION
SELECT  10 AS "Порядок", maclabel AS "MAC", 'Ограничение' AS "Тип", 0 AS "Столбец", conname AS "Наименование"
FROM    pg_constraint WHERE conrelid = 'table_u_1_01'::regclass::oid
UNION
SELECT  11 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", refobjsubid AS "Столбец", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = 'table_u_1_01'::regclass::oid
UNION
SELECT  12 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", objsubid AS "Столбец", description AS "Наименование"
FROM    pg_description WHERE objoid = 'table_u_1_01'::regclass::oid
UNION
SELECT  13 AS "Порядок", maclabel AS "MAC", 'Статистика' AS "Тип", staattnum AS "Столбец", 'table_u_1_01' AS "Наименование"
FROM    pg_statistic WHERE starelid = 'table_u_1_01'::regclass::oid
UNION
SELECT  14 AS "Порядок", maclabel AS "MAC", 'Статистика' AS "Тип", staattnum AS "Столбец", 'table_u_1_01_PK' AS "Наименование"
FROM    pg_statistic WHERE starelid = 'table_u_1_01_PK'::regclass::oid
UNION
SELECT  15 AS "Порядок", maclabel AS "MAC", 'Правило' AS "Тип", 0 AS "Столбец", 'table_u_1_01_RU' AS "Наименование"
FROM    pg_rewrite WHERE ev_class = 'table_u_1_01'::regclass::oid
UNION
SELECT  16 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", 0 AS "Столбец", description AS "Наименование"
FROM    pg_description WHERE objoid = (SELECT oid FROM pg_rewrite WHERE rulename = 'table_u_1_01_ru')
UNION
SELECT  17 AS "Порядок", maclabel AS "MAC", 'Триггер' AS "Тип", 0 AS "Столбец", 'trg_u_1_01' AS "Наименование"
FROM    pg_trigger WHERE tgrelid = 'table_u_1_01'::regclass::oid
ORDER BY 1, 4, 5;
--
-- Проверяем все объекты, относящиеся к таблице table_u_1_01_С
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Таблица' AS "Тип", 0 AS "Столбец", relname AS "Наименование"
FROM    pg_class WHERE relname = 'table_u_1_01_С'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Столбец таблицы' AS "Тип", attnum AS "Столбец", attname AS "Наименование"
FROM    pg_attribute WHERE attrelid = 'table_u_1_01_С'::regclass::oid
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Default' AS "Тип", adnum AS "Столбец", '' AS "Наименование"
FROM    pg_attrdef WHERE adrelid = 'table_u_1_01_С'::regclass::oid
UNION
SELECT  3 AS "Порядок", maclabel AS "MAC", 'Индекс' AS "Тип", 0 AS "Столбец",
(SELECT relname from pg_class where oid=indexrelid) AS "Наименование"
FROM    pg_index WHERE indrelid = 'table_u_1_01_С'::regclass::oid
UNION
SELECT  4 AS "Порядок", maclabel AS "MAC", 'Таблица индекса' AS "Тип", 0 AS "Столбец", relname AS "Наименование"
FROM    pg_class WHERE oid in (SELECT indexrelid FROM pg_index WHERE indrelid = 'table_u_1_01_С'::regclass::oid)
UNION
SELECT  5 AS "Порядок", maclabel AS "MAC", 'Столбец индекса' AS "Тип", attnum AS "Столбец", attname AS "Наименование"
FROM    pg_attribute WHERE attrelid in (SELECT indexrelid FROM pg_index WHERE indrelid = 'table_u_1_01_С'::regclass::oid)
UNION
SELECT  6 AS "Порядок", maclabel AS "MAC", 'Тип таблицы' AS "Тип", 0 AS "Столбец", typname AS "Наименование"
FROM    pg_type WHERE oid = (select reltype from pg_class where relname = 'table_u_1_01_С')
UNION
SELECT  7 AS "Порядок", maclabel AS "MAC", 'Тип таблицы' AS "Тип", 0 AS "Столбец", typname AS "Наименование"
FROM    pg_type WHERE oid = (select reloftype from pg_class where relname = 'table_u_1_01_С')
UNION
SELECT  8 AS "Порядок", maclabel AS "MAC", 'Ограничение' AS "Тип", 0 AS "Столбец", conname AS "Наименование"
FROM    pg_constraint WHERE conrelid = 'table_u_1_01_С'::regclass::oid
UNION
SELECT  9 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", refobjsubid AS "Столбец", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = 'table_u_1_01_С'::regclass::oid
UNION
SELECT  10 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", objsubid AS "Столбец", description AS "Наименование"
FROM    pg_description WHERE objoid = 'table_u_1_01_С'::regclass::oid
UNION
SELECT  14 AS "Порядок", maclabel AS "MAC", 'Наследование' AS "Тип", 0 AS "Столбец", 'table_u_1_01' AS "Наименование"
FROM    pg_inherits WHERE inhrelid = 'table_u_1_01_С'::regclass::oid
ORDER BY 1, 4, 5;
--
-- Проверяем все объекты, относящиеся к виду view_u_1_01
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Представление' AS "Тип", 0 AS "Столбец", relname AS "Наименование"
FROM    pg_class WHERE relname = 'view_u_1_01'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Столбец представления' AS "Тип", attnum AS "Столбец", attname AS "Наименование"
FROM    pg_attribute WHERE attrelid = 'view_u_1_01'::regclass::oid
UNION
SELECT  6 AS "Порядок", maclabel AS "MAC", 'Тип представления' AS "Тип", 0 AS "Столбец", typname AS "Наименование"
FROM    pg_type WHERE oid = (select reltype from pg_class where relname = 'view_u_1_01')
UNION
SELECT  7 AS "Порядок", maclabel AS "MAC", 'Тип представления' AS "Тип", 0 AS "Столбец", typname AS "Наименование"
FROM    pg_type WHERE oid = (select reloftype from pg_class where relname = 'view_u_1_01')
UNION
SELECT  9 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", refobjsubid AS "Столбец", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = 'view_u_1_01'::regclass::oid
UNION
SELECT  10 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", objsubid AS "Столбец", description AS "Наименование"
FROM    pg_description WHERE objoid = 'view_u_1_01'::regclass::oid
UNION
SELECT  11 AS "Порядок", maclabel AS "MAC", 'Статистика' AS "Тип", staattnum AS "Столбец", 'view_u_1_01' AS "Наименование"
FROM    pg_statistic WHERE starelid = 'view_u_1_01'::regclass::oid
ORDER BY 1, 4, 5;
--
-- Проверяем все объекты, относящиеся к материализованному виду matview_u_1_01_2
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Материализованное представление' AS "Тип", 0 AS "Столбец", relname AS "Наименование"
FROM    pg_class WHERE relname = 'matview_u_1_01_2'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Столбец представления' AS "Тип", attnum AS "Столбец", attname AS "Наименование"
FROM    pg_attribute WHERE attrelid = 'matview_u_1_01_2'::regclass::oid
UNION
SELECT  6 AS "Порядок", maclabel AS "MAC", 'Тип представления' AS "Тип", 0 AS "Столбец", typname AS "Наименование"
FROM    pg_type WHERE oid = (select reltype from pg_class where relname = 'matview_u_1_01_2')
UNION
SELECT  7 AS "Порядок", maclabel AS "MAC", 'Тип представления' AS "Тип", 0 AS "Столбец", typname AS "Наименование"
FROM    pg_type WHERE oid = (select reloftype from pg_class where relname = 'matview_u_1_01_2')
UNION
SELECT  9 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", refobjsubid AS "Столбец", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = 'matview_u_1_01_2'::regclass::oid
UNION
SELECT  10 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", objsubid AS "Столбец", description AS "Наименование"
FROM    pg_description WHERE objoid = 'matview_u_1_01_2'::regclass::oid
UNION
SELECT  11 AS "Порядок", maclabel AS "MAC", 'Статистика' AS "Тип", staattnum AS "Столбец", 'matview_u_1_01_2' AS "Наименование"
FROM    pg_statistic WHERE starelid = 'matview_u_1_01_2'::regclass::oid
ORDER BY 1, 4, 5;
--
-- Проверяем все объекты, относящиеся к последовательности seq_u_1_01
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Последовательность' AS "Тип", 0 AS "Столбец", relname AS "Наименование"
FROM    pg_class WHERE relname = 'seq_u_1_01'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Столбец последовательности' AS "Тип", attnum AS "Столбец", attname AS "Наименование"
FROM    pg_attribute WHERE attrelid = 'seq_u_1_01'::regclass::oid
UNION
SELECT  6 AS "Порядок", maclabel AS "MAC", 'Тип последовательности' AS "Тип", 0 AS "Столбец", typname AS "Наименование"
FROM    pg_type WHERE oid = (select reltype from pg_class where relname = 'seq_u_1_01')
UNION
SELECT  7 AS "Порядок", maclabel AS "MAC", 'Тип последовательности' AS "Тип", 0 AS "Столбец", typname AS "Наименование"
FROM    pg_type WHERE oid = (select reloftype from pg_class where relname = 'seq_u_1_01')
UNION
SELECT  9 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", refobjsubid AS "Столбец", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = 'seq_u_1_01'::regclass::oid
UNION
SELECT  10 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", objsubid AS "Столбец", description AS "Наименование"
FROM    pg_description WHERE objoid = 'seq_u_1_01'::regclass::oid
UNION
SELECT  11 AS "Порядок", maclabel AS "MAC", 'Статистика' AS "Тип", staattnum AS "Столбец", 'seq_u_1_01' AS "Наименование"
FROM    pg_statistic WHERE starelid = 'seq_u_1_01'::regclass::oid
ORDER BY 1, 4, 5;
--
-- Проверяем все объекты, относящиеся к типу type_u_1_01_c
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Тип ' || CAST(typtype AS varchar) AS "Тип", typname AS "Наименование"
FROM    pg_type WHERE typname = 'type_u_1_01_c'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Таблица типа' AS "Тип", relname AS "Наименование"
FROM    pg_class WHERE oid = (SELECT typrelid FROM pg_type WHERE typname = 'type_u_1_01_c')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Атрибут типа' AS "Тип", attname AS "Наименование"
FROM    pg_attribute WHERE attrelid = (SELECT typrelid FROM pg_type WHERE typname = 'type_u_1_01_c')
UNION
SELECT  3 AS "Порядок", maclabel AS "MAC", 'Элемент enum' AS "Тип", enumlabel AS "Наименование"
FROM    pg_enum WHERE enumtypid = 'type_u_1_01_c'::regtype::oid
UNION
SELECT  4 AS "Порядок", maclabel AS "MAC", 'Свойства range' AS "Тип", 'type_u_1_01_c' AS "Наименование"
FROM    pg_range WHERE rngtypid = 'type_u_1_01_c'::regtype::oid
UNION
SELECT  5 AS "Порядок", maclabel AS "MAC", 'Ограничение' AS "Тип", conname AS "Наименование"
FROM    pg_constraint WHERE contypid = 'type_u_1_01_c'::regtype::oid
UNION
SELECT  6 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = 'type_u_1_01_c'::regtype::oid
UNION
SELECT  7 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = 'type_u_1_01_c'::regtype::oid
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к типу type_u_1_01_e
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Тип ' || CAST(typtype AS varchar) AS "Тип", typname AS "Наименование"
FROM    pg_type WHERE typname = 'type_u_1_01_e'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Таблица типа' AS "Тип", relname AS "Наименование"
FROM    pg_class WHERE oid = (SELECT typrelid FROM pg_type WHERE typname = 'type_u_1_01_e')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Атрибут типа' AS "Тип", attname AS "Наименование"
FROM    pg_attribute WHERE attrelid = (SELECT typrelid FROM pg_type WHERE typname = 'type_u_1_01_e')
UNION
SELECT  3 AS "Порядок", maclabel AS "MAC", 'Элемент enum' AS "Тип", enumlabel AS "Наименование"
FROM    pg_enum WHERE enumtypid = 'type_u_1_01_e'::regtype::oid
UNION
SELECT  4 AS "Порядок", maclabel AS "MAC", 'Свойства range' AS "Тип", 'type_u_1_01_e' AS "Наименование"
FROM    pg_range WHERE rngtypid = 'type_u_1_01_e'::regtype::oid
UNION
SELECT  5 AS "Порядок", maclabel AS "MAC", 'Ограничение' AS "Тип", conname AS "Наименование"
FROM    pg_constraint WHERE contypid = 'type_u_1_01_e'::regtype::oid
UNION
SELECT  6 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = 'type_u_1_01_e'::regtype::oid
UNION
SELECT  7 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = 'type_u_1_01_e'::regtype::oid
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к типу type_u_1_01_r
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Тип ' || CAST(typtype AS varchar) AS "Тип", typname AS "Наименование"
FROM    pg_type WHERE typname = 'type_u_1_01_r'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Таблица типа' AS "Тип", relname AS "Наименование"
FROM    pg_class WHERE oid = (SELECT typrelid FROM pg_type WHERE typname = 'type_u_1_01_r')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Атрибут типа' AS "Тип", attname AS "Наименование"
FROM    pg_attribute WHERE attrelid = (SELECT typrelid FROM pg_type WHERE typname = 'type_u_1_01_r')
UNION
SELECT  3 AS "Порядок", maclabel AS "MAC", 'Элемент enum' AS "Тип", enumlabel AS "Наименование"
FROM    pg_enum WHERE enumtypid = 'type_u_1_01_r'::regtype::oid
UNION
SELECT  4 AS "Порядок", maclabel AS "MAC", 'Свойства range' AS "Тип", 'type_u_1_01_r' AS "Наименование"
FROM    pg_range WHERE rngtypid = 'type_u_1_01_r'::regtype::oid
UNION
SELECT  5 AS "Порядок", maclabel AS "MAC", 'Ограничение' AS "Тип", conname AS "Наименование"
FROM    pg_constraint WHERE contypid = 'type_u_1_01_r'::regtype::oid
UNION
SELECT  6 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = 'type_u_1_01_r'::regtype::oid
UNION
SELECT  7 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = 'type_u_1_01_r'::regtype::oid
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к домену type_u_1_01_d
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Тип ' || CAST(typtype AS varchar) AS "Тип", typname AS "Наименование"
FROM    pg_type WHERE typname = 'type_u_1_01_d'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Таблица типа' AS "Тип", relname AS "Наименование"
FROM    pg_class WHERE oid = (SELECT typrelid FROM pg_type WHERE typname = 'type_u_1_01_d')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Атрибут типа' AS "Тип", attname AS "Наименование"
FROM    pg_attribute WHERE attrelid = (SELECT typrelid FROM pg_type WHERE typname = 'type_u_1_01_d')
UNION
SELECT  3 AS "Порядок", maclabel AS "MAC", 'Элемент enum' AS "Тип", enumlabel AS "Наименование"
FROM    pg_enum WHERE enumtypid = 'type_u_1_01_d'::regtype::oid
UNION
SELECT  4 AS "Порядок", maclabel AS "MAC", 'Свойства range' AS "Тип", 'type_u_1_01_d' AS "Наименование"
FROM    pg_range WHERE rngtypid = 'type_u_1_01_d'::regtype::oid
UNION
SELECT  5 AS "Порядок", maclabel AS "MAC", 'Ограничение' AS "Тип", conname AS "Наименование"
FROM    pg_constraint WHERE contypid = 'type_u_1_01_d'::regtype::oid
UNION
SELECT  6 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = 'type_u_1_01_d'::regtype::oid
UNION
SELECT  7 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = 'type_u_1_01_d'::regtype::oid
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к функции mac2varchar
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Функция' AS "Тип", proname AS "Наименование"
FROM    pg_proc WHERE proname = 'mac2varchar'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Агрегат' AS "Тип", 'mac2varchar' AS "Наименование"
FROM    pg_aggregate WHERE aggfnoid = 'mac2varchar'::regproc::oid
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = 'mac2varchar'::regproc::oid
UNION
SELECT  3 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = 'mac2varchar'::regproc::oid
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к функции func_u_1_01
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Функция' AS "Тип", proname AS "Наименование"
FROM    pg_proc WHERE proname = 'func_u_1_01'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Агрегат' AS "Тип", 'func_u_1_01' AS "Наименование"
FROM    pg_aggregate WHERE aggfnoid = 'func_u_1_01'::regproc::oid
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = 'func_u_1_01'::regproc::oid
UNION
SELECT  3 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = 'func_u_1_01'::regproc::oid
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к триггеру trg_u_1_01
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Триггер' AS "Тип", tgname AS "Наименование"
FROM    pg_trigger WHERE tgname = 'trg_u_1_01'
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = (SELECT oid FROM pg_trigger WHERE tgname = 'trg_u_1_01')
UNION
SELECT  3 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = (SELECT oid FROM pg_trigger WHERE tgname = 'trg_u_1_01')
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к агрегату agg_u_1_01
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Функция' AS "Тип", proname AS "Наименование"
FROM    pg_proc WHERE proname = 'agg_u_1_01'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Агрегат' AS "Тип", 'agg_u_1_01' AS "Наименование"
FROM    pg_aggregate WHERE aggfnoid = 'agg_u_1_01'::regproc::oid
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = 'agg_u_1_01'::regproc::oid
UNION
SELECT  3 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = 'agg_u_1_01'::regproc::oid
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к CAST u_1_01
SELECT  0 AS "Порядок", maclabel AS "MAC", 'CAST' AS "Тип", 'CAST(maclabel AS varchar)' AS "Наименование"
FROM    pg_cast WHERE castsource = 7283 and casttarget = 1043
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = (SELECT oid FROM pg_cast WHERE castsource = 7283 and casttarget = 1043)
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = (SELECT oid FROM pg_cast WHERE castsource = 7283 and casttarget = 1043)
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к COLLATION rus_u_1_01
SELECT  0 AS "Порядок", maclabel AS "MAC", 'COLLATION' AS "Тип", collname AS "Наименование"
FROM    pg_collation WHERE collname = 'rus_u_1_01'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = (SELECT oid FROM pg_collation WHERE collname = 'rus_u_1_01')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = (SELECT oid FROM pg_collation WHERE collname = 'rus_u_1_01')
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к CONVERSION conv_u_1_01
SELECT  0 AS "Порядок", maclabel AS "MAC", 'CONVERSION' AS "Тип", conname AS "Наименование"
FROM    pg_conversion WHERE conname = 'conv_u_1_01'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = (SELECT oid FROM pg_conversion WHERE conname = 'conv_u_1_01')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = (SELECT oid FROM pg_conversion WHERE conname = 'conv_u_1_01')
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к внешнему серверу srv_u_1_01
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Внешний сервер' AS "Тип", srvname AS "Наименование"
FROM    pg_foreign_server WHERE srvname = 'srv_u_1_01'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = (SELECT oid FROM pg_foreign_server WHERE srvname = 'srv_u_1_01')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = (SELECT oid FROM pg_foreign_server WHERE srvname = 'srv_u_1_01')
UNION
SELECT  3 AS "Порядок", maclabel AS "MAC", 'User mapping' AS "Тип",
    array_to_string(umoptions, ',') AS "Наименование"
FROM    pg_user_mapping WHERE umserver = (SELECT oid FROM pg_foreign_server WHERE srvname = 'srv_u_1_01')
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к внешней таблице ftbl_u_1_01
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Внешняя таблица' AS "Тип", 0 AS "Столбец", relname AS "Наименование"
FROM    pg_class WHERE relname = 'ftbl_u_1_01'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Столбец внешней таблицы' AS "Тип", attnum AS "Столбец", attname AS "Наименование"
FROM    pg_attribute WHERE attrelid = 'ftbl_u_1_01'::regclass::oid
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Свойства внешней таблицы' AS "Тип", 0 AS "Столбец", 'ftbl_u_1_01' AS "Наименование"
FROM    pg_foreign_table WHERE ftrelid = 'ftbl_u_1_01'::regclass::oid
UNION
SELECT  6 AS "Порядок", maclabel AS "MAC", 'Тип внешней таблицы' AS "Тип", 0 AS "Столбец", typname AS "Наименование"
FROM    pg_type WHERE oid = (SELECT reltype FROM pg_class WHERE relname = 'ftbl_u_1_01')
UNION
SELECT  7 AS "Порядок", maclabel AS "MAC", 'Тип внешней таблицы' AS "Тип", 0 AS "Столбец", typname AS "Наименование"
FROM    pg_type WHERE oid = (SELECT reloftype FROM pg_class WHERE relname = 'ftbl_u_1_01')
UNION
SELECT  9 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", refobjsubid AS "Столбец", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = 'ftbl_u_1_01'::regclass::oid
UNION
SELECT  10 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", objsubid AS "Столбец", description AS "Наименование"
FROM    pg_description WHERE objoid = 'ftbl_u_1_01'::regclass::oid
UNION
SELECT  11 AS "Порядок", maclabel AS "MAC", 'Статистика' AS "Тип", staattnum AS "Столбец", 'ftbl_u_1_01' AS "Наименование"
FROM    pg_statistic WHERE starelid = 'ftbl_u_1_01'::regclass::oid
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к большому объекту
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Метаданные большого объекта' AS "Тип", '200000' AS "Наименование"
FROM    pg_largeobject_metadata WHERE oid = 200000
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Данные большого объекта' AS "Тип", CAST(pageno AS varchar) AS "Наименование"
FROM    pg_largeobject WHERE loid = 200000
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = 200000
UNION
SELECT  3 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = 200000
ORDER BY 1, 4;
--
select * FROM   pg_opfamily WHERE opfname = 'opf1';
-- Проверяем все объекты, относящиеся к семейству операторов
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Семейство операторов' AS "Тип", opfname AS "Наименование"
FROM    pg_opfamily WHERE opfname = 'opf1'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = (SELECT oid FROM pg_opfamily WHERE opfname = 'opf1')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = (SELECT oid FROM pg_opfamily WHERE opfname = 'opf1')
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к классу операторов
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Класс операторов' AS "Тип", opcname AS "Наименование"
FROM pg_opclass WHERE opcname = 'opc1'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = (SELECT oid FROM pg_opclass WHERE opcname = 'opc1')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = (SELECT oid FROM pg_opclass WHERE opcname = 'opc1')
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к оператору
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Оператор' AS "Тип", oprname AS "Наименование"
FROM    pg_operator WHERE oprname = '##' and oprleft = 602 and oprright = 602
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = (SELECT oid FROM pg_operator WHERE oprname = '##' and oprleft = 602 and oprright = 602)
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = (SELECT oid FROM pg_operator WHERE oprname = '##' and oprleft = 602 and oprright = 602)
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к TS parser
SELECT  0 AS "Порядок", maclabel AS "MAC", 'TS parser' AS "Тип", prsname AS "Наименование"
FROM    pg_ts_parser WHERE prsname = 'tsp1'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = (SELECT oid FROM pg_ts_parser WHERE prsname = 'tsp1')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = (SELECT oid FROM pg_ts_parser WHERE prsname = 'tsp1')
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к TS template
SELECT  0 AS "Порядок", maclabel AS "MAC", 'TS template' AS "Тип", tmplname AS "Наименование"
FROM    pg_ts_template WHERE tmplname = 'tst1'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = (SELECT oid FROM pg_ts_template WHERE tmplname = 'tst1')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = (SELECT oid FROM pg_ts_template WHERE tmplname = 'tst1')
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к TS dictionary
SELECT  0 AS "Порядок", maclabel AS "MAC", 'TS dictionary' AS "Тип", dictname AS "Наименование"
FROM    pg_ts_dict WHERE dictname = 'tsd_u_1_01'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = (SELECT oid FROM pg_ts_dict WHERE dictname = 'tsd_u_1_01')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = (SELECT oid FROM pg_ts_dict WHERE dictname = 'tsd_u_1_01')
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к TS configuration
SELECT  0 AS "Порядок", maclabel AS "MAC", 'TS configuration' AS "Тип", cfgname AS "Наименование"
FROM    pg_ts_config WHERE cfgname = 'tsc_u_1_01'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'TS configuration map' AS "Тип",
    (SELECT dictname FROM pg_ts_dict WHERE oid = mapdict) AS "Наименование"
FROM    pg_ts_config_map WHERE mapcfg = (SELECT oid FROM pg_ts_config WHERE cfgname = 'tsc_u_1_01')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = (SELECT oid FROM pg_ts_config WHERE cfgname = 'tsc_u_1_01')
UNION
SELECT  3 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = (SELECT oid FROM pg_ts_config WHERE cfgname = 'tsc_u_1_01')
ORDER BY 1, 4;
--
-- Проверяем все объекты, относящиеся к событийному триггеру ev_trigger
SELECT  0 AS "Порядок", maclabel AS "MAC", 'Событийный триггер' AS "Тип", evtname AS "Наименование"
FROM    pg_event_trigger WHERE evtname = 'ev_trigger'
UNION
SELECT  1 AS "Порядок", maclabel AS "MAC", 'Зависимость' AS "Тип", CAST(deptype AS varchar) AS "Наименование"
FROM    pg_depend WHERE objid = (SELECT oid FROM pg_event_trigger WHERE evtname = 'ev_trigger')
UNION
SELECT  2 AS "Порядок", maclabel AS "MAC", 'Описание' AS "Тип", description AS "Наименование"
FROM    pg_description WHERE objoid = (SELECT oid FROM pg_event_trigger WHERE evtname = 'ev_trigger')
ORDER BY 1, 4;
--
