-- Создание объектов для тестов
--
\i 'support/adminpack.sql'
--
-- Подключаемся как суперпользователь
\connect - postgres
SET ac_session_maclabel = '{0,1}';
--
-- Добавляем информацию по глобальным объектам
ALTER DATABASE mtest SET search_path TO public, pg_catalog;
COMMENT ON DATABASE mtest IS 'База данных для тестирования MAC';
CREATE TABLESPACE dbspace LOCATION '/var/lib/postgresql/9.6/testspace';
COMMENT ON TABLESPACE dbspace IS 'Tablespace для тестирования MAC';
CREATE ROLE group1 WITH NOLOGIN;
ALTER GROUP group1 ADD USER u_1_01;
ALTER ROLE group1 SET search_path TO public;
ALTER DEFAULT PRIVILEGES FOR ROLE group1 GRANT ALL ON TABLES TO PUBLIC;
ALTER DEFAULT PRIVILEGES FOR ROLE group1 IN SCHEMA public GRANT ALL ON TABLES TO PUBLIC;
ALTER DEFAULT PRIVILEGES FOR ROLE group1 IN SCHEMA "Схема_1_01" GRANT ALL ON TABLES TO PUBLIC;
COMMENT ON ROLE group1 IS 'Группа для теста';
COMMENT ON SCHEMA public IS 'Общая схема';
CREATE FOREIGN DATA WRAPPER dummy;
COMMENT ON FOREIGN DATA WRAPPER dummy IS 'Пустая обертка dummy';
GRANT ALL ON FOREIGN DATA WRAPPER dummy TO public;
CREATE OPERATOR FAMILY opf1 USING btree;
ALTER OPERATOR FAMILY opf1 USING btree ADD OPERATOR 1 <(int4,int2);
COMMENT ON OPERATOR FAMILY opf1 USING btree IS 'Семейство операторов opf1';
CREATE OPERATOR CLASS opc1 FOR TYPE maclabel USING btree AS
OPERATOR 3 =;
COMMENT ON OPERATOR CLASS opc1 USING btree IS 'Класс операторов opc1';
CREATE TEXT SEARCH PARSER tsp1 (
    START = prsd_start,
    GETTOKEN = prsd_nexttoken,
    END = prsd_end,
    LEXTYPES = prsd_lextype,
    HEADLINE = prsd_headline
);
COMMENT ON TEXT SEARCH PARSER tsp1 IS 'TS parser tsp1';
CREATE TEXT SEARCH TEMPLATE tst1 (
    INIT = dsimple_init,
    LEXIZE = dsimple_lexize
);
COMMENT ON TEXT SEARCH TEMPLATE tst1 IS 'TS шаблон tst1';
CREATE FUNCTION mac2varchar(l maclabel) RETURNS varchar LANGUAGE plpgsql AS $$
BEGIN
    RETURN text(l);
END;$$ SECURITY INVOKER;
CREATE CAST (maclabel AS varchar) WITH FUNCTION mac2varchar(maclabel) AS ASSIGNMENT;
COMMENT ON CAST (maclabel AS varchar) IS 'CAST 01';
CREATE FUNCTION ddl_trigger_func() RETURNS event_trigger LANGUAGE plpgsql AS $$
BEGIN
/*    RAISE NOTICE 'ddl_trigger_func: % %', tg_event, tg_tag;*/
END;$$;
CREATE EVENT TRIGGER ev_trigger on ddl_command_start
EXECUTE PROCEDURE ddl_trigger_func();
COMMENT ON EVENT TRIGGER ev_trigger IS 'EVENT TRIGGER ev_trigger';
--
--
-- Подключаемся как пользователь c меткой {1,1}
\connect - u_1_01
--
-- Изменяем права по умолчанию для своей роли
ALTER DEFAULT PRIVILEGES FOR ROLE u_1_01 GRANT ALL ON TABLES TO PUBLIC;
--
-- Создаем таблицу с параметрами по умолчанию
-- И создаем набор ограничений и описаний для нее
CREATE TABLE table_u_1_01 (
ID  integer CONSTRAINT table_u_1_01_PK PRIMARY KEY,
data    TEXT NOT NULL CHECK (data <> ''),
FL  float DEFAULT 12
);
--
-- Создаем уникальный индекс
CREATE UNIQUE INDEX test_u_1_01_data ON table_u_1_01 (data);
--
-- Комментируем
COMMENT ON TABLE table_u_1_01 IS 'Таблица u_1_01';
COMMENT ON COLUMN table_u_1_01.ID IS 'Столбец table_u_1_01.ID';
COMMENT ON CONSTRAINT table_u_1_01_PK ON table_u_1_01 IS 'PK для Таблица u_1_01';
COMMENT ON INDEX table_u_1_01_PK IS 'Индекс для PK для Таблица u_1_01';
--
-- Определяем статистику
INSERT INTO table_u_1_01(ID, data, FL) VALUES(1, 'test', 2.1);
ANALYZE table_u_1_01;
--
-- Создаем правило для операции UPDATE
CREATE RULE table_u_1_01_RU AS ON UPDATE TO table_u_1_01
        WHERE NEW.ID > 28 DO ALSO
    UPDATE table_u_1_01 SET FL = FL + NEW.ID
    WHERE ID = NEW.ID-1;
--
-- Ожидается, что метка таблицы соответствует метке пользователя
-- записи таблицы незащищены (меток не имеют)
-- признак MAC CCR установлен
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname = 'table_u_1_01';
--
-- Создаем таблицу порожденную от table_u_1_01
CREATE TABLE table_u_1_01_С (ss integer) INHERITS (table_u_1_01);
--
-- Ожидается, что метка таблицы соответствует метке пользователя
-- записи таблицы незащищены (меток не имеют)
-- признак MAC CCR установлен
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname = 'table_u_1_01_С';
--
-- Проверяем режим создания таблицы, записи которой защищены мандатной меткой
CREATE TABLE test_u_1_01_2 (data TEXT) WITH (MACS=TRUE);
--
-- Ожидается, что метка созданной таблицы соответствует метке пользователя
-- записи таблицы защищены (имеют метки)
-- признак MAC CCR установлен
SELECT maclabel, relname, relhasmacs, relmacccr
FROM pg_class WHERE relname = 'test_u_1_01_2';
--
-- Проверяем режим создания таблицы с явным заданием отсутствия мандатных меток у записей
CREATE TABLE test_u_1_01_3 (data TEXT) WITH (MACS=FALSE);
--
-- Ожидается, что метка созданной таблицы соответствует метке пользователя {1,1}
-- и записи таблицы незащищены (меток не имеют)
-- признак MAC CCR установлен
SELECT maclabel, relname, relhasmacs, relmacccr
FROM pg_class WHERE relname = 'test_u_1_01_3';
--
-- Проверяем режим создания вида
CREATE VIEW view_u_1_01 AS SELECT * FROM table_u_1_01;
COMMENT ON VIEW view_u_1_01 IS 'Представление u_1_01';
COMMENT ON COLUMN view_u_1_01.ID IS 'Столбец view_u_1_01.ID';
--
-- Ожидается, что метка созданного вида соответствует метке пользователя {1,1}
-- и записи вида незащищены (меток не имеют)
-- признак MAC CCR установлен
SELECT maclabel, relname, relhasmacs, relmacccr
FROM pg_class WHERE relname = 'view_u_1_01';
--
-- Проверяем режим создания материализованного вида
CREATE MATERIALIZED VIEW matview_u_1_01_2 AS SELECT * FROM test_u_1_01_2;
COMMENT ON MATERIALIZED VIEW matview_u_1_01_2 IS 'МатПредставление u_1_01_2';
COMMENT ON COLUMN matview_u_1_01_2.data IS 'Столбец matview_u_1_01_2.data';
--
-- Ожидается, что метка созданного материализованного вида соответствует
-- метке пользователя {1,1} и записи вида незащищены (меток не имеют)
-- признак MAC CCR установлен
SELECT maclabel, relname, relhasmacs, relmacccr
FROM pg_class WHERE relname = 'matview_u_1_01_2';
--
-- Проверяем режим создания последовательности
CREATE SEQUENCE seq_u_1_01
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
COMMENT ON SEQUENCE seq_u_1_01 IS 'Последовательность u_1_01';
--
-- Ожидается, что метка созданной последовательности соответствует метке пользователя {1,1}
-- и записи последовательности незащищены (меток не имеют)
-- признак MAC CCR установлен
SELECT maclabel, relname, relhasmacs, relmacccr
FROM pg_class WHERE relname = 'seq_u_1_01';
--
--
-- Проверяем режим создания типа
CREATE TYPE type_u_1_01_c AS (f1 integer, f2 integer);
COMMENT ON TYPE type_u_1_01_c IS 'Composite тип u_1_01';
--
CREATE TYPE type_u_1_01_e AS ENUM ('Первый', 'Второй');
COMMENT ON TYPE type_u_1_01_e IS 'Enum тип u_1_01';
--
CREATE TYPE type_u_1_01_r AS RANGE (subtype = float8, subtype_diff = float8mi);
COMMENT ON TYPE type_u_1_01_r IS 'Range тип u_1_01';
--
CREATE DOMAIN type_u_1_01_d AS TEXT
CHECK(
VALUE ~ '^\d{5}$'
);
COMMENT ON DOMAIN type_u_1_01_d IS 'Domain тип u_1_01';
--
-- Ожидается, что метки созданных типов соответствует метке пользователя {1,1}
SELECT maclabel, typname, typtype
FROM pg_type WHERE typname LIKE 'type_u_1_01_%';
--
--
-- Проверяем режим создания функции
CREATE FUNCTION func_u_1_01() RETURNS trigger LANGUAGE plpgsql AS $$
BEGIN
    NEW.data = NEW.data || '-Функ1';
    RETURN NEW;
END;$$ SECURITY INVOKER;
COMMENT ON FUNCTION func_u_1_01() IS 'Функция u_1_01';
--
-- Ожидается, что метка созданной функции соответствует метке пользователя {1,1}
SELECT maclabel, proname
FROM pg_proc WHERE proname = 'func_u_1_01';
--
--
-- Проверяем режим создания триггера
CREATE TRIGGER trg_u_1_01
  BEFORE UPDATE
  ON table_u_1_01
  FOR EACH ROW
  EXECUTE PROCEDURE func_u_1_01();
COMMENT ON TRIGGER trg_u_1_01 ON table_u_1_01 IS 'Триггер u_1_01';
--
--
-- Ожидается, что метка созданного триггера соответствует метке пользователя {1,1}
SELECT maclabel, tgname
FROM pg_trigger WHERE tgname = 'trg_u_1_01';
--
--
-- Проверяем режим создания агрегата
CREATE AGGREGATE agg_u_1_01(float8)
(
  sfunc = float8_accum,
  stype = float8[],
  finalfunc = float8_avg,
  initcond = '{0,0,0}'
);
COMMENT ON AGGREGATE agg_u_1_01(float8) IS 'Агрегат u_1_01';
--
-- Ожидается, что метка созданной функции соответствует метке пользователя {1,1}
SELECT maclabel, proname
FROM pg_proc WHERE proname = 'agg_u_1_01';
--
-- Проверяем режим создания COLLATION
CREATE COLLATION rus_u_1_01 (LOCALE = 'ru_RU.utf8');
COMMENT ON COLLATION rus_u_1_01 IS 'COLLATION rus_u_1_01';
--
-- Ожидается, что метка созданного COLLATION соответствует метке пользователя {1,1}
SELECT maclabel, collname
FROM pg_collation WHERE collname = 'rus_u_1_01';
--
--
-- Проверяем режим создания CONVERSION
CREATE CONVERSION conv_u_1_01 FOR 'UTF8' TO 'KOI8-R' FROM utf8_to_koi8r;
COMMENT ON CONVERSION conv_u_1_01 IS 'CONVERSION conv_u_1_01';
--
-- Ожидается, что метка созданного CONVERSION соответствует метке пользователя {1,1}
SELECT maclabel, conname
FROM pg_conversion WHERE conname = 'conv_u_1_01';
--
--
-- Проверяем режим создания внешнего сервера
CREATE SERVER srv_u_1_01 FOREIGN DATA WRAPPER dummy;
COMMENT ON SERVER srv_u_1_01 IS 'Сервер srv_u_1_01';
--
-- Ожидается, что метка созданного внешнего сервера соответствует метке пользователя {1,1}
SELECT maclabel, srvname
FROM pg_foreign_server WHERE srvname = 'srv_u_1_01';
--
--
-- Проверяем режим создания usermapping
CREATE USER MAPPING FOR GROUP1 SERVER srv_u_1_01 OPTIONS (user 'group1', password 'secret');
CREATE USER MAPPING FOR PUBLIC SERVER srv_u_1_01 OPTIONS (user 'guest', password 'guest');
--
--
-- Проверяем режим создания внешней таблицы
CREATE FOREIGN TABLE ftbl_u_1_01
(
id integer
)
SERVER srv_u_1_01;
COMMENT ON FOREIGN TABLE ftbl_u_1_01 IS 'Внешняя таблица u_1_01';
--
-- Ожидается, что метка таблицы соответствует метке пользователя
-- записи таблицы незащищены (меток не имеют)
-- признак MAC CCR установлен
SELECT  maclabel, relmacccr, relname, relhasmacs
FROM    pg_class WHERE relname = 'ftbl_u_1_01';
--
--
-- Проверяем режим создания больших объектов
-- Создаем тестовую таблицу
CREATE TABLE blob_u_1_01 (loid oid, fd integer);
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Создаем объект
INSERT INTO blob_u_1_01 (loid) SELECT lo_create(200000);
--
-- Начинаем транзакцию
BEGIN;
--
-- Открываем объект на чтение и запись
--   INV_READ  = 0x40000
--   INV_WRITE = 0x20000
UPDATE blob_u_1_01 SET fd = lo_open(loid, CAST(x'20000' | x'40000' AS integer));
--
-- Проверка записи, ожидается успех
SELECT lowrite(fd, '
Whose woods these are I think I know,
His house is in the village though.
He will not see me stopping here,
To watch his woods fill up with snow.

My little horse must think it queer,
To stop without a farmhouse near,
Between the woods and frozen lake,
The darkest evening of the year.

He gives his harness bells a shake,
To ask if there is some mistake.
The only other sound''s the sweep,
Of easy wind and downy flake.

The woods are lovely, dark and deep,
But I have promises to keep,
And miles to go before I sleep,
And miles to go before I sleep.

         -- Robert Frost
') FROM blob_u_1_01;
--
-- Закрываем объект
-- Ожидается 0 - отсутствие ошибок, отрицательное значение - ошибка
SELECT lo_close(fd) FROM blob_u_1_01;
--
-- Завершаем транзакцию
END;
--
COMMENT ON LARGE OBJECT 200000 IS 'Большой объект u_1_01';
--
-- Проверяем атрибуты объекта
-- Ожидается, что объект приобрел метку пользователя {1,1}
SELECT maclabel, oid FROM pg_largeobject_metadata;
--
--
-- Проверяем режим создания операторов
CREATE OPERATOR ## (
 leftarg = path,
 rightarg = path,
 procedure = path_inter,
 commutator = ##
);
COMMENT ON OPERATOR ##(path,path) IS 'Оператор u_1_01';
--
-- Проверяем атрибуты объекта
-- Ожидается, что объект приобрел метку пользователя {1,1}
SELECT maclabel, oprname, oprleft, oprright
FROM pg_operator WHERE oprname = '##' and oprleft = 602 and oprright = 602;
--
--
-- Проверяем режим создания объектов текстового поиска
CREATE TEXT SEARCH DICTIONARY tsd_u_1_01 (
    TEMPLATE = tst1
);
COMMENT ON TEXT SEARCH DICTIONARY tsd_u_1_01 IS 'TS словарь u_1_01';
--
-- Проверяем атрибуты объекта
-- Ожидается, что объект приобрел метку пользователя {1,1}
SELECT maclabel, dictname
FROM pg_ts_dict WHERE dictname = 'tsd_u_1_01';
--
CREATE TEXT SEARCH CONFIGURATION tsc_u_1_01 (
    COPY = simple
);
COMMENT ON TEXT SEARCH CONFIGURATION tsc_u_1_01 IS 'TS конфигурация u_1_01';
--
-- Проверяем атрибуты объекта
-- Ожидается, что объект приобрел метку пользователя {1,1}
SELECT maclabel, cfgname
FROM pg_ts_config WHERE cfgname = 'tsc_u_1_01';
--
