--
-- Регистрация вспомогательных функций анализа бинарных файлов
--
\i 'support/init-plpythonu.sql'
--
-- Создаем функцию проверки строки в бинарном файле БД
-- Параметры:
--  relation -- имя отношения
--  pattern -- строка для поиска
CREATE OR REPLACE FUNCTION check_pattern_in_db_file(relation TEXT, pattern TEXT) RETURNS BOOLEAN
AS
$$
  from subprocess import check_call, CalledProcessError
  from os import devnull

  DEVNULL = open(devnull, mode='w')

  # Получаем путь до кластера
  try:
    data = plpy.execute('SELECT setting FROM pg_settings WHERE name = \'data_directory\';')
  except plpy.SPIError as e:
    plpy.error(e)
  data_directory = data[0]['setting']
  # Получаем путь до файла в ФС отношения
  try:
    data = plpy.execute('SELECT pg_relation_filepath(\'%s\'::regclass) AS filepath' % relation)
  except plpy.SPIError as e:
    plpy.error(e)
  filepath = data[0]['filepath']
  fullpath = '%s/%s' % (data_directory, filepath)
  # Запускаем fgrep
  found = True
  try:
    check_call(['fgrep', pattern, fullpath], stdout=DEVNULL, stderr=DEVNULL)
  except CalledProcessError as e:
    found = False
  return found
$$ LANGUAGE plpythonu;
--
-- Создаем функцию проверки строки в бинарном файле образа
-- Параметры:
--  pattern -- строка для поиска
CREATE OR REPLACE FUNCTION check_pattern_in_image(pattern TEXT) RETURNS BOOLEAN
AS
$$
  from subprocess import check_call, CalledProcessError
  from os import devnull

  DEVNULL = open(devnull, mode='w')

  # Получаем путь до кластера
  try:
    data = plpy.execute('SELECT setting FROM pg_settings WHERE name = \'data_directory\';')
  except plpy.SPIError as e:
    plpy.error(e)
  data_directory = data[0]['setting']
  # Формируем путь до файла-образа
  fullpath = '%s/../image' % data_directory
  # Запускаем fgrep
  found = True
  try:
    check_call(['fgrep', pattern, fullpath], stdout=DEVNULL, stderr=DEVNULL)
  except CalledProcessError as e:
    found = False
  return found
$$ LANGUAGE plpythonu;
--
-- Создаем функцию для дампа бинарного файла образа
-- Параметры:
--  relation -- имя отношения
--  pattern -- строка-шаблон поиска
CREATE OR REPLACE FUNCTION get_db_file_dump(relation TEXT, pattern TEXT) RETURNS SETOF TEXT
AS
$$
  import struct
  import io

  ASCII_CHARSET = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'
  ASCII_RETURN = '\r\n\t'
  OUTPUT_LENGHT = 80

  if '|' in pattern:
    patterns = pattern.split('|')
  else:
    patterns = [pattern]

  # Получаем путь до кластера
  try:
    data = plpy.execute('SELECT setting FROM pg_settings WHERE name = \'data_directory\';')
  except plpy.SPIError as e:
    plpy.error(e)
  data_directory = data[0]['setting']
  # Получаем путь до файла в ФС отношения
  try:
    data = plpy.execute('SELECT pg_relation_filepath(\'%s\'::regclass) AS filepath' % relation)
  except plpy.SPIError as e:
    plpy.error(e)
  filepath = data[0]['filepath']
  fullpath = '%s/%s' % (data_directory, filepath)

  try:
    fp = open(fullpath, mode='rb')
  except IOError as e:
    plpy.error(e)

  offset = 0
  more, instar = True, True

  while more:
    try:
      fp.seek(offset)
      buf = fp.read(OUTPUT_LENGHT)
      if not len(buf) == OUTPUT_LENGHT:
        more = False
    except EOFError:
      break

    if not buf:
      break

    actual_count = len(buf)
    fmt = '<' + 'B' * actual_count
    values = struct.unpack(fmt, buf)

    if not buf.translate(None, b'\x00'):
      instar = True
      offset += OUTPUT_LENGHT
      continue

    instar = False

    a = str()
    for ch in values:
      if not ch:
        a += '.'
      elif ch == 32:
        a += ' '
      elif chr(ch) in ASCII_CHARSET:
        a += chr(ch)
      else:
        a += '.'
    offset += OUTPUT_LENGHT

    match = re.match(pattern, a)
    if match:
      yield(match.group('target'))

  fp.close()
$$
LANGUAGE plpythonu;
--
-- Создаем функцию для дампа бинарного файла образа
-- Параметры:
--  pattern -- строка-шаблон поиска
--  rows -- количество строк вывода
CREATE OR REPLACE FUNCTION get_image_dump(pattern TEXT, rows INTEGER) RETURNS TEXT
AS
$$
  import struct
  import io
  import re

  ASCII_CHARSET = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'
  ASCII_RETURN = '\r\n\t'
  OUTPUT_LENGHT = 80

  global pattern

  # Получаем путь до кластера
  try:
    data = plpy.execute('SELECT setting FROM pg_settings WHERE name = \'data_directory\';')
  except plpy.SPIError as e:
    plpy.error(e)
  data_directory = data[0]['setting']

  image_path = '%s/../image' % data_directory

  try:
    fp = open(image_path, mode='rb')
  except IOError as e:
    plpy.error(e)

  offset = 0
  more, instar = True, True
  result = []

  while more:
    try:
      fp.seek(offset)
      buf = fp.read(OUTPUT_LENGHT)
      if not len(buf) == OUTPUT_LENGHT:
        more = False
    except EOFError:
      break

    if not buf:
      break

    actual_count = len(buf)
    fmt = '<' + 'B' * actual_count
    values = struct.unpack(fmt, buf)

    if not buf.translate(None, b'\x00'):
      instar = True
      offset += OUTPUT_LENGHT
      continue

    instar = False

    a = str()
    for ch in values:
      if not ch:
        a += '.'
      elif ch == 32:
        a += ' '
      elif chr(ch) in ASCII_CHARSET:
        a += chr(ch)
      else:
        a += '.'
    offset += OUTPUT_LENGHT

    m = re.search(pattern, a)
    if m:
      result += m.re.findall(a)

  fp.close()
  row = '\n'.join(sorted(result)[:rows])
  return row
$$
LANGUAGE plpythonu;
--
-- Завершено
--