--
-- Инициализация процедуры конфигурирования сервера
--
-- В базе должен быть установлен модуль adminpack
\i 'support/adminpack.sql'
--
-- Подключаемся как администратор кластера
\connect mtest postgres

--
--  Заменяет значение конфигурационного параметра
--  Функция не добавляет в файл ранее отсутствовавшие параметры и не раскомментировывает существующие.
--  Параметр должен быть раскомментирован заранее. Также подразумевается,
--  что он может быть только типа bool и задаваться строкой true или false.

CREATE OR REPLACE FUNCTION ac_change_param( param_name text, new_value boolean )
RETURNS boolean
LANGUAGE plpgsql
AS
$function$
DECLARE
    txt_old_value   varchar;
    txt_new_value   varchar;
    result      boolean;
BEGIN
    -- Получение старого значения параметра
    SELECT setting FROM pg_settings WHERE name = param_name INTO txt_old_value;

    -- Преобразование значения в текстовую форму
    CASE txt_old_value
        WHEN 'on', 'true', 't' THEN
            txt_old_value = 'true';
        WHEN 'off', 'false', 'f' THEN
            txt_old_value = 'false';
        ELSE
            -- Параметр не найден или не соответствует по типу
            RETURN false;
    END CASE;

    -- Преобразование нового значения в текстовую форму
    IF new_value THEN txt_new_value = 'true'; ELSE txt_new_value = 'false'; END IF;

    -- Если старое и новое значение равны, изменение конфига не требуется
    IF txt_new_value = txt_old_value THEN
        RETURN true;
    END IF;

    -- Манипуляции с файлом конфига
    PERFORM pg_catalog.pg_file_unlink( '/etc/postgresql/9.6/setest/postgresql.conf.bak' );
    PERFORM pg_catalog.pg_file_unlink( '/etc/postgresql/9.6/setest/postgresql.conf.tmp' );

    SELECT pg_catalog.pg_file_write( '/etc/postgresql/9.6/setest/postgresql.conf.tmp',
                    pg_catalog.regexp_replace( pg_catalog.pg_file_read( '/etc/postgresql/9.6/setest/postgresql.conf',
                                    0, pg_catalog.pg_file_length( '/etc/postgresql/9.6/setest/postgresql.conf') ),
                                    param_name || E'\\s+=\\s+' || txt_old_value,
                                    param_name || ' = ' || txt_new_value, 'g' ),
                    false ) <> 0 INTO result;

    IF not result THEN
          RETURN result;
    END IF;

    SELECT pg_catalog.pg_file_rename( '/etc/postgresql/9.6/setest/postgresql.conf.tmp',
                      '/etc/postgresql/9.6/setest/postgresql.conf',
                      '/etc/postgresql/9.6/setest/postgresql.conf.bak' ) INTO result;

    IF not result THEN
          RETURN result;
    END IF;

    -- Перечитывание конфига по сигналу SIGHUP
    SELECT pg_catalog.pg_reload_conf() INTO result;

    RETURN result;
END
$function$ ;
--
--  Заменяет значение конфигурационного параметра
--  Функция не добавляет в файл ранее отсутствовавшие параметры и не раскомментировывает существующие.
--  Параметр должен быть раскомментирован заранее. Также подразумевается,
--  что он может быть только строкой.

CREATE OR REPLACE FUNCTION ac_change_param_text( param_name text, new_value text )
RETURNS boolean
LANGUAGE plpgsql
AS
$function$
DECLARE
    txt_old_value   varchar;
    result          boolean;
BEGIN
    -- Получение старого значения параметра
    SELECT setting FROM pg_settings WHERE name = param_name INTO txt_old_value;

    -- Если старое и новое значение равны, изменение конфига не требуется
    IF txt_old_value = new_value THEN
        RETURN true;
    END IF;

    -- Манипуляции с файлом конфига
    PERFORM pg_catalog.pg_file_unlink( '/etc/postgresql/9.6/setest/postgresql.conf.bak' );
    PERFORM pg_catalog.pg_file_unlink( '/etc/postgresql/9.6/setest/postgresql.conf.tmp' );

    SELECT pg_catalog.pg_file_write( '/etc/postgresql/9.6/setest/postgresql.conf.tmp',
                    pg_catalog.regexp_replace( pg_catalog.pg_file_read( '/etc/postgresql/9.6/setest/postgresql.conf',
                                    0, pg_catalog.pg_file_length( '/etc/postgresql/9.6/setest/postgresql.conf') ),
                                    param_name || E'\\s+=\\s+' ||  '''' || txt_old_value ||  '''',
                                    param_name || ' = ' ||  '''' || new_value ||  '''' ),
                    false ) <> 0 INTO result;

    IF not result THEN
          RETURN result;
    END IF;

    SELECT pg_catalog.pg_file_rename( '/etc/postgresql/9.6/setest/postgresql.conf.tmp',
                      '/etc/postgresql/9.6/setest/postgresql.conf',
                      '/etc/postgresql/9.6/setest/postgresql.conf.bak' ) INTO result;

    IF not result THEN
          RETURN result;
    END IF;

    -- Перечитывание конфига по сигналу SIGHUP
    SELECT pg_catalog.pg_reload_conf() INTO result;

    RETURN result;
END
$function$ ;
--
-- Инициализация завершена.
--
